package imcacat.imcabc;

import imcacat.epicsca4j.ca.EpicsCaException;

import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.WindowConstants;

/**
 * I am a view for displaying details about an automatic {@link Optimization}.
 * <p>
 * Subclasses should override my <code>buildMainPanel</code> method to provide
 * a more useful (i.e. not blank) interface.
 *
 * @see AutoOptimizationDetails
 * @see AutoOptimizationDetailsPresentationModel
 */
public class AutoOptimizationDetailsDialog extends JDialog {
  private static final long serialVersionUID = 1;

  protected AutoOptimizationDetailsPresentationModel model;
  protected Action doneAction;

  protected AutoOptimizationDetailsDialog()  {
    super();
  }

  public AutoOptimizationDetailsDialog(Frame owner, String title,
      boolean modal) throws EpicsCaException {
    this(owner, title, modal, new AutoOptimizationDetailsPresentationModel());
  }

  public AutoOptimizationDetailsDialog(Frame owner, String title,
      boolean modal, AutoOptimizationDetailsPresentationModel model) {
    super(owner, title, modal);
    this.model = model;
    initializeActions();
    initializeDialog();
  }

  protected void initializeActions() {
    doneAction = new AbstractAction() {
      private static final long serialVersionUID = 1;
      public void actionPerformed(ActionEvent e) {
        doneActionPerformed();
      }
    };
    doneAction.putValue(Action.NAME, "Done");
  }

  protected void initializeDialog() {
    setContentPane(buildContentPane());

    setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        doneActionPerformed();
      }
    });
  }

  protected JComponent buildContentPane() {
    JPanel panel = new JPanel(new BorderLayout());
    panel.add(buildMainPanel(), BorderLayout.CENTER);
    return panel;
  }

  /**
   * Subclasses should override me to return the contents of the main panel.
   */
  protected JComponent buildMainPanel() {
    return new JLabel("<html>Subclasses should override " +
      "<tt>buildMainPanel</tt>.</html>");
  }

  protected void doneActionPerformed() {
    dispose();
  }

  public AutoOptimizationDetailsPresentationModel getPresentationModel() {
    return model;
  }
}
