package imcacat.imcabc;

import imcacat.epicsca4j.bean.OneWayPropertyChangeMediator;
import imcacat.epicsca4j.bean.ExternallyFireablePropertyChangeBean;
import imcacat.epicsca4j.ca.EpicsCaException;
import imcacat.epicsca4j.convert.ShortToBooleanEpicsCaValueConverter;
import imcacat.epicsca4j.model.EpicsPv;

import java.beans.PropertyChangeEvent;

import com.jgoodies.binding.beans.Model;

/**
 * I am a Facade providing an easy to use interface for things related to beam
 * delivery.
 */
public abstract class BeamFacade extends Model
implements ExternallyFireablePropertyChangeBean {
  private static final long serialVersionUID = 1;

  protected static final short B_SHUTTER_STATE_OPEN = 1;
  protected static final short B_SHUTTER_STATE_CLOSED = 0;

  public static final String PROPNAME_RING_CURRENT =
    "ringCurrent";

  public static final String PROPNAME_B_SHUTTER_OPEN =
    "BShutterOpen";

  public static final String PROPNAME_ENERGY_POSITION =
    "energyPosition";
  public static final String PROPNAME_WAVELENGTH_POSITION =
    "wavelengthPosition";
  public static final String PROPNAME_ENERGY_DESIRED_POSITION =
    "energyDesiredPosition";
  public static final String PROPNAME_WAVELENGTH_DESIRED_POSITION =
    "wavelengthDesiredPosition";
  public static final String PROPNAME_ENERGY_MOVING =
    "energyMoving";

  protected EpicsPv ringCurrent;
  protected OneWayPropertyChangeMediator ringCurrentMediator;

  protected EpicsPv bShutterBi;
  protected OneWayPropertyChangeMediator bShutterBiMediator;

  protected EpicsPv bShutterBo;

  public BeamFacade()
  throws InterruptedException, EpicsCaException {
    super();
    initialize();
  }

  protected void initialize()
  throws InterruptedException, EpicsCaException {
    initializeGeneralSection();
    initializeShutterSection();
  }

  protected void initializeGeneralSection()
  throws InterruptedException, EpicsCaException {
    ringCurrent = new EpicsPv(getResources().getRingCurrentPvName());
    ringCurrentMediator = new OneWayPropertyChangeMediator(ringCurrent, this,
      EpicsPv.PROPNAME_VALUE, BeamFacade.PROPNAME_RING_CURRENT);
  }

  protected void initializeShutterSection()
  throws InterruptedException, EpicsCaException {
    bShutterBi = new EpicsPv(
      getResources().getBShutterBiPvName(), false, true,
      new ShortToBooleanEpicsCaValueConverter());
    bShutterBiMediator = new OneWayPropertyChangeMediator(bShutterBi, this,
      EpicsPv.PROPNAME_VALUE,
      BeamFacade.PROPNAME_B_SHUTTER_OPEN);

    bShutterBo = new EpicsPv(
      getResources().getBShutterBoPvName(), false, false,
      new ShortToBooleanEpicsCaValueConverter());
  }

  protected Resources getResources() {
    return Resources.getDefault();
  }

  public double getRingCurrent()
  throws InterruptedException, EpicsCaException {
    return (Double)ringCurrent.getValue();
  }

  public boolean isBShutterOpen()
  throws InterruptedException, EpicsCaException {
    return (Boolean)bShutterBi.getValue();
  }

  public void setBShutterOpen(boolean newValue)
  throws InterruptedException, EpicsCaException {
    bShutterBo.setValue(newValue);
  }

  public abstract double getEnergyDesiredPosition()
  throws InterruptedException, EpicsCaException;

  public abstract double getEnergyPosition()
  throws InterruptedException, EpicsCaException;

  public abstract double getWavelengthDesiredPosition()
  throws InterruptedException, EpicsCaException;

  public abstract double getWavelengthPosition()
  throws InterruptedException, EpicsCaException;

  public abstract boolean isEnergyMoving()
  throws InterruptedException, EpicsCaException;

  public abstract FilterRack getFilterRackModel();

  public void externallyFirePropertyChange(PropertyChangeEvent event) {
    firePropertyChange(event);
  }
}
