package imcacat.imcabc;

import java.util.HashMap;
import java.util.Map;

import imcacat.epicsca4j.bean.OneWayPropertyChangeMediator;
import imcacat.epicsca4j.ca.EpicsCaException;
import imcacat.epicsca4j.convert.DoubleToLongEpicsCaValueConverter;
import imcacat.epicsca4j.model.EpicsMotor;
import imcacat.epicsca4j.model.EpicsMotorMotionCommand;
import imcacat.epicsca4j.model.EpicsPv;
import imcacat.epicsca4j.model.EpicsSlit;

/**
 * I am a <code>BeamFacade</code> for the 17-BM beamline.
 */
public class BeamFacade17bm extends BeamFacade {
  private static final long serialVersionUID = 1;

  public static final String PROPNAME_ENERGY_MOVE_MODE =
    "energyMoveMode";
  public static final String PROPNAME_ENERGY_MOTION_COMMAND =
    "energyMotionCommand";

  public static final String PROPNAME_VERTICAL_DEFINING_SLIT_SIZE =
    "verticalDefiningSlitSize";
  public static final String PROPNAME_HORIZONTAL_DEFINING_SLIT_SIZE =
    "horizontalDefiningSlitSize";

  public static final String PROPNAME_BEAMLINE_BPM_INTENSITY =
    "beamlineBpmIntensity";
  public static final String PROPNAME_SLIT_ALIGNMENT_INTENSITY =
    "slitAlignmentIntensity";
  public static final String PROPNAME_SMALL_ION_CHAMBER_INTENSITY =
    "smallIonChamberIntensity";

  public static final String PROPNAME_FILTER_STATES =
    "filterStates";

  protected EnergyEpicsMotor17bm energyMotor;
  protected OneWayPropertyChangeMediator energyMotorMediator;

  protected EpicsSlit verticalDefiningSlit;
  protected OneWayPropertyChangeMediator verticalDefiningSlitMediator;

  protected EpicsMotor horizontalDefiningSlit;
  protected OneWayPropertyChangeMediator horizontalDefiningSlitMediator;

  protected EpicsPv beamlineBpmIntensity;
  protected OneWayPropertyChangeMediator beamlineBpmIntensityMediator;

  protected EpicsPv slitAlignmentIntensity;
  protected OneWayPropertyChangeMediator slitAlignmentIntensityMediator;

  protected EpicsPv smallIonChamberIntensity;
  protected OneWayPropertyChangeMediator smallIonChamberIntensityMediator;

  protected FilterRack filterRack;
  protected OneWayPropertyChangeMediator filterRackMediator;

  public BeamFacade17bm() throws InterruptedException, EpicsCaException {
    super();
  }

  @Override
  protected void initialize() throws InterruptedException, EpicsCaException {
    super.initialize();
    initializeEnergySection();
    initializeDefiningSlitsSection();
    initializeIntensitySection();
    initializeAttenuationSection();
  }

  protected void initializeEnergySection()
  throws InterruptedException, EpicsCaException {
    energyMotor = new EnergyEpicsMotor17bm(
      Resources.getDefault().getEnergyMotorPvName());
    Map<String,String> energyMotorMediatorPropertyMap =
      new HashMap<String,String>();
    energyMotorMediatorPropertyMap.put(
      EnergyEpicsMotor17bm.PROPNAME_POSITION, PROPNAME_ENERGY_POSITION);
    energyMotorMediatorPropertyMap.put(
      EnergyEpicsMotor17bm.PROPNAME_WAVELENGTH_POSITION,
      PROPNAME_WAVELENGTH_POSITION);
    energyMotorMediatorPropertyMap.put(
      EnergyEpicsMotor17bm.PROPNAME_MOVE_MODE,
      BeamFacade17bm.PROPNAME_ENERGY_MOVE_MODE);
    energyMotorMediatorPropertyMap.put(
      EnergyEpicsMotor17bm.PROPNAME_MOVING, PROPNAME_ENERGY_MOVING);
    energyMotorMediatorPropertyMap.put(
      EnergyEpicsMotor17bm.PROPNAME_DESIRED_POSITION,
      PROPNAME_ENERGY_DESIRED_POSITION);
    energyMotorMediatorPropertyMap.put(
      EnergyEpicsMotor17bm.PROPNAME_WAVELENGTH_DESIRED_POSITION,
      PROPNAME_WAVELENGTH_DESIRED_POSITION);
    energyMotorMediator = new OneWayPropertyChangeMediator(energyMotor, this,
      energyMotorMediatorPropertyMap);
  }

  protected void initializeDefiningSlitsSection()
  throws InterruptedException, EpicsCaException {
    verticalDefiningSlit = new EpicsSlit(
      Resources.getDefault().getDownstreamVerticalSlitPvName(),
      "\u03BCm", 3, null, null, 50.0, 200.0);
    verticalDefiningSlitMediator = new OneWayPropertyChangeMediator(
      verticalDefiningSlit, this, EpicsSlit.PROPNAME_SIZE,
        PROPNAME_VERTICAL_DEFINING_SLIT_SIZE);

    horizontalDefiningSlit = new EpicsMotor(
      Resources.getDefault().getSmallHorizontalSlitSizeMotorPvName(),
      "\u03BCm", 3, 50.0, 200.0, "size");
    horizontalDefiningSlitMediator = new OneWayPropertyChangeMediator(
        horizontalDefiningSlit, this, EpicsMotor.PROPNAME_POSITION,
          PROPNAME_HORIZONTAL_DEFINING_SLIT_SIZE);
  }

  protected void initializeIntensitySection()
  throws InterruptedException, EpicsCaException {
    beamlineBpmIntensity = new EpicsPv(
      Resources.getDefault().getBpmIntensitySumPvName(),
      false, true, new DoubleToLongEpicsCaValueConverter());
    beamlineBpmIntensityMediator = new OneWayPropertyChangeMediator(
      beamlineBpmIntensity, this, EpicsPv.PROPNAME_VALUE,
      PROPNAME_BEAMLINE_BPM_INTENSITY);

    slitAlignmentIntensity = new EpicsPv(
      Resources.getDefault().getSlitAlignmentIntensitySumPvName(),
      false, true, new DoubleToLongEpicsCaValueConverter());
    slitAlignmentIntensityMediator = new OneWayPropertyChangeMediator(
      slitAlignmentIntensity, this, EpicsPv.PROPNAME_VALUE,
      PROPNAME_SLIT_ALIGNMENT_INTENSITY);

    smallIonChamberIntensity = new EpicsPv(
      Resources.getDefault().getSmallIonChamberIntensitySumPvName(),
      false, true, new DoubleToLongEpicsCaValueConverter());
    smallIonChamberIntensityMediator = new OneWayPropertyChangeMediator(
      smallIonChamberIntensity, this, EpicsPv.PROPNAME_VALUE,
      PROPNAME_SMALL_ION_CHAMBER_INTENSITY);
  }

  protected void initializeAttenuationSection()
  throws InterruptedException, EpicsCaException {
    filterRack = new FilterRack(
      Resources.getDefault().getFilterRackBasePvName());
    filterRackMediator = new OneWayPropertyChangeMediator(filterRack, this,
      FilterRack.PROPNAME_FILTER_STATES, PROPNAME_FILTER_STATES);
  }

  public double getEnergyPosition()
  throws InterruptedException, EpicsCaException {
    return energyMotor.getPosition();
  }

  public double getWavelengthPosition()
  throws InterruptedException, EpicsCaException {
    return energyMotor.getWavelengthPosition();
  }

  public EnergyMotorMoveMode getEnergyMoveMode()
  throws InterruptedException, EpicsCaException {
    return energyMotor.getMoveMode();
  }

  public void setEnergyMoveMode(EnergyMotorMoveMode newValue)
  throws InterruptedException, EpicsCaException {
    energyMotor.setMoveMode(newValue);
  }

  public boolean isEnergyMoving()
  throws InterruptedException, EpicsCaException {
    return energyMotor.isMoving();
  }

  public double getEnergyDesiredPosition() {
    return energyMotor.getDesiredPosition();
  }

  public void setEnergyDesiredPosition(double newValue) {
    energyMotor.setDesiredPosition(newValue);
  }

  public double getWavelengthDesiredPosition()
  throws EpicsCaException {
    return energyMotor.getWavelengthDesiredPosition();
  }

  public void setWavelengthDesiredPosition(double newValue)
  throws EpicsCaException {
    energyMotor.setWavelengthDesiredPosition(newValue);
  }

  public void setEnergyMotionCommand(EpicsMotorMotionCommand newValue)
  throws InterruptedException, EpicsCaException {
    energyMotor.setMotionCommand(newValue);
  }

  public double getVerticalDefiningSlitSize()
  throws InterruptedException, EpicsCaException {
    return verticalDefiningSlit.getSize();
  }

  public double getHorizontalDefiningSlitSize()
  throws InterruptedException, EpicsCaException {
    return horizontalDefiningSlit.getPosition();
  }

  public long getBeamlineBpmIntensity()
  throws InterruptedException, EpicsCaException {
    return (Long)beamlineBpmIntensity.getValue();
  }

  public long getSlitAlignmentIntensity()
  throws InterruptedException, EpicsCaException {
    return (Long)slitAlignmentIntensity.getValue();
  }

  public long getSmallIonChamberIntensity()
  throws InterruptedException, EpicsCaException {
    return (Long)smallIonChamberIntensity.getValue();
  }

  public EpicsSlit getVerticalDefiningSlitModel() {
    return verticalDefiningSlit;
  }

  public EpicsMotor getHorizontalDefiningSlitModel() {
    return horizontalDefiningSlit;
  }

  public FilterRackFilterState[] getFilterStates()
  throws InterruptedException, EpicsCaException {
    return filterRack.getFilterStates();
  }

  @Override
  public FilterRack getFilterRackModel() {
    return filterRack;
  }
}
