package imcacat.imcabc;

import imcacat.imcabc.util.TestUtilities;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.concurrent.CountDownLatch;

import org.junit.BeforeClass;
import org.junit.Test;

import static org.junit.Assert.assertEquals;

/**
 * I test {@link BeamFacade17bm}.
 */
public class BeamFacade17bmTest {
  protected static final String SET_RING_CURRENT_PV_NAME = "S:SRcurrentAI";

  @BeforeClass
  public static void oneTimeSetUp() {
    assertEquals("Beamline must be " + Beamline.BM17 + " for these tests",
      Beamline.BM17, Resources.getDefault().getBeamline());
  }

  @Test(timeout=5000)
  public void propertyChange01() throws Exception {
    TestUtilities.caputAndWait(SET_RING_CURRENT_PV_NAME, 99.0);
    BeamFacade subject = new BeamFacade17bm();
    final CountDownLatch changeSignal = new CountDownLatch(1);
    subject.addPropertyChangeListener(BeamFacade.PROPNAME_RING_CURRENT,
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent e) {
          if (100.0 < (Double)e.getNewValue())
            changeSignal.countDown();
        }
    });
    TestUtilities.caputAndWait(SET_RING_CURRENT_PV_NAME, 100.1);
    changeSignal.await();
  }

  @Test(timeout=5000)
  public void getRingCurrent01() throws Exception {
    TestUtilities.caputAndWait(SET_RING_CURRENT_PV_NAME, 99.0);
    BeamFacade subject = new BeamFacade17bm();
    assertEquals(99.0, subject.getRingCurrent(), 0.01);
  }

  @Test(timeout=5000)
  public void getRingCurrent02() throws Exception {
    TestUtilities.caputAndWait(SET_RING_CURRENT_PV_NAME, 99.0);
    BeamFacade subject = new BeamFacade17bm();
    final CountDownLatch changeSignal = new CountDownLatch(1);
    subject.addPropertyChangeListener(BeamFacade.PROPNAME_RING_CURRENT,
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent e) {
          if (100.0 < (Double)e.getNewValue())
            changeSignal.countDown();
        }
    });
    TestUtilities.caputAndWait(SET_RING_CURRENT_PV_NAME, 100.1);
    changeSignal.await();
    assertEquals(100.1, subject.getRingCurrent(), 0.01);
  }
}
