package imcacat.imcabc;

import java.awt.Color;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.text.DecimalFormat;
import java.util.EnumMap;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.text.NumberFormatter;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import imcacat.epicsca4j.ca.EpicsCaException;
import imcacat.epicsca4j.convert.ConverterFactory;
import imcacat.epicsca4j.format.DoubleDecimalFormat;
import imcacat.epicsca4j.gui.OpenMoveMotorDialogAction;
import imcacat.epicsca4j.gui.OpenMoveSlitSizeDialogAction;
import imcacat.epicsca4j.model.EpicsMotorMotionCommand;
import imcacat.epicsca4j.util.UiUtilities;
import imcacat.imcabc.util.PhysicsUtilities;

/**
 * I am a <code>BeamPanel</code> for the 17-BM beamline.
 */
public class BeamPanel17bm extends BeamPanel<BeamPresentationModel17bm> {
  private static final long serialVersionUID = 1;

  public BeamPanel17bm() throws InterruptedException, EpicsCaException {
    this(new BeamPresentationModel17bm());
  }

  public BeamPanel17bm(BeamPresentationModel17bm model)
      throws InterruptedException, EpicsCaException {
    super(model);
  }

  protected JComponent buildLeftPanel() {
    FormLayout layout = new FormLayout(
      // Columns
      "right:max(85dlu;pref), 4dlu, right:40dlu, 4dlu, pref, " +
        "4dlu, max(40dlu;pref)",
      // General section
      "pref, 4dlu, pref, 4dlu, " +
      // Shutter section
      "pref, 4dlu, pref, 4dlu, " +
      // Energy section
      "pref, 4dlu, pref, 4dlu, pref, 2dlu, pref, 2dlu, pref, 2dlu, pref, " +
        "4dlu, " +
      // Beam defining slits section
      "pref, 4dlu, pref, 2dlu, pref, 4dlu, " +
      // Intensity monitors section
      "pref, 4dlu, pref, 2dlu, pref, 2dlu, pref, 4dlu, " +
      // Attenuation section
      "pref, 4dlu, pref"
    );
    layout.setRowGroups(new int[][] {
      // separators
      {1, 5, 9, 21, 27, 35},
      // fields
      {3, 7, 11, 13, 15, 17, 19, 23, 25, 29, 31, 33, 37}
    });
    PanelBuilder builder = new PanelBuilder(layout);
    builder.border(Borders.DIALOG);
    CellConstraints cc = new CellConstraints();

    builder.addSeparator("General", cc.xyw(1, 1, 7));

    builder.addLabel("Ring current:", cc.xy(1, 3));
    JLabel ringCurrentValueLabel = ComponentFactory.createLabel(
      model.getModel(BeamFacade.PROPNAME_RING_CURRENT),
      new DecimalFormat("##0.0"));
    builder.add(ringCurrentValueLabel, cc.xy(3, 3));
    builder.addLabel("mA", cc.xy(5, 3));

    builder.addSeparator("Shutter", cc.xyw(1, 5, 7));

    builder.addLabel("B shutter state:", cc.xy(1, 7));
    JLabel bShutterStateValueLabel = ComponentFactory.createBooleanLabel(
      model.getModel(BeamFacade.PROPNAME_B_SHUTTER_OPEN), "Open", "Closed",
      Color.RED, Color.BLACK);
    builder.add(bShutterStateValueLabel, cc.xy(3, 7));
    builder.add(new JButton(new TwoStateValueModelButtonAction("Open",
      "Close", true, false, false, true,
      model.getModel(BeamFacade.PROPNAME_B_SHUTTER_OPEN))),
      cc.xy(7, 7));

    builder.addSeparator("Energy", cc.xyw(1, 9, 7));

    builder.addLabel("Move mode:", cc.xy(1, 11));
    Map<EnergyMotorMoveMode,String> moveModeToStringMap =
      new EnumMap<EnergyMotorMoveMode,String>(EnergyMotorMoveMode.class);
    moveModeToStringMap.put(EnergyMotorMoveMode.FINE, "Fine");
    moveModeToStringMap.put(EnergyMotorMoveMode.COARSE, "Coarse");
    moveModeToStringMap.put(EnergyMotorMoveMode.CUSTOM, "Custom");
    JLabel moveModeValueLabel = ComponentFactory.createLabel(
      ConverterFactory.createEnumToStringConverter(
      model.getModel(BeamFacade17bm.PROPNAME_ENERGY_MOVE_MODE),
      moveModeToStringMap));
    builder.add(moveModeValueLabel, cc.xy(3, 11));
    builder.add(new JButton(model.getSetEnergyMotorMoveModeAction()),
      cc.xy(7, 11));

    builder.addLabel("Energy:", cc.xy(1, 13));
    JLabel energyValueLabel = ComponentFactory.createLabel(
      model.getModel(BeamFacade17bm.PROPNAME_ENERGY_POSITION),
      new DecimalFormat("####0.000"));
    builder.add(energyValueLabel, cc.xy(3, 13));
    builder.addLabel("eV", cc.xy(5, 13));

    builder.addLabel("Wavelength:", cc.xy(1, 15));
    JLabel wavelengthValueLabel = ComponentFactory.createLabel(
      model.getModel(BeamFacade17bm.PROPNAME_WAVELENGTH_POSITION),
      new DecimalFormat("0.0000000"));
    builder.add(wavelengthValueLabel, cc.xy(3, 15));
    builder.addLabel("\u00C5", cc.xy(5, 15));

    JLabel movingLabel = ComponentFactory.createBooleanLabel(
      model.getModel(BeamFacade17bm.PROPNAME_ENERGY_MOVING), "Moving...", "",
      Color.RED, Color.BLACK);
    builder.add(movingLabel, cc.xywh(7, 13, 1, 3, "default, center"));

    builder.addLabel("Desired Energy:", cc.xy(1, 17));
    NumberFormatter energyTextFieldFormatter = new NumberFormatter(
      new DoubleDecimalFormat("####0.000"));
    energyTextFieldFormatter.setMinimum(new Double(
      Resources.getDefault().getEnergyLowLimit()));
    energyTextFieldFormatter.setMaximum(new Double(
      Resources.getDefault().getEnergyHighLimit()));
    JTextField energyTextField = ComponentFactory.createFormattedTextField(
      model.getModel(BeamFacade17bm.PROPNAME_ENERGY_DESIRED_POSITION),
      energyTextFieldFormatter);
    energyTextField.setHorizontalAlignment(JTextField.RIGHT);
    builder.add(energyTextField, cc.xy(3, 17, "fill, default"));
    builder.addLabel("eV", cc.xy(5, 17));

    builder.addLabel("Desired Wavelength:", cc.xy(1, 19));
    NumberFormatter wavelengthTextFieldFormatter = new NumberFormatter(
      new DoubleDecimalFormat("0.0000000"));
    wavelengthTextFieldFormatter.setMinimum(new Double(
      PhysicsUtilities.convertToWavelength(
        Resources.getDefault().getEnergyHighLimit())));
    wavelengthTextFieldFormatter.setMaximum(new Double(
      PhysicsUtilities.convertToWavelength(
        Resources.getDefault().getEnergyLowLimit())));
    JTextField wavelengthTextField =
      ComponentFactory.createFormattedTextField(
        model.getModel(BeamFacade17bm.PROPNAME_WAVELENGTH_DESIRED_POSITION),
        wavelengthTextFieldFormatter);
    wavelengthTextField.setHorizontalAlignment(JTextField.RIGHT);
    builder.add(wavelengthTextField, cc.xy(3, 19, "fill, default"));
    builder.addLabel("\u00C5", cc.xy(5, 19));

    TwoStateButtonAction moveStopAction = new TwoStateValueModelButtonAction(
      "Move", "Stop",
      EpicsMotorMotionCommand.MOVE, EpicsMotorMotionCommand.STOP,
      false, true,
      model.getModel(BeamFacade17bm.PROPNAME_ENERGY_MOVING),
      model.getModel(BeamFacade17bm.PROPNAME_ENERGY_MOTION_COMMAND),
      false);
    registerInterestInCommandedEnergyMove(moveStopAction,
      TwoStateButtonAction.STATE_1_COMMAND);
    builder.add(new JButton(moveStopAction),
      cc.xywh(7, 17, 1, 3, "default, center"));

    builder.addSeparator("Defining slits", cc.xyw(1, 21, 7));

    builder.addLabel("Vertical size:", cc.xy(1, 23));
    JLabel verticalSizeValueLabel = ComponentFactory.createLabel(
      model.getModel(BeamFacade17bm.PROPNAME_VERTICAL_DEFINING_SLIT_SIZE),
      new DecimalFormat("####0.000"));
    builder.add(verticalSizeValueLabel, cc.xy(3, 23));
    builder.addLabel("\u03BCm", cc.xy(5, 23));
    Action openVerticalSlitSizeDialogAction =
      new OpenMoveSlitSizeDialogAction(UiUtilities.getRootFrame(),
        "Change...", "Defining slit vertical size",
        model.getVerticalDefiningSlitModel());
    builder.add(new JButton(openVerticalSlitSizeDialogAction), cc.xy(7, 23));

    builder.addLabel("Horizontal size:", cc.xy(1, 25));
    JLabel horizontalSizeValueLabel = ComponentFactory.createLabel(
      model.getModel(BeamFacade17bm.PROPNAME_HORIZONTAL_DEFINING_SLIT_SIZE),
      new DecimalFormat("####0.000"));
    builder.add(horizontalSizeValueLabel, cc.xy(3, 25));
    builder.addLabel("\u03BCm", cc.xy(5, 25));
    Action openHorizontalSlitSizeDialogAction =
      new OpenMoveMotorDialogAction(UiUtilities.getRootFrame(),
        "Change...", "Defining slit horizontal size",
        model.getHorizontalDefiningSlitModel(), "Current size:",
        "Desired size:");
    builder.add(new JButton(openHorizontalSlitSizeDialogAction),
      cc.xy(7, 25));

    builder.addSeparator("Intensity", cc.xyw(1, 27, 7));

    builder.addLabel("Beamline BPM:", cc.xy(1, 29));
    JLabel beamlineBpmValueLabel = ComponentFactory.createLabel(
      model.getModel(BeamFacade17bm.PROPNAME_BEAMLINE_BPM_INTENSITY),
      new DecimalFormat("#####0"));
    builder.add(beamlineBpmValueLabel, cc.xy(3, 29));
    builder.addLabel("counts", cc.xy(5, 29));

    builder.addLabel("Slit alignment:", cc.xy(1, 31));
    JLabel slitAlignmentValueLabel = ComponentFactory.createLabel(
      model.getModel(BeamFacade17bm.PROPNAME_SLIT_ALIGNMENT_INTENSITY),
      new DecimalFormat("#####0"));
    builder.add(slitAlignmentValueLabel, cc.xy(3, 31));
    builder.addLabel("counts", cc.xy(5, 31));

    builder.addLabel("Small ion chamber:", cc.xy(1, 33));
    JLabel smallIonChamberValueLabel = ComponentFactory.createLabel(
      model.getModel(BeamFacade17bm.PROPNAME_SMALL_ION_CHAMBER_INTENSITY),
      new DecimalFormat("#####0"));
    builder.add(smallIonChamberValueLabel, cc.xy(3, 33));
    builder.addLabel("a.u.", cc.xy(5, 33));

    builder.addSeparator("Attenuation", cc.xyw(1, 35, 7));

    builder.addLabel("Inserted filters:", cc.xy(1, 37));
    JLabel insertedFiltersValueLabel = ComponentFactory.createLabel(
      model.getInsertedFiltersModel());
    builder.add(insertedFiltersValueLabel, cc.xy(3, 37));
    Action changeAttenuationAction = new AbstractAction() {
      private static final long serialVersionUID = 1;
      public void actionPerformed(ActionEvent e) {
        changeAttenuationActionPerformed();
      }
    };
    changeAttenuationAction.putValue(Action.NAME, "Change...");
    builder.add(new JButton(changeAttenuationAction), cc.xy(7, 37));

    return builder.getPanel();
  }

  @Override
  protected JDialog createFilterRackDialog(Frame root) {
    return new FilterRackDialog17bm<FilterRackPresentationModel<FilterRack>>(
      root, "Attenuation", false, model.getFilterRackPresentationModel());
  }
}
