package imcacat.imcabc;

import java.awt.Color;
import java.awt.Component;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.text.DecimalFormat;
import java.util.Comparator;
import java.util.EnumMap;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SortingFocusTraversalPolicy;
import javax.swing.text.NumberFormatter;

import com.jgoodies.binding.beans.PropertyConnector;
import com.jgoodies.binding.value.ValueModel;
import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import imcacat.epicsca4j.ca.EpicsCaException;
import imcacat.epicsca4j.format.DoubleDecimalFormat;
import imcacat.epicsca4j.model.TpmacAssemblyMotionCommand;
import imcacat.imcabc.util.PhysicsUtilities;

/**
 * I am a <code>BeamPanel</code> for the 17-ID beamline.
 */
public class BeamPanel17id extends BeamPanel<BeamPresentationModel17id> {
  private static final long serialVersionUID = 1;

  public BeamPanel17id() throws InterruptedException, EpicsCaException {
    this(new BeamPresentationModel17id());
  }

  public BeamPanel17id(BeamPresentationModel17id model)
      throws InterruptedException, EpicsCaException {
    super(model);
  }

  protected JComponent buildLeftPanel() {
    FormLayout layout = new FormLayout(
      // Columns
      "right:max(85dlu;pref), 4dlu, right:max(45dlu;pref), 4dlu, " +
        "max(10dlu;pref), 4dlu, max(40dlu;pref)",
      // General section
      "pref, 4dlu, pref, 4dlu, " +
      // Shutter section
      "pref, 4dlu, pref, 4dlu, " +
      // Energy section
      "pref, 4dlu, pref, 2dlu, pref, 4dlu, pref, 2dlu, pref, 2dlu, pref, " +
        "2dlu, pref, 4dlu, " +
      // Beam-defining aperture section
      "pref, 4dlu, pref, 2dlu, pref, 4dlu, " +
      // Intensity monitors section
      "pref, 4dlu, pref, 2dlu, pref, 2dlu, pref, 2dlu, pref, 4dlu, " +
      // Attenuation section
      "pref, 4dlu, pref"
    );
    layout.setRowGroups(new int[][] {
      // separators
      {1, 5, 9, 23, 27, 39},
      // fields
      {3, 7, 11, 13, 17, 19, 21, 25, 29, 31, 33, 35, 37, 41}
    });
    PanelBuilder builder = new PanelBuilder(layout);
    builder.border(Borders.DIALOG);
    CellConstraints cc = new CellConstraints();

    builder.addSeparator("General", cc.xyw(1, 1, 7));

    builder.addLabel("Ring current:", cc.xy(1, 3));
    JLabel ringCurrentValueLabel = ComponentFactory.createLabel(
      model.getModel(BeamFacade.PROPNAME_RING_CURRENT),
      new DecimalFormat("##0.0"));
    builder.add(ringCurrentValueLabel, cc.xy(3, 3));
    builder.addLabel("mA", cc.xy(5, 3));

    builder.addSeparator("Shutter", cc.xyw(1, 5, 7));

    builder.addLabel("B shutter state:", cc.xy(1, 7));
    JLabel bShutterStateValueLabel = ComponentFactory.createBooleanLabel(
      model.getModel(BeamFacade.PROPNAME_B_SHUTTER_OPEN), "Open", "Closed",
      Color.RED, Color.BLACK);
    builder.add(bShutterStateValueLabel, cc.xy(3, 7));
    final JButton bShutterOpenCloseButton = new JButton(
      new TwoStateValueModelButtonAction("Open", "Close", true, false, false,
        true, model.getModel(BeamFacade.PROPNAME_B_SHUTTER_OPEN)));
    builder.add(bShutterOpenCloseButton, cc.xy(7, 7));

    builder.addSeparator("Energy", cc.xyw(1, 9, 7));

    builder.addLabel("Harmonic:", cc.xy(1, 11));
    JLabel harmonicValueLabel = ComponentFactory.createLabel(
      model.getModel(BeamFacade17id.PROPNAME_HARMONIC),
      new DecimalFormat("#0"));
    builder.add(harmonicValueLabel, cc.xy(3, 11));

    builder.addLabel("Undulator gap:", cc.xy(1, 13));
    JLabel gapValueLabel = ComponentFactory.createLabel(
      model.getModel(BeamFacade17id.PROPNAME_GAP),
      new DecimalFormat("##0.000"));
    builder.add(gapValueLabel, cc.xy(3, 13));
    builder.addLabel("mm", cc.xy(5, 13));

    builder.addLabel("Energy:", cc.xy(1, 15));
    JLabel energyValueLabel = ComponentFactory.createLabel(
      model.getModel(BeamFacade17id.PROPNAME_ENERGY_POSITION),
      new DecimalFormat("####0.000"));
    builder.add(energyValueLabel, cc.xy(3, 15));
    builder.addLabel("eV", cc.xy(5, 15));

    builder.addLabel("Wavelength:", cc.xy(1, 17));
    JLabel wavelengthValueLabel = ComponentFactory.createLabel(
      model.getModel(BeamFacade17id.PROPNAME_WAVELENGTH_POSITION),
      new DecimalFormat("0.0000000"));
    builder.add(wavelengthValueLabel, cc.xy(3, 17));
    builder.addLabel("\u00C5", cc.xy(5, 17));

    JLabel movingLabel = ComponentFactory.createBooleanLabel(
      model.getModel(BeamFacade17id.PROPNAME_ENERGY_MOVING), "Moving...", "",
      Color.RED, Color.BLACK);
    builder.add(movingLabel, cc.xywh(7, 15, 1, 3, "default, center"));

    builder.addLabel("Desired Energy:", cc.xy(1, 19));
    NumberFormatter energyTextFieldFormatter = new NumberFormatter(
      new DoubleDecimalFormat("####0.000"));
    energyTextFieldFormatter.setMinimum(new Double(
      Resources.getDefault().getEnergyLowLimit()));
    energyTextFieldFormatter.setMaximum(new Double(
      Resources.getDefault().getEnergyHighLimit()));
    final JTextField energyTextField =
      ComponentFactory.createFormattedTextField(
        model.getModel(BeamFacade17id.PROPNAME_ENERGY_DESIRED_POSITION),
        energyTextFieldFormatter);
    energyTextField.setHorizontalAlignment(JTextField.RIGHT);
    builder.add(energyTextField, cc.xy(3, 19, "fill, default"));
    builder.addLabel("eV", cc.xy(5, 19));

    builder.addLabel("Desired Wavelength:", cc.xy(1, 21));
    NumberFormatter wavelengthTextFieldFormatter = new NumberFormatter(
      new DoubleDecimalFormat("0.0000000"));
    wavelengthTextFieldFormatter.setMinimum(new Double(
      PhysicsUtilities.convertToWavelength(
        Resources.getDefault().getEnergyHighLimit())));
    wavelengthTextFieldFormatter.setMaximum(new Double(
      PhysicsUtilities.convertToWavelength(
        Resources.getDefault().getEnergyLowLimit())));
    final JTextField wavelengthTextField =
      ComponentFactory.createFormattedTextField(
        model.getModel(BeamFacade17id.PROPNAME_WAVELENGTH_DESIRED_POSITION),
        wavelengthTextFieldFormatter);
    wavelengthTextField.setHorizontalAlignment(JTextField.RIGHT);
    builder.add(wavelengthTextField, cc.xy(3, 21, "fill, default"));
    builder.addLabel("\u00C5", cc.xy(5, 21));

    TwoStateButtonAction moveStopAction = new TwoStateValueModelButtonAction(
      "Move", "Stop",
      TpmacAssemblyMotionCommand.START, TpmacAssemblyMotionCommand.ABORT,
      false, true,
      model.getModel(BeamFacade17id.PROPNAME_ENERGY_MOVING),
      model.getModel(BeamFacade17id.PROPNAME_ENERGY_MOTION_COMMAND),
      false);
    registerInterestInCommandedEnergyMove(moveStopAction,
      TwoStateButtonAction.STATE_1_COMMAND);
    final JButton energyMoveStopButton = new JButton(moveStopAction);
    builder.add(energyMoveStopButton,
      cc.xywh(7, 19, 1, 3, "default, center"));

    builder.addSeparator("Beam-defining aperture", cc.xyw(1, 23, 7));

    builder.addLabel("Aperture:", cc.xy(1, 25));
    JLabel apertureRbvLabel = ComponentFactory.createLabel(
      model.getApertureRbvDisplayStringModel());
    Map<MinibeamApertureRbv,Color> apertureRbvToColorMap =
      new EnumMap<MinibeamApertureRbv,Color>(MinibeamApertureRbv.class);
    apertureRbvToColorMap.put(MinibeamApertureRbv.MOVING, Color.RED);
    apertureRbvToColorMap.put(MinibeamApertureRbv.SCANNING, Color.RED);
    apertureRbvToColorMap.put(MinibeamApertureRbv.UNKNOWN, Color.RED);
    ValueModel apertureRbvLabelColorModel =
      ConverterFactory.createEnumToColorConverter(
        model.getModel(BeamFacade17id.PROPNAME_APERTURE_RBV),
        apertureRbvToColorMap);
    PropertyConnector.connectAndUpdate(apertureRbvLabelColorModel,
      apertureRbvLabel, "foreground");
    builder.add(apertureRbvLabel, cc.xy(3, 25, "left, default"));

    builder.addLabel("Desired Aperture:", cc.xy(1, 27));
    final JComboBox desiredApertureComboBox = ComponentFactory.createComboBox(
      model.getApertureSelectionInList());
    builder.add(desiredApertureComboBox, cc.xy(3, 27));
    final JButton desiredApertureMoveButton = new JButton(
      new TwoStateValueModelButtonAction("Move", "Stop",
        MinibeamApertureMotionCommand.MOVE,
        MinibeamApertureMotionCommand.ABORT, false, true,
        model.getModel(BeamFacade17id.PROPNAME_APERTURE_MOVING),
        model.getModel(BeamFacade17id.PROPNAME_APERTURE_MOTION_COMMAND),
        false));
    builder.add(desiredApertureMoveButton, cc.xy(7, 27));

    builder.addSeparator("Intensity", cc.xyw(1, 29, 7));

    builder.addLabel("Mirror BPM:", cc.xy(1, 31));
    JLabel mirrorBpmValueLabel = ComponentFactory.createLabel(
      model.getModel(BeamFacade17id.PROPNAME_MIRROR_BPM_INTENSITY),
      new DecimalFormat("#####0"));
    builder.add(mirrorBpmValueLabel, cc.xy(3, 31));

    builder.addLabel("Sample BPM:", cc.xy(1, 33));
    JLabel sampleBpmValueLabel = ComponentFactory.createLabel(
      model.getModel(BeamFacade17id.PROPNAME_SAMPLE_BPM_INTENSITY),
      new DecimalFormat("#####0"));
    builder.add(sampleBpmValueLabel, cc.xy(3, 33));

    builder.addLabel("Ionization chamber:", cc.xy(1, 35));
    JLabel ionChamberValueLabel = ComponentFactory.createLabel(
      model.getModel(BeamFacade17id.PROPNAME_ION_CHAMBER_INTENSITY),
      new DecimalFormat("#####0"));
    builder.add(ionChamberValueLabel, cc.xy(3, 35));

    builder.addLabel("Beam stop:", cc.xy(1, 37));
    JLabel beamStopValueLabel = ComponentFactory.createLabel(
      model.getModel(BeamFacade17id.PROPNAME_BEAM_STOP_INTENSITY),
      new DecimalFormat("#####0"));
    builder.add(beamStopValueLabel, cc.xy(3, 37));

    builder.addSeparator("Attenuation", cc.xyw(1, 39, 7));

    builder.addLabel("Flux attenuated:", cc.xy(1, 41));
    JLabel thicknessValueLabel = ComponentFactory.createLabel(
      model.getModel(BeamFacade17id.PROPNAME_ATTENUATION_PERCENTAGE),
      new DecimalFormat("##0.0"));
    builder.add(thicknessValueLabel, cc.xy(3, 41));
    builder.addLabel("%", cc.xy(5, 41));
    Action changeAttenuationAction = new AbstractAction() {
      private static final long serialVersionUID = 1;
      public void actionPerformed(ActionEvent e) {
        changeAttenuationActionPerformed();
      }
    };
    changeAttenuationAction.putValue(Action.NAME, "Change...");
    final JButton changeAttenuationButton = new JButton(
      changeAttenuationAction);
    builder.add(changeAttenuationButton, cc.xy(7, 41));

    JPanel result = builder.getPanel();
    result.setFocusTraversalPolicy(new SortingFocusTraversalPolicy(
      new Comparator<Component>() {
        private final Component[] components = {
          bShutterOpenCloseButton,
          energyTextField,
          wavelengthTextField,
          energyMoveStopButton,
          desiredApertureComboBox,
          desiredApertureMoveButton,
          changeAttenuationButton
        };

        @Override
        public int compare(Component o1, Component o2) {
          return oneBasedIndexOf(o1) - oneBasedIndexOf(o2);
        }

        private int oneBasedIndexOf(Component c) {
          for (int i = 0; i < components.length; i++) {
            if (components[i] == c) return i + 1;
          }
          return components.length + 1;
        }
      }));
    result.setFocusTraversalPolicyProvider(true);
    result.setFocusable(false);

    return result;
  }

  @Override
  protected JDialog createFilterRackDialog(Frame root) {
    return new
      ImcaFilterRackDialog<ImcaFilterRackPresentationModel<ImcaFilterRack>>(
        root, "Attenuation", false, model.getFilterRackPresentationModel());
  }
}
