package imcacat.imcabc;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.EnumSet;

import javax.swing.Action;

import imcacat.epicsca4j.ca.EpicsCaException;
import imcacat.epicsca4j.model.EpicsMotor;
import imcacat.epicsca4j.model.EpicsSlit;

import com.jgoodies.binding.value.ValueHolder;
import com.jgoodies.binding.value.ValueModel;

/**
 * I am the presentation model for {@link BeamPanel17bm}.
 */
public class BeamPresentationModel17bm
extends BeamPresentationModel<BeamFacade17bm,
    FilterRackPresentationModel<FilterRack>> {
  private static final long serialVersionUID = 1;

  protected Action setEnergyMotorMoveModeAction;
  protected ValueModel insertedFiltersModel;

  public BeamPresentationModel17bm() throws InterruptedException,
      EpicsCaException {
    this(new BeamFacade17bm());
  }

  public BeamPresentationModel17bm(BeamFacade17bm bean) {
    this(new ValueHolder(bean, true));
  }

  public BeamPresentationModel17bm(ValueModel beanChannel) {
    super(beanChannel);

    getModel(BeamFacade17bm.PROPNAME_ENERGY_MOVING).addValueChangeListener(
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          energyMotorMovingPropertyChanged((Boolean)event.getNewValue());
        }
      });

    setEnergyMotorMoveModeAction = new TwoStateValueModelButtonAction("Fine",
      "Coarse", EnergyMotorMoveMode.FINE, EnergyMotorMoveMode.COARSE,
      EnumSet.of(EnergyMotorMoveMode.COARSE, EnergyMotorMoveMode.CUSTOM),
      EnergyMotorMoveMode.FINE,
      getModel(BeamFacade17bm.PROPNAME_ENERGY_MOVE_MODE));

    insertedFiltersModel = new ValueHolder("");
    getModel(BeamFacade17bm.PROPNAME_FILTER_STATES).addValueChangeListener(
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          filterStatesChanged((FilterRackFilterState[])event.getNewValue());
        }
      });
    filterStatesChanged((FilterRackFilterState[])getModel(
      BeamFacade17bm.PROPNAME_FILTER_STATES).getValue());
  }

  public EpicsSlit getVerticalDefiningSlitModel() {
    return getBean().getVerticalDefiningSlitModel();
  }

  public EpicsMotor getHorizontalDefiningSlitModel() {
    return getBean().getHorizontalDefiningSlitModel();
  }

  public FilterRack getFilterRackModel() {
    return getBean().getFilterRackModel();
  }

  public ValueModel getInsertedFiltersModel() {
    return insertedFiltersModel;
  }

  public Action getSetEnergyMotorMoveModeAction() {
    return setEnergyMotorMoveModeAction;
  }

  protected FilterRackPresentationModel<FilterRack>
      createFilterRackPresentationModel() {
    return new FilterRackPresentationModel<FilterRack>(
      getBean().getFilterRackModel());
  }

  protected void energyMotorMovingPropertyChanged(Boolean newValue) {
    if (newValue == null) return;
    setEnergyMotorMoveModeAction.setEnabled(!newValue.booleanValue());
  }

  protected void filterStatesChanged(FilterRackFilterState[] newStates) {
    insertedFiltersModel.setValue(filterStatesToString(newStates));
  }

  protected String filterStatesToString(FilterRackFilterState[] states) {
    StringBuilder result = new StringBuilder();
    boolean atLeastOneFilterInserted = false;
    for (int i = 0; i < states.length; i++) {
      if (FilterRackFilterState.INSERTED.equals(states[i])) {
        result.append(i + 1);
        atLeastOneFilterInserted = true;
      } else {
        result.append("-");
      }
    }
    return atLeastOneFilterInserted ? result.toString() : "None";
  }
}
