package imcacat.imcabc;

import java.beans.IndexedPropertyChangeEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.EnumSet;

import javax.swing.DefaultListModel;

import org.apache.log4j.Logger;

import imcacat.epicsca4j.ca.EpicsCaException;

import com.jgoodies.binding.list.SelectionInList;
import com.jgoodies.binding.value.ValueHolder;
import com.jgoodies.binding.value.ValueModel;

/**
 * I am the presentation model for {@link BeamPanel17id}.
 */
public class BeamPresentationModel17id
extends BeamPresentationModel<BeamFacade17id,
    ImcaFilterRackPresentationModel<ImcaFilterRack>> {
  private static final long serialVersionUID = 1;

  private static final Logger LOG = Logger.getLogger(
    BeamPresentationModel17id.class);

  protected static final EnumSet<MinibeamApertureRbv>
    APERTURE_RBV_REAL_APERTURES = EnumSet.of(MinibeamApertureRbv.ONE,
      MinibeamApertureRbv.TWO, MinibeamApertureRbv.THREE,
      MinibeamApertureRbv.FOUR);

  protected ValueModel apertureRbvDisplayStringModel;
  protected MinibeamApertureRbv apertureRbv;

  protected MinibeamAperture aperture;
  protected DefaultListModel apertureListModel;
  protected SelectionInList<String> apertureSelectionInList;
  protected int apertureSelectionIndex;

  protected PropertyChangeListener apertureSizeChangedListener;
  protected Double[] apertureSize;

  public BeamPresentationModel17id() throws InterruptedException,
      EpicsCaException {
    this(new BeamFacade17id());
  }

  public BeamPresentationModel17id(BeamFacade17id bean)
  throws InterruptedException, EpicsCaException {
    this(new ValueHolder(bean, true));
  }

  public BeamPresentationModel17id(ValueModel beanChannel)
  throws InterruptedException, EpicsCaException {
    super(beanChannel);

    apertureSize = new Double[Minibeam.NUM_APERTURES];
    for (int i = 0; i < Minibeam.NUM_APERTURES; i++)
      apertureSize[i] = getBean().getApertureSize(i);
    apertureRbv = getBean().getApertureRbv();
    aperture = getBean().getAperture();

    apertureRbvDisplayStringModel = new ValueHolder(getDisplayString(
      apertureRbv));

    apertureListModel = new DefaultListModel();
    for (MinibeamAperture each : MinibeamAperture.values())
      apertureListModel.addElement(getDisplayString(each));
    apertureSelectionInList = new SelectionInList<String>(apertureListModel);
    apertureSelectionInList.setSelectionIndex(aperture.ordinal());
    apertureSelectionIndex = apertureSelectionInList.getSelectionIndex();

    apertureSelectionInList.addPropertyChangeListener(
      SelectionInList.PROPERTY_SELECTION_INDEX,
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          apertureSelectionIndexChanged((Integer)event.getNewValue());
        }
    });

    getModel(BeamFacade17id.PROPNAME_APERTURE_RBV).addValueChangeListener(
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          apertureRbvChanged((MinibeamApertureRbv)event.getNewValue());
        }
      });

    getModel(BeamFacade17id.PROPNAME_APERTURE).addValueChangeListener(
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          apertureChanged((MinibeamAperture)event.getNewValue());
        }
      });

    apertureSizeChangedListener = new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent event) {
        if (event instanceof IndexedPropertyChangeEvent) {
          apertureSizeChanged(
            ((IndexedPropertyChangeEvent)event).getIndex(),
            (Double)event.getNewValue());
        } else {
          LOG.warn("discarded non-indexed property change event for " +
            BeamFacade17id.class.getName() + " property " +
            BeamFacade17id.PROPNAME_APERTURE_SIZE);
        }
      }
    };
    addApertureSizeChangedListenerTo(getBean());
  }

  public ValueModel getApertureRbvDisplayStringModel() {
    return apertureRbvDisplayStringModel;
  }

  public SelectionInList<String> getApertureSelectionInList() {
    return apertureSelectionInList;
  }

  public FilterRack getFilterRackModel() {
    return getBean().getFilterRackModel();
  }

  @Override
  public void beforeBeanChange(BeamFacade17id oldBean,
      BeamFacade17id newBean) {
    if (oldBean != null)
      oldBean.removePropertyChangeListener(
        BeamFacade17id.PROPNAME_APERTURE_SIZE, apertureSizeChangedListener);
    super.beforeBeanChange(oldBean, newBean);
  }

  @Override
  public void afterBeanChange(BeamFacade17id oldBean,
      BeamFacade17id newBean) {
    addApertureSizeChangedListenerTo(newBean);
    super.afterBeanChange(oldBean, newBean);
  }

  protected ImcaFilterRackPresentationModel<ImcaFilterRack>
  createFilterRackPresentationModel() {
    return new ImcaFilterRackPresentationModel<ImcaFilterRack>(
      getBean().getFilterRackModel());
  }

  protected void addApertureSizeChangedListenerTo(BeamFacade17id aBean) {
    if (aBean != null)
      aBean.addPropertyChangeListener(
        BeamFacade17id.PROPNAME_APERTURE_SIZE, apertureSizeChangedListener);
  }

  protected void apertureRbvChanged(MinibeamApertureRbv newApertureRbv) {
    apertureRbv = newApertureRbv;
    updateApertureRbvDisplayString();
  }

  protected void apertureChanged(MinibeamAperture newAperture) {
    aperture = newAperture;
    apertureSelectionInList.setSelectionIndex(newAperture.ordinal());
  }

  protected void apertureSelectionIndexChanged(int newIndex) {
    int oldIndex = apertureSelectionIndex;
    apertureSelectionIndex = newIndex;
    if (oldIndex == newIndex) return;
    try {
      getBean().setAperture(MinibeamAperture.values()[newIndex]);
    } catch (Exception e) {
      LOG.error("failed to set aperture in Bean", e);
    }
  }

  protected void apertureSizeChanged(int index, Double newApertureSize) {
    if (index >= 0 && index < Minibeam.NUM_APERTURES) {
      apertureSize[index] = newApertureSize;
      if (APERTURE_RBV_REAL_APERTURES.contains(apertureRbv))
        updateApertureRbvDisplayString();
      apertureListModel.set(index + 1,
        getDisplayString(MinibeamAperture.values()[index + 1]));
    } else {
      LOG.warn("invalid aperture size index: " + index);
    }
  }

  protected void updateApertureRbvDisplayString() {
    apertureRbvDisplayStringModel.setValue(getDisplayString(apertureRbv));
  }

  protected String getDisplayString(MinibeamApertureRbv apertureRbv) {
    if (apertureRbv == MinibeamApertureRbv.NONE)
      return "None";
    if (apertureRbv == MinibeamApertureRbv.ONE)
      return String.format("1 (%.0f \u03BCm)", apertureSize[0]);
    if (apertureRbv == MinibeamApertureRbv.TWO)
      return String.format("2 (%.0f \u03BCm)", apertureSize[1]);
    if (apertureRbv == MinibeamApertureRbv.THREE)
      return String.format("3 (%.0f \u03BCm)", apertureSize[2]);
    if (apertureRbv == MinibeamApertureRbv.FOUR)
      return String.format("4 (%.0f \u03BCm)", apertureSize[3]);
    if (apertureRbv == MinibeamApertureRbv.MOVING)
      return "Moving...";
    if (apertureRbv == MinibeamApertureRbv.SCANNING)
      return "Scanning...";
    if (apertureRbv == MinibeamApertureRbv.UNKNOWN)
      return "UNKNOWN";
    if (apertureRbv == null)
      return "";
    throw new ProgrammerError("unexpected MinibeamApertureRbv: " +
      apertureRbv);
  }

  protected String getDisplayString(MinibeamAperture aperture) {
    if (aperture == MinibeamAperture.NONE)
      return "None";
    if (aperture == MinibeamAperture.ONE)
      return String.format("1 (%.0f \u03BCm)", apertureSize[0]);
    if (aperture == MinibeamAperture.TWO)
      return String.format("2 (%.0f \u03BCm)", apertureSize[1]);
    if (aperture == MinibeamAperture.THREE)
      return String.format("3 (%.0f \u03BCm)", apertureSize[2]);
    if (aperture == MinibeamAperture.FOUR)
      return String.format("4 (%.0f \u03BCm)", apertureSize[3]);
    if (aperture == null)
      return "";
    throw new ProgrammerError("unexpected MinibeamAperture: " + aperture);
  }
}
