package imcacat.imcabc;

import imcacat.epicsca4j.ca.EpicsCaException;
import imcacat.epicsca4j.model.EpicsPv;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.jfree.data.xy.XYDataset;

import com.jgoodies.binding.beans.Model;

/**
 * I am a BPM (Beam Position Monitor). I model the position of the beam.
 */
public class Bpm extends Model {
  private static final long serialVersionUID = 1;

  public static final String PROPNAME_X = "x";
  public static final String PROPNAME_Y = "y";

  protected BpmXYDataset dataset;
  protected EpicsPv xPv;
  protected EpicsPv yPv;

  public Bpm() throws InterruptedException, EpicsCaException {
    super();

    dataset = new BpmXYDataset();

    xPv = new EpicsPv(Resources.getDefault().getBpmXPositionPvName());
    xPv.addPropertyChangeListener(EpicsPv.PROPNAME_VALUE,
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent e) {
          updateX((Double)e.getNewValue());
        }
      });

    yPv = new EpicsPv(Resources.getDefault().getBpmYPositionPvName());
    yPv.addPropertyChangeListener(EpicsPv.PROPNAME_VALUE,
        new PropertyChangeListener() {
          public void propertyChange(PropertyChangeEvent e) {
            updateY((Double)e.getNewValue());
          }
      });
  }

  public double getX() {
    return dataset.getXValue(0, 0);
  }

  public double getY() {
    return dataset.getYValue(0, 0);
  }

  public XYDataset getXYDataset() {
    return dataset;
  }

  protected void updateX(Double newValue) {
    Double oldValue = getX();
    dataset.setX(newValue);
    if (!oldValue.equals(newValue))
      firePropertyChange(PROPNAME_X, oldValue, newValue);
  }

  protected void updateY(Double newValue) {
    Double oldValue = getY();
    dataset.setY(newValue);
    if (!oldValue.equals(newValue))
      firePropertyChange(PROPNAME_Y, oldValue, newValue);
  }
}
