package imcacat.imcabc;

import imcacat.epicsca4j.bean.OneWayPropertyChangeMediator;
import imcacat.epicsca4j.ca.EpicsCaException;
import imcacat.epicsca4j.model.EpicsPv;

/**
 * I am a model for details about a {@link BpmAutoOptimization17bm}.  I
 * provide the positions of the motors optimized by a
 * <code>BpmAutoOptimization17bm</code>.
 */
public class BpmAutoOptimizationDetails17bm
extends AutoOptimizationDetails {
  private static final long serialVersionUID = 1;

  public static final String PROPNAME_BPM_HORIZONTAL_MOTOR_POSITION =
    "bpmHorizontalMotorPosition";
  public static final String PROPNAME_BPM_VERTICAL_MOTOR_POSITION =
    "bpmVerticalMotorPosition";

  protected EpicsPv bpmHorizontalMotorPositionPv;
  protected OneWayPropertyChangeMediator bpmHorizontalMotorPositionMediator;
  protected EpicsPv bpmVerticalMotorPositionPv;
  protected OneWayPropertyChangeMediator bpmVerticalMotorPositionMediator;

  public BpmAutoOptimizationDetails17bm()
  throws InterruptedException, EpicsCaException {
    super();

    bpmHorizontalMotorPositionPv = new EpicsPv(
      Resources.getDefault().getBpmHorizontalPositionPvName(), false,
      true);
    bpmHorizontalMotorPositionMediator = new OneWayPropertyChangeMediator(
      bpmHorizontalMotorPositionPv, this, EpicsPv.PROPNAME_VALUE,
      PROPNAME_BPM_HORIZONTAL_MOTOR_POSITION);

    bpmVerticalMotorPositionPv = new EpicsPv(
      Resources.getDefault().getBpmVerticalPositionPvName(), false,
      true);
    bpmVerticalMotorPositionMediator = new OneWayPropertyChangeMediator(
      bpmVerticalMotorPositionPv, this, EpicsPv.PROPNAME_VALUE,
      PROPNAME_BPM_VERTICAL_MOTOR_POSITION);
  }

  @Override
  public void release() {
    bpmHorizontalMotorPositionMediator.release();
    bpmHorizontalMotorPositionPv.release();
    bpmVerticalMotorPositionMediator.release();
    bpmVerticalMotorPositionPv.release();
    super.release();
  }

  public double getBpmHorizontalMotorPosition()
  throws InterruptedException, EpicsCaException {
    return (Double)bpmHorizontalMotorPositionPv.getValue();
  }

  public double getBpmVerticalMotorPosition()
  throws InterruptedException, EpicsCaException {
    return (Double)bpmVerticalMotorPositionPv.getValue();
  }
}
