package imcacat.imcabc;

import imcacat.epicsca4j.ca.EpicsCaException;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYDataset;
import org.jfree.ui.RectangleInsets;

import com.jgoodies.binding.PresentationModel;
import com.jgoodies.binding.value.ValueHolder;
import com.jgoodies.binding.value.ValueModel;

/**
 * I am the presentation model for {@link BpmChartPanel}.
 */
public class BpmChartPresentationModel extends PresentationModel<Bpm> {
  private static final long serialVersionUID = 1;

  protected XYPlot plot;
  protected JFreeChart chart;

  public BpmChartPresentationModel()
  throws InterruptedException, EpicsCaException {
    this(new Bpm());
  }

  public BpmChartPresentationModel(Bpm bean) {
    this(new ValueHolder(bean, true));
  }

  public BpmChartPresentationModel(ValueModel beanChannel) {
    super(beanChannel);
    initializePlot();
    initializeChart();
  }

  protected void initializePlot() {
    NumberAxis xAxis = new NumberAxis("X (\u03BCm)");
    xAxis.setAutoRange(false);
    xAxis.setRange(-4000.0, 4000.0);
    NumberAxis yAxis = new NumberAxis("Y (\u03BCm)");
    yAxis.setAutoRange(false);
    yAxis.setRange(-4000.0, 4000.0);
    XYItemRenderer renderer = new XYLineAndShapeRenderer(false, true);
    renderer.setBaseItemLabelsVisible(true);
    renderer.setBaseItemLabelGenerator(new BpmXYItemLabelGenerator());
    plot = new XYPlot(getXYDataset(), xAxis, yAxis, renderer);
    plot.setOrientation(PlotOrientation.VERTICAL);
  }

  protected void initializeChart() {
    chart = new JFreeChart(null, JFreeChart.DEFAULT_TITLE_FONT, plot, false);
    // Hack. Setting the right inset to 15 pixels is required to work around
    // what appears to be a bug where if the x-axis tick label is being
    // displayed on the far right it gets truncated if it is long.
    chart.setPadding(new RectangleInsets(0, 0, 0, 15));
  }

  public JFreeChart getChart() {
    return chart;
  }

  public XYDataset getXYDataset() {
    return getBean().getXYDataset();
  }
}
