package imcacat.imcabc;

import org.jfree.data.xy.AbstractXYDataset;

/**
 * I am an <code>XYDataset</code> for the position of the beam. I may be used
 * with a {@link BpmChartPanel} to provide a chart of the beam position.
 */
public class BpmXYDataset extends AbstractXYDataset {
  private static final long serialVersionUID = 1;

  protected static final String SERIES_KEY = "BpmXandY";

  protected Double x = 0.0;
  protected Double y = 0.0;

  public BpmXYDataset() {
    this(null, null);
  }

  public BpmXYDataset(Double x, Double y) {
    super();
    setX(x);
    setY(y);
  }

  public void setX(Double newValue) {
    if (newValue == null) return;
    Double oldValue = x;
    x = newValue;
    if (!oldValue.equals(newValue)) fireDatasetChanged();
  }

  public void setY(Double newValue) {
    if (newValue == null) return;
    Double oldValue = y;
    y = newValue;
    if (!oldValue.equals(newValue)) fireDatasetChanged();
  }

  @Override
  public int getSeriesCount() {
    return 1;
  }

  @Override
  public Comparable<String> getSeriesKey(int series) {
    if (series != 0) throw new IndexOutOfBoundsException("bad series index");
    return SERIES_KEY;
  }

  public int getItemCount(int series) {
    if (series != 0) throw new IndexOutOfBoundsException("bad series index");
    return 1;
  }

  public Number getX(int series, int item) {
    if (series != 0) throw new IndexOutOfBoundsException("bad series index");
    if (item != 0) throw new IndexOutOfBoundsException("bad item index");
    return x;
  }

  public Number getY(int series, int item) {
    if (series != 0) throw new IndexOutOfBoundsException("bad series index");
    if (item != 0) throw new IndexOutOfBoundsException("bad item index");
    return y;
  }
}
