package imcacat.imcabc;

import static org.junit.Assert.assertEquals;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.text.ParseException;
import java.util.List;

import org.junit.Test;

/**
 * I test {@link ChoochAnomalousScatteringFactorsReader}.
 */
public class ChoochAnomalousScatteringFactorsReaderTest {
  protected static final String CHARACTER_SET_NAME = "US-ASCII";

  /*
   * Ensure correctly handles leading whitespace before energies.
   */
  @Test
  public void testParse01()
  throws UnsupportedEncodingException, IOException, ParseException {
    String input =
      " 7092.1947     0.48    -5.40\n" +
      " 7092.3932     0.47    -5.41\n" +
      " 7092.5918     0.47    -5.42\n";
    Double expectedEnergies[] = {7092.1947, 7092.3932, 7092.5918};
    Double expectedFPrimes[] = {-5.40, -5.41, -5.42};
    Double expectedFDoublePrimes[] = {0.48, 0.47, 0.47};
    ChoochAnomalousScatteringFactorsReader reader = null;
    AnomalousScatteringFactors factors;
    try {
      reader = new ChoochAnomalousScatteringFactorsReader(
        new ByteArrayInputStream(input.getBytes(CHARACTER_SET_NAME)));
      factors = reader.parse();
    } finally {
      if (reader != null) reader.close();
    }
    List<Double> actualEnergies = factors.getEnergies();
    assertEquals(expectedEnergies.length, actualEnergies.size());
    for (int i = 0; i < expectedEnergies.length; i++)
      assertEquals(expectedEnergies[i], actualEnergies.get(i), 0.0001);
    List<Double> actualFPrimes = factors.getFPrimes();
    assertEquals(expectedFPrimes.length, actualFPrimes.size());
    for (int i = 0; i < expectedFPrimes.length; i++)
      assertEquals(expectedFPrimes[i], actualFPrimes.get(i), 0.01);
    List<Double> actualFDoublePrimes = factors.getFDoublePrimes();
    assertEquals(expectedFDoublePrimes.length, actualFDoublePrimes.size());
    for (int i = 0; i < expectedFDoublePrimes.length; i++)
      assertEquals(expectedFDoublePrimes[i], actualFDoublePrimes.get(i),
        0.01);
  }

  /*
   * Ensure correctly handles no leading whitespace before energies.
   */
  @Test
  public void testParse02()
  throws UnsupportedEncodingException, IOException, ParseException {
    String input =
      "12092.1947     0.48    -5.40\n" +
      "12092.3932     0.47    -5.41\n" +
      "12092.5918     0.47    -5.42\n";
    Double expectedEnergies[] = {12092.1947, 12092.3932, 12092.5918};
    Double expectedFPrimes[] = {-5.40, -5.41, -5.42};
    Double expectedFDoublePrimes[] = {0.48, 0.47, 0.47};
    ChoochAnomalousScatteringFactorsReader reader = null;
    AnomalousScatteringFactors factors;
    try {
      reader = new ChoochAnomalousScatteringFactorsReader(
        new ByteArrayInputStream(input.getBytes(CHARACTER_SET_NAME)));
      factors = reader.parse();
    } finally {
      if (reader != null) reader.close();
    }
    List<Double> actualEnergies = factors.getEnergies();
    assertEquals(expectedEnergies.length, actualEnergies.size());
    for (int i = 0; i < expectedEnergies.length; i++)
      assertEquals(expectedEnergies[i], actualEnergies.get(i), 0.0001);
    List<Double> actualFPrimes = factors.getFPrimes();
    assertEquals(expectedFPrimes.length, actualFPrimes.size());
    for (int i = 0; i < expectedFPrimes.length; i++)
      assertEquals(expectedFPrimes[i], actualFPrimes.get(i), 0.01);
    List<Double> actualFDoublePrimes = factors.getFDoublePrimes();
    assertEquals(expectedFDoublePrimes.length, actualFDoublePrimes.size());
    for (int i = 0; i < expectedFDoublePrimes.length; i++)
      assertEquals(expectedFDoublePrimes[i], actualFDoublePrimes.get(i),
        0.01);
  }
}
