package imcacat.imcabc;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

import com.jgoodies.binding.value.ValueModel;

/**
 * I open a file chooser dialog, and I set the value of my
 * <code>ValueModel</code> to the chosen file.
 */
public class ChooseFileAction extends AbstractAction {
  private static final long serialVersionUID = 1;

  protected static final String DEFAULT_APPROVE_TEXT = "Select";

  protected Component parent;
  protected String dialogTitle;
  protected String approveButtonText;
  protected FileFilter fileFilter;
  protected File selectedFile;
  protected ValueModel model;

  public ChooseFileAction(Component parent, String name,
      String dialogTitle, ValueModel model) {
    this(parent, name, dialogTitle, DEFAULT_APPROVE_TEXT, model);
  }

  public ChooseFileAction(Component parent, String name,
      String dialogTitle, ValueModel model, FileFilter fileFilter) {
    this(parent, name, dialogTitle, DEFAULT_APPROVE_TEXT, model, fileFilter);
  }

  public ChooseFileAction(Component parent, String name,
      String dialogTitle, String approveButtonText, ValueModel model) {
    this(parent, name, dialogTitle, approveButtonText, model, null);
  }

  public ChooseFileAction(Component parent, String name,
      String dialogTitle, String approveButtonText, ValueModel model,
      FileFilter fileFilter) {
    super();

    this.parent = parent;
    this.dialogTitle = dialogTitle;
    this.approveButtonText = approveButtonText;
    this.model = model;
    this.fileFilter = fileFilter;
    putValue(Action.NAME, name);
  }

  public void setSelectedFile(File selectedFile) {
    this.selectedFile = selectedFile;
  }

  public void actionPerformed(ActionEvent e) {
    doit();
  }

  public void doit() {
    JFileChooser chooser = new JFileChooser();
    chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
    chooser.setDialogTitle(dialogTitle);
    if (selectedFile != null) {
      chooser.setSelectedFile(selectedFile);
    } else {
      File value = (File)model.getValue();
      chooser.setCurrentDirectory(value == null ?
        null : value.getParentFile());
    }
    if (fileFilter != null) chooser.setFileFilter(fileFilter);
    int returnState = chooser.showDialog(parent, approveButtonText);
    if (returnState == JFileChooser.APPROVE_OPTION)
      model.setValue(chooser.getSelectedFile());
  }
}
