package imcacat.imcabc;

import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;

import joptsimple.OptionException;
import joptsimple.OptionParser;
import joptsimple.OptionSet;

/**
 * I parse this application's command line options.
 */
public class CliArgumentsParser {
  private static final String CONF_FILE = "f";
  private static final String HELP = "h";
  private static final String VERSION = "V";

  private static final OptionParser parser;

  static {
    parser = createOptionParser();
  }

  public static CliArguments parse(String[] args)
  throws CliArgumentException {
    try {
      OptionSet options = parser.parse(args);
      CliArguments result = new CliArguments();
      result.help(parseHelp(options));
      if (result.help()) return result;
      result.version(parseVersion(options));
      if (result.version()) return result;
      result.confFile(parseConfFile(options));
      return result;
    } catch (OptionException e) {
      if (e.getMessage() != null)
        throw new CliArgumentException(e.getMessage(), e);
      else
        throw new CliArgumentException("Failed to parse CLI arguments", e);
    }
  }

  private static boolean parseHelp(OptionSet options) {
    return options.has(HELP);
  }

  private static boolean parseVersion(OptionSet options) {
    return options.has(VERSION);
  }

  private static File parseConfFile(OptionSet options) {
    String confFilePath = (String)options.valueOf(CONF_FILE);
    return (confFilePath == null) ? null : new File(confFilePath);
  }

  public static void printHelp(OutputStream sink) throws IOException {
    Writer sinkWriter = new OutputStreamWriter(sink);
    try {
      printHelp(sinkWriter);
    } finally {
      sinkWriter.close();
    }
  }

  public static void printHelp(Writer sink) throws IOException {
    String newline = Resources.getLineSeparator();
    String[] lines = {
      "Imcabc is a program for controlling an IMCA-CAT beamline.",
      "",
      "Usage: imcabc [-hV] [-f conf_file]",
      "",
      "Options:",
      "  -f FILE  Read configuration from FILE",
      "  -h       Print this help message and exit",
      "  -V       Print version number and exit"
    };
    for (String each : lines) {
      sink.write(each);
      sink.write(newline);
    }
  }

  private static OptionParser createOptionParser() {
    OptionParser result = new OptionParser();
    result.accepts(CONF_FILE).withRequiredArg();
    result.accepts(HELP);
    result.accepts(VERSION);
    return result;
  }
}
