package imcacat.imcabc;

import imcacat.epicsca4j.ca.EpicsCaException;
import imcacat.epicsca4j.model.EpicsPv;

/**
 * I am an <code>Optimization</code> related to the DCM for the 17-ID
 * beamline.
 */
public abstract class DcmOptimization17id extends Optimization17id {
  /* Appropriate for giving real time intensity reading time to adjust to new
   * intensity after a move of Second Crystal Pitch or Roll. */
  protected static final long BEFORE_RESET_MONO_AUTOBP_AVG_INT_DLY = 3000;
  /* Appropriate for mono auto beam positioner feedback loop rate of 1 second
   * or faster; if slower than 1 second, this should be increased to be longer
   * than the feedback loop rate */
  protected static final long AFTER_RESET_MONO_AUTOBP_AVG_INT_DLY = 1500;

  protected EpicsPv monoAutobpAverageIntensityResetPv;

  @Override
  protected void initialize()
  throws InterruptedException, EpicsCaException {
    super.initialize();
    monoAutobpAverageIntensityResetPv = new EpicsPv(
      Resources.getDefault().getMonoAutobpAverageIntensityResetPvName(),
      false, false);
  }

  @Override
  public void release() {
    if (monoAutobpAverageIntensityResetPv != null)
      monoAutobpAverageIntensityResetPv.release();
    super.release();
  }

  protected void resetMonoAutobpAverageIntensity()
  throws InterruptedException, EpicsCaException {
    /* Wait a little bit to be sure the real time intensity reading has
     * updated with the latest value after a move of Second Crystal Pitch or
     * Roll. */
    Thread.sleep(BEFORE_RESET_MONO_AUTOBP_AVG_INT_DLY);
    monoAutobpAverageIntensityResetPv.setValueAndWait((short)1);
    /* Wait just a little bit to be sure the auto beam positioner has seen
     * this change; otherwise, if this method call is followed by enabling the
     * auto beam positioner, it might act on the old average intensity value
     * rather than the new value after the reset */
    Thread.sleep(AFTER_RESET_MONO_AUTOBP_AVG_INT_DLY);
  }
}
