package imcacat.imcabc;

import imcacat.epicsca4j.convert.BasicEpicsChannelAccessValueConverter;
import gov.aps.jca.dbr.DBR;

/**
 * I convert <code>Double</code> values from EPICS Channel Access to
 * <code>Short</code> values and vice-versa.
 */
public class DoubleToShortEpicsCaValueConverter extends
    BasicEpicsChannelAccessValueConverter {
  public DoubleToShortEpicsCaValueConverter() {
    super();
  }

  public Object convertToJava(DBR dbrValue, boolean isArray) {
    Object value = dbrValue.getValue();
    return isArray ? convertToShort((double[])value)
      : convertToShort(((double[])value)[0]);
  }

  public Object convertToChannelAccess(Object javaValue) {
    return (javaValue instanceof short[])
      ? convertToDouble((short[])javaValue)
      : convertToDouble((Short)javaValue);
  }

  protected short[] convertToShort(double[] values) {
    short newValues[] = new short[values.length];
    for (int i = 0; i < values.length; i++)
      newValues[i] = convertToShort(values[i]);
    return newValues;
  }

  protected Short convertToShort(double value) {
    return (short)value;
  }

  protected double[] convertToDouble(short[] values) {
    double newValues[] = new double[values.length];
    for (int i = 0; i < values.length; i++)
      newValues[i] = convertToDouble(values[i]);
    return newValues;
  }

  protected Double convertToDouble(Short value) {
    return value.doubleValue();
  }
}
