package imcacat.imcabc;

import java.text.FieldPosition;
import java.text.Format;
import java.text.ParsePosition;

/**
 * I am a <code>Format</code> that accepts <code>null</code> values. I format
 * <code>null</code> as an empty string and parse a <code>null</code> or
 * empty string as a <code>null</code> object.
 */
public class EmptyIfNullFormat extends Format {
  private static final long serialVersionUID = 1;

  protected Format decoratedFormat;

  public EmptyIfNullFormat() {
    super();
  }

  public EmptyIfNullFormat(Format decoratedFormat) {
    super();
    this.decoratedFormat = decoratedFormat;
  }

  public StringBuffer format(Object obj, StringBuffer toAppendTo,
      FieldPosition pos) {
    if (decoratedFormat == null) return toAppendTo;
    if (obj == null) return toAppendTo;
    return decoratedFormat.format(obj, toAppendTo, pos);
  }

  public Object parseObject(String source, ParsePosition pos) {
    if (decoratedFormat == null) return null;
    if (source == null || source.equals("")) return null;
    return decoratedFormat.parseObject(source, pos);
  }
}
