package imcacat.imcabc;

import java.awt.Color;
import java.util.Map;

import com.jgoodies.binding.value.BindingConverter;

/**
 * I convert <code>Enum</code> objects to <code>Color</code> objects using
 * the specified <code>Map</code>. I only convert in one direction (i.e. I do
 * not convert from <code>Color</code> objects to <code>Enum</code> objects).
 * <p>
 * I throw a <code>ClassCastException</code> if the value to convert is not an
 * <code>Enum</code>.
 */
public class EnumToColorConverter<E extends Enum<E>>
implements BindingConverter {
  protected Map<E,Color> enumToColorMap;
  protected Color defaultColor;

  /**
   * Calls <code>EnumToColorConverter(ValueModel, Map&lt;E,Color&gt;,
   * Color)</code> constructor with a default color of black.
   */
  public EnumToColorConverter(Map<E,Color> enumToColorMap) {
    this(enumToColorMap, Color.BLACK);
  }

  public EnumToColorConverter(Map<E,Color> enumToColorMap,
      Color defaultColor) {
    super();
    this.enumToColorMap = enumToColorMap;
    this.defaultColor = defaultColor;
  }

  /**
   * Converts the <code>Enum</code> source value to a <code>Color</code>
   * target value.
   *
   * @param sourceValue  the <code>Enum</code> to convert to a
   *                     <code>Color</code>
   *
   * @return the <code>Color</code> associated with <code>sourceValue</code>,
   *         or the default <code>Color</code> if <code>sourceValue</code> is
   *          <code>null</code> or does not exist in the <code>Map</code>
   *
   * @throws ClassCastException  if <code>sourceValue</code> is not an
   *         <code>Enum</code> nor <code>null</code>
   */
  public Object targetValue(Object sourceValue) {
    if (sourceValue == null) return defaultColor;
    Color value = enumToColorMap.get((Enum<?>)sourceValue);
    return value == null ? defaultColor : value;
  }

  /**
   * Returns <code>null</code>.
   */
  public Object sourceValue(Object targetValue) {
    return null;
  }
}
