package imcacat.imcabc;

import java.awt.BorderLayout;
import java.awt.Frame;
import java.util.EnumMap;
import java.util.Map;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.WindowConstants;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.ColumnSpec;
import com.jgoodies.forms.layout.FormLayout;

/**
 * I am a view for controlling a {@link FilterRack}.
 */
public class FilterRackDialog<T extends FilterRackPresentationModel<?>>
extends JDialog {
  private static final long serialVersionUID = 1;

  protected T model;

  protected Action doneAction;

  public FilterRackDialog(Frame owner, String title, boolean modal, T model) {
    super(owner, title, modal);
    this.model = model;
    setContentPane(buildContentPane());
    setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
  }

  protected JComponent buildContentPane() {
    JPanel panel = new JPanel(new BorderLayout());
    panel.add(buildMainPanel(), BorderLayout.CENTER);
    return panel;
  }

  protected JComponent buildMainPanel() {
    FormLayout layout = new FormLayout(
      "right:pref, 4dlu, right:pref, 4dlu, right:max(40dlu;pref), 4dlu, " +
        "pref, 4dlu, pref",
      "pref, 4dlu, pref, 2dlu, pref, 2dlu, pref, 2dlu, pref, 2dlu, pref, " +
        "2dlu, pref, 2dlu, pref, 2dlu, pref"
    );
    layout.setRowGroups(new int[][] {{1, 3, 5, 7, 9, 11, 13, 15}});
    layout.setColumnGroups(new int[][] {{7, 9}});
    PanelBuilder builder = new PanelBuilder(layout);
    builder.border(Borders.DIALOG);
    CellConstraints cc = new CellConstraints();

    int row = buildTopOfMainPanel(builder, cc, 1);

    builder.add(new JButton(new SetValueButtonAction("Insert All",
      model.getModel(FilterRack.PROPNAME_INSERT),
      FilterRackFilter.ALL)), cc.xy(7, row));
    builder.add(new JButton(new SetValueButtonAction("Retract All",
        model.getModel(FilterRack.PROPNAME_RETRACT),
        FilterRackFilter.ALL)), cc.xy(9, row));
    row += 2;

    String[] filterDescriptions = getFilterDescriptions();
    boolean filterDescriptionsEmpty = true;
    for (int i = 0; i < filterDescriptions.length; i++) {
      if (filterDescriptions[i].length() > 0) {
        filterDescriptionsEmpty = false;
        break;
      }
    }
    if (filterDescriptionsEmpty)
      layout.setColumnSpec(2, ColumnSpec.decode("0dlu"));

    Map<FilterRackFilterState,String> filterStateToStringMap =
      new EnumMap<FilterRackFilterState,String>(FilterRackFilterState.class);
    filterStateToStringMap.put(FilterRackFilterState.INSERTED, "Inserted");
    filterStateToStringMap.put(FilterRackFilterState.RETRACTED, "Retracted");
    filterStateToStringMap.put(FilterRackFilterState.OPEN_CIRCUIT,
      "Open circuit");
    filterStateToStringMap.put(FilterRackFilterState.SHORT_CIRCUIT,
      "Short circuit");
    filterStateToStringMap.put(FilterRackFilterState.COMMUNICATION_ERROR,
      "Comm. error");
    for (int i = 0; i < FilterRack.NUM_FILTERS; i++) {
      String filterLabelText = "Filter " + (i + 1);
      String filterLabelDescription = filterDescriptions[i] + ":";
      if ("".equals(filterDescriptions[i])) {
        filterLabelText += ":";
        filterLabelDescription = "";
      }
      builder.addLabel(filterLabelText, cc.xy(1, row));
      builder.addLabel(filterLabelDescription, cc.xy(3, row));
      builder.add(ComponentFactory.createEnumLabel(
          model.getModel(FilterRack.getStatePropertyName(i)),
          filterStateToStringMap),
        cc.xy(5, row));
      builder.add(new JButton(new SetValueButtonAction("Insert",
          model.getModel(FilterRack.PROPNAME_INSERT),
          FilterRack.getFilter(i))),
        cc.xy(7, row));
      builder.add(new JButton(new SetValueButtonAction("Retract",
          model.getModel(FilterRack.PROPNAME_RETRACT),
          FilterRack.getFilter(i))),
        cc.xy(9, row));
      row += 2;
    }

    return builder.getPanel();
  }

  protected int buildTopOfMainPanel(PanelBuilder builder, CellConstraints cc,
      int startRow) {
    return startRow;
  }

  protected String[] getFilterDescriptions() {
    String[] descriptions = new String[ImcaFilterRack.NUM_FILTERS];
    for (int i = 0; i < descriptions.length; i++) descriptions[i] = "";
    return descriptions;
  }
}
