package imcacat.imcabc;

import java.net.URL;

import javax.help.DefaultHelpModel;
import javax.help.HelpSet;
import javax.help.HelpSetException;
import javax.help.InvalidHelpSetContextException;
import javax.help.Map;
import javax.help.TextHelpModel;

import org.apache.log4j.Logger;

import com.jgoodies.binding.beans.Model;

/**
 * I am the model for the help system of this application.
 */
public class Help extends Model {
  private static final long serialVersionUID = 1;

  private static final Logger LOG = Logger.getLogger(Help.class);

  protected static final String ROOT_MAP_ID_NAME = "N40001";

  public static final String PROPNAME_HELPSET = "helpSet";

  protected HelpSet helpSet;
  protected TextHelpModel textHelpModel;

  public Help(Beamline beamline) {
    helpSet = createHelpSet(beamline);
    textHelpModel = new DefaultHelpModel(helpSet);
    try {
      textHelpModel.setCurrentID(Map.ID.create(ROOT_MAP_ID_NAME, helpSet));
    } catch (InvalidHelpSetContextException e) {
      LOG.error("Failed to set current help ID");
    }
  }

  public HelpSet getHelpSet() {
    return helpSet;
  }

  public TextHelpModel getTextHelpModel() {
    return textHelpModel;
  }

  protected static HelpSet createHelpSet(Beamline beamline) {
    URL helpSetUrl = Help.class.getResource(getHelpSetResourcePath(beamline));
    if (helpSetUrl == null) {
      LOG.error("Failed to find HelpSet resource");
      return null;
    } else {
      try {
        return new HelpSet(Help.class.getClassLoader(), helpSetUrl);
      } catch (HelpSetException e) {
        LOG.error("Failed to create HelpSet", e);
        return null;
      }
    }
  }

  protected static String getHelpSetResourcePath(Beamline beamline) {
    if (Beamline.ID17 == beamline) return "/resources/help17id/jhelpset.hs";
    if (Beamline.BM17 == beamline) return "/resources/help17bm/jhelpset.hs";
    throw new ProgrammerError("unexpected beamline: " + beamline);
  }
}
