package imcacat.imcabc;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;

import javax.help.HelpSet;
import javax.help.JHelpContentViewer;
import javax.help.TextHelpModel;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;

/**
 * I am the help frame (i.e. window) for this application.
 */
public class HelpFrame extends JFrame {
  private static final long serialVersionUID = 1;

  protected static final Dimension PREFERRED_SIZE = new Dimension(640, 480);
  protected static final int TABBED_PANE_LEFT_SIDE_WIDTH = 150;
  protected static final String TOC_VIEW_ID = "TOC";

  protected HelpPresentationModel model;

  public HelpFrame(Beamline beamline) {
    this(new HelpPresentationModel(new Help(beamline)));
  }

  public HelpFrame(HelpPresentationModel model) {
    super();
    this.model = model;
    initialize();
  }

  protected void initialize() {
    setTitle(Resources.getAppName() + " Help");
    setIconImage(Resources.getAppIcon().getImage());
    setContentPane(buildContentPane());
  }

  protected HelpSet getHelpSet() {
    return model.getBean().getHelpSet();
  }

  protected TextHelpModel getTextHelpModel() {
    return model.getBean().getTextHelpModel();
  }

  protected JComponent buildContentPane() {
    JPanel panel = new JPanel(new BorderLayout());
    panel.add(buildMainPanel(), BorderLayout.CENTER);
    return panel;
  }

  protected JComponent buildMainPanel() {
    JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
    HelpSet helpSet = getHelpSet();
    TextHelpModel textHelpModel = getTextHelpModel();
    Component tocNavigator = (helpSet == null)
      ? new JLabel("Unavailable.", SwingConstants.CENTER)
      : helpSet.getNavigatorView(TOC_VIEW_ID).createNavigator(textHelpModel);
    JComponent contentViewer = (helpSet == null)
      ? new JLabel("Unavailable.", SwingConstants.CENTER)
      : new JHelpContentViewer(textHelpModel);
    splitPane.setLeftComponent(tocNavigator);
    splitPane.setRightComponent(contentViewer);
    splitPane.setDividerLocation(TABBED_PANE_LEFT_SIDE_WIDTH);
    splitPane.setBorder(new EmptyBorder(10, 10, 10, 10));
    splitPane.setPreferredSize(PREFERRED_SIZE);
    return splitPane;
  }
}
