package imcacat.imcabc;

import imcacat.epicsca4j.format.DoubleDecimalFormat;

import java.awt.Frame;
import java.text.DecimalFormat;

import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.text.NumberFormatter;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.RowSpec;

/**
 * I am a view for controlling an {@link ImcaFilterRack}.
 */
public class ImcaFilterRackDialog
<T extends ImcaFilterRackPresentationModel<?>>
extends FilterRackDialog<T> {
  private static final long serialVersionUID = 1;

  public ImcaFilterRackDialog(Frame owner, String title, boolean modal,
      T model) {
    super(owner, title, modal, model);
  }

  @Override
  protected int buildTopOfMainPanel(PanelBuilder builder, CellConstraints cc,
      int startRow) {
    int row = super.buildTopOfMainPanel(builder, cc, startRow);

    FormLayout layout = builder.getLayout();

    layout.insertRow(row, RowSpec.decode("pref"));
    layout.insertRow(row + 1, RowSpec.decode("8dlu"));
    layout.insertRow(row + 2, RowSpec.decode("pref"));
    layout.insertRow(row + 3, RowSpec.decode("8dlu"));

    int[][] rowGroups = layout.getRowGroups();
    int[][] newRowGroups = new int[rowGroups.length + 1][];
    for (int i = 0; i < rowGroups.length; i++)
      newRowGroups[i] = rowGroups[i];
    newRowGroups[newRowGroups.length - 1] = new int[] {row, row + 2};
    layout.setRowGroups(newRowGroups);

    builder.addLabel("Flux attenuated:",
      cc.xyw(1, row, 3, "right, default"));
    JLabel attenuationValueLabel = ComponentFactory.createLabel(
      model.getModel(ImcaFilterRack.PROPNAME_ATTENUATION),
      new DecimalFormat("##0.0"));
    builder.add(attenuationValueLabel, cc.xy(5, row));
    NumberFormatter requestedAttenuationTextFieldFormatter =
      new NumberFormatter(new DoubleDecimalFormat("##0.0"));
    requestedAttenuationTextFieldFormatter.setMinimum(new Double(0.0));
    requestedAttenuationTextFieldFormatter.setMaximum(new Double(100.0));
    JTextField requestedAttenuationTextField =
      ComponentFactory.createFormattedTextField(
        model.getModel(ImcaFilterRack.PROPNAME_REQUESTED_ATTENUATION),
        requestedAttenuationTextFieldFormatter);
    builder.add(requestedAttenuationTextField, cc.xy(7, row));
    builder.addLabel("%", cc.xy(9, row));
    row += 2;

    builder.addLabel("Total thickness:", cc.xyw(1, row, 3, "right, default"));
    JLabel thicknessValueLabel = ComponentFactory.createLabel(
      model.getModel(ImcaFilterRack.PROPNAME_THICKNESS),
      new DecimalFormat("####0"));
    builder.add(thicknessValueLabel, cc.xy(5, row));
    NumberFormatter requestedThicknessTextFieldFormatter =
      new NumberFormatter(new DoubleDecimalFormat("####0"));
    requestedThicknessTextFieldFormatter.setMinimum(new Double(0.0));
    requestedThicknessTextFieldFormatter.setMaximum(new Double(
      model.getMaxFilterThickness()));
    JTextField requestedThicknessTextField =
      ComponentFactory.createFormattedTextField(
        model.getModel(ImcaFilterRack.PROPNAME_REQUESTED_THICKNESS),
        requestedThicknessTextFieldFormatter);
    builder.add(requestedThicknessTextField, cc.xy(7, row));
    builder.addLabel("\u03BCm", cc.xy(9, row));
    row += 2;

    return row;
  }

  @Override
  protected String[] getFilterDescriptions() {
    int[] filterThicknesses = model.getFilterThicknesses();
    String[] descriptions = new String[filterThicknesses.length];
    for (int i = 0; i < filterThicknesses.length; i++)
      descriptions[i] = "(" + filterThicknesses[i] + " \u03BCm" + ")";
    return descriptions;
  }
}
