package imcacat.imcabc;

import javax.swing.JComponent;
import javax.swing.JTabbedPane;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;

import imcacat.epicsca4j.ca.EpicsCaException;

/**
 * I am an <code>ImcabcFrame</code> for the 17-BM beamline.
 */
public class ImcabcFrame17bm extends ImcabcFrame {
  private static final long serialVersionUID = 1;

  public ImcabcFrame17bm() throws InterruptedException, EpicsCaException {
    this(new ImcabcPresentationModel17bm());
  }

  public ImcabcFrame17bm(ImcabcPresentationModel model)
      throws InterruptedException, EpicsCaException {
    super(model);
  }

  protected JComponent buildMainPanel()
  throws InterruptedException, EpicsCaException {
    JTabbedPane tabbedPane = new JTabbedPane(SwingConstants.TOP);
    tabbedPane.addTab("Beam", new BeamPanel17bm());
    tabbedPane.addTab("MAD", new MadPanel(
      new MadPresentationModel(new MadFacade(
        model.getFluorescenceScanBeanChannel()), model)));
    tabbedPane.addTab("Optimize", new OptimizePanel17bm(
      new OptimizePresentationModel17bm(new OptimizeFacade17bm(),
        model.getOptimizeTabOptimizationBeanChannel(), model)));
    tabbedPane.addTab("Staff", new StaffPanel(
      new StaffPresentationModel(new StaffFacade(),
        model.getStaffTabOptimizationBeanChannel(), model)));
    tabbedPane.setBorder(new EmptyBorder(10, 10, 10, 10));
    return tabbedPane;
  }

  protected Beamline getBeamline() {
    return Beamline.BM17;
  }

  protected String getHelpSetResourcePath() {
    return "resources/help17bm/jhelpset.hs";
  }
}
