package imcacat.imcabc;

import javax.swing.JComponent;
import javax.swing.JTabbedPane;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;

import imcacat.epicsca4j.ca.EpicsCaException;

/**
 * I am an <code>ImcabcFrame</code> for the 17-ID beamline.
 */
public class ImcabcFrame17id extends ImcabcFrame {
  private static final long serialVersionUID = 1;

  public ImcabcFrame17id() throws InterruptedException, EpicsCaException {
    this(new ImcabcPresentationModel17id());
  }

  public ImcabcFrame17id(ImcabcPresentationModel model)
      throws InterruptedException, EpicsCaException {
    super(model);
  }

  protected JComponent buildMainPanel()
  throws InterruptedException, EpicsCaException {
    JTabbedPane tabbedPane = new JTabbedPane(SwingConstants.TOP);
    tabbedPane.addTab("Beam", new BeamPanel17id());
    tabbedPane.addTab("MAD", new MadPanel(
      new MadPresentationModel(new MadFacade(
        model.getFluorescenceScanBeanChannel()), model)));
    tabbedPane.addTab("Optimize", new OptimizePanel17id(
      new OptimizePresentationModel17id(new OptimizeFacade17id(),
        model.getOptimizeTabOptimizationBeanChannel(), model)));
    tabbedPane.setBorder(new EmptyBorder(10, 10, 10, 10));
    return tabbedPane;
  }

  protected Beamline getBeamline() {
    return Beamline.ID17;
  }

  protected String getHelpSetResourcePath() {
    return "resources/help17id/jhelpset.hs";
  }
}
