package imcacat.imcabc;

import imcacat.epicsca4j.ca.EpicsCaException;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import com.jgoodies.binding.PresentationModel;
import com.jgoodies.binding.beans.PropertyAdapter;
import com.jgoodies.binding.value.ValueHolder;
import com.jgoodies.binding.value.ValueModel;

/**
 * I am the presentation model for {@link ImcabcFrame}.
 */
public abstract class ImcabcPresentationModel
extends PresentationModel<Imcabc> {
  private static final long serialVersionUID = 1;

  public static final String PROPNAME_FLUORESCENCE_SCAN_ENABLED =
    "fluorescenceScanEnabled";
  public static final String PROPNAME_OPTIMIZATION_ENABLED =
    "optimizationEnabled";
  public static final String PROPNAME_OPTIMIZATION_RUNNING =
    "optimizationRunning";

  protected Operation operation;
  protected boolean isFluorescenceScanEnabled;
  protected boolean isOptimizationEnabled;
  protected boolean isOptimizationRunning;

  protected ValueModel fluorescenceScanBeanChannel;
  protected ValueModel fluorescenceScanIsRunningModel;
  protected ValueModel optimizeTabOptimizationBeanChannel;
  protected ValueModel optimizeTabOptimizationIsRunningModel;
  protected ValueModel staffTabOptimizationBeanChannel;
  protected ValueModel staffTabOptimizationIsRunningModel;

  public ImcabcPresentationModel()
  throws InterruptedException, EpicsCaException {
    this(new Imcabc());
  }

  public ImcabcPresentationModel(Imcabc bean)
  throws InterruptedException, EpicsCaException {
    super(bean);

    operation = Operation.IDLE;
    isFluorescenceScanEnabled = true;
    isOptimizationEnabled = true;
    isOptimizationRunning = false;

    fluorescenceScanBeanChannel = new ValueHolder(createMadFluorescenceScan(),
      true);
    fluorescenceScanIsRunningModel = new PropertyAdapter<MadFluorescenceScan>(
      fluorescenceScanBeanChannel, MadFluorescenceScan.PROPNAME_RUNNING,
      true);
    fluorescenceScanIsRunningModel.addValueChangeListener(
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          fluorescenceScanIsRunningChanged((Boolean)event.getNewValue());
        }
      });

    optimizeTabOptimizationBeanChannel = new ValueHolder(
      new NullOptimization(), true);
    optimizeTabOptimizationIsRunningModel = new PropertyAdapter<Optimization>(
      optimizeTabOptimizationBeanChannel, Optimization.PROPNAME_RUNNING,
      true);
    optimizeTabOptimizationIsRunningModel.addValueChangeListener(
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          optimizationIsRunningChanged((Boolean)event.getNewValue());
        }
      });

    staffTabOptimizationBeanChannel = new ValueHolder(new NullOptimization(),
      true);
    staffTabOptimizationIsRunningModel = new PropertyAdapter<Optimization>(
      staffTabOptimizationBeanChannel, Optimization.PROPNAME_RUNNING,
      true);
    staffTabOptimizationIsRunningModel.addValueChangeListener(
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          optimizationIsRunningChanged((Boolean)event.getNewValue());
        }
      });
  }

  protected abstract MadFluorescenceScan createMadFluorescenceScan()
  throws InterruptedException, EpicsCaException;

  public ValueModel getFluorescenceScanBeanChannel() {
    return fluorescenceScanBeanChannel;
  }

  public ValueModel getOptimizeTabOptimizationBeanChannel() {
    return optimizeTabOptimizationBeanChannel;
  }

  public ValueModel getStaffTabOptimizationBeanChannel() {
    return staffTabOptimizationBeanChannel;
  }

  public boolean isFluorescenceScanEnabled() {
    return isFluorescenceScanEnabled;
  }
  public void setFluorescenceScanEnabled(boolean newValue) {
    boolean oldValue = isFluorescenceScanEnabled;
    isFluorescenceScanEnabled = newValue;
    if (oldValue != newValue)
      firePropertyChange(PROPNAME_FLUORESCENCE_SCAN_ENABLED,
        Boolean.valueOf(oldValue), Boolean.valueOf(newValue));
  }

  public boolean isOptimizationEnabled() {
    return isOptimizationEnabled;
  }
  public void setOptimizationEnabled(boolean newValue) {
    boolean oldValue = isOptimizationEnabled;
    isOptimizationEnabled = newValue;
    if (oldValue != newValue)
      firePropertyChange(PROPNAME_OPTIMIZATION_ENABLED,
        Boolean.valueOf(oldValue), Boolean.valueOf(newValue));
  }

  public boolean isOptimizationRunning() {
    return isOptimizationRunning;
  }
  public void setOptimizationRunning(boolean newValue) {
    boolean oldValue = isOptimizationRunning;
    isOptimizationRunning = newValue;
    if (oldValue != newValue)
      firePropertyChange(PROPNAME_OPTIMIZATION_RUNNING,
        Boolean.valueOf(oldValue), Boolean.valueOf(newValue));
  }

  protected void fluorescenceScanIsRunningChanged(Boolean newValue) {
    setOptimizationEnabled(!newValue.booleanValue());
    getBean().setOperation(newValue.booleanValue() ?
      Operation.FLUORESCENCE_SCAN : Operation.IDLE);
  }

  protected void optimizationIsRunningChanged(Boolean newValue) {
    setOptimizationRunning(newValue);
    setFluorescenceScanEnabled(!newValue.booleanValue());
    getBean().setOperation(newValue.booleanValue() ?
      Operation.OPTIMIZATION : Operation.IDLE);
  }
}
