package imcacat.imcabc;

import org.junit.Test;

import static org.junit.Assert.assertEquals;

/**
 * I test {@link LinearInterpolator}.
 */
public class LinearInterpolatorTest {
  @Test
  public void getX01() throws ValueOutOfRangeException {
    LinearInterpolator interpolator = new LinearInterpolator(
      new double[][] {
        { 0.0, 1.0 },
        { 0.0, 2.0 }});
    assertEquals(0.0, interpolator.getX(0.0), 0.001);
    assertEquals(1.0, interpolator.getX(2.0), 0.001);
    assertEquals(0.25, interpolator.getX(0.5), 0.001);
    assertEquals(1.0 / 6.0, interpolator.getX(1.0 / 3.0), 0.001);
  }

  @Test(expected=ValueOutOfRangeException.class)
  public void getX02() throws ValueOutOfRangeException {
    LinearInterpolator interpolator = new LinearInterpolator(
      new double[][] {
        { 0.0, 1.0 },
        { 0.0, 2.0 }});
    interpolator.getX(-1.0);
  }

  @Test(expected=ValueOutOfRangeException.class)
  public void getX03() throws ValueOutOfRangeException {
    LinearInterpolator interpolator = new LinearInterpolator(
      new double[][] {
        { 0.0, 1.0 },
        { 0.0, 2.0 }});
    interpolator.getX(3.0);
  }

  @Test(expected=ValueOutOfRangeException.class)
  public void getX04() throws ValueOutOfRangeException {
    LinearInterpolator interpolator = new LinearInterpolator(
      new double[][] {
        { 0.0 },
        { 0.0 }});
    interpolator.getX(0.0);
  }

  @Test(expected=ValueOutOfRangeException.class)
  public void getX05() throws ValueOutOfRangeException {
    LinearInterpolator interpolator = new LinearInterpolator(
      new double[][] {
        { },
        { }});
    interpolator.getX(0.0);
  }

  // descending y values
  @Test
  public void getX06() throws ValueOutOfRangeException {
    LinearInterpolator interpolator = new LinearInterpolator(
      new double[][] {
        {  2.0, 1.0, 0.0, -1.0, -2.0 },
        {  2.0, 1.0, 0.0, -1.0, -2.0 }});
    assertEquals(1.5, interpolator.getX(1.5), 0.001);
  }

  @Test
  public void getY01() throws ValueOutOfRangeException {
    LinearInterpolator interpolator = new LinearInterpolator(
      new double[][] {
        { 0.0, 1.0 },
        { 0.0, 2.0 }});
    assertEquals(0.0, interpolator.getY(0.0), 0.001);
    assertEquals(2.0, interpolator.getY(1.0), 0.001);
    assertEquals(1.0, interpolator.getY(0.5), 0.001);
    assertEquals(1.0 / 1.5, interpolator.getY(1.0 / 3.0), 0.001);
  }

  @Test(expected=ValueOutOfRangeException.class)
  public void getY02() throws ValueOutOfRangeException {
    LinearInterpolator interpolator = new LinearInterpolator(
      new double[][] {
        { 0.0, 1.0 },
        { 0.0, 2.0 }});
    interpolator.getY(-1.0);
  }

  @Test(expected=ValueOutOfRangeException.class)
  public void getY03() throws ValueOutOfRangeException {
    LinearInterpolator interpolator = new LinearInterpolator(
      new double[][] {
        { 0.0, 1.0 },
        { 0.0, 2.0 }});
    interpolator.getY(2.0);
  }

  @Test
  public void getY04() throws ValueOutOfRangeException {
    LinearInterpolator interpolator = new LinearInterpolator(
      new double[][] {
        { 0.0, 1.0, 2.0 },
        { 0.0, 1.0, 5.0 }});
    assertEquals(0.5, interpolator.getY(0.5), 0.001);
    assertEquals(3.0, interpolator.getY(1.5), 0.001);
  }

  // 0 slope
  @Test
  public void getY05() throws ValueOutOfRangeException {
    LinearInterpolator interpolator = new LinearInterpolator(
      new double[][] {
        { 0.0, 1.0, 2.0, 3.0 },
        { 0.0, 1.0, 5.0, 5.0 }});
    assertEquals(5.0, interpolator.getY(2.5), 0.001);
  }

  // negative slope
  @Test
  public void getY06() throws ValueOutOfRangeException {
    LinearInterpolator interpolator = new LinearInterpolator(
      new double[][] {
        { 0.0, 1.0, 2.0, 3.0, 4.0 },
        { 0.0, 1.0, 5.0, 5.0, 3.0 }});
    assertEquals(4.0, interpolator.getY(3.5), 0.001);
  }

  // negative values
  @Test
  public void getY07() throws ValueOutOfRangeException {
    LinearInterpolator interpolator = new LinearInterpolator(
      new double[][] {
        { -2.0, -1.0, 0.0, 1.0, 2.0 },
        {  1.0,  0.0, 1.0, 2.0, 3.0 }});
    assertEquals(0.5, interpolator.getY(-1.5), 0.001);
  }

  // descending x values
  @Test
  public void getY08() throws ValueOutOfRangeException {
    LinearInterpolator interpolator = new LinearInterpolator(
      new double[][] {
        {  2.0, 1.0, 0.0, -1.0, -2.0 },
        {  2.0, 1.0, 0.0, -1.0, -2.0 }});
    assertEquals(1.5, interpolator.getY(1.5), 0.001);
  }
}
