package imcacat.imcabc;

import imcacat.epicsca4j.util.StringUtilities;
import imcacat.epicsca4j.util.UiUtilities;

import java.awt.Component;
import java.awt.Cursor;

import javax.swing.JOptionPane;

import com.jgoodies.binding.value.ValueModel;

/**
 * I am an action that starts or stops the calculation of anomalous scattering
 * factors via {@link MadFacade}.  I display an error message if the
 * calculation failed.
 *
 * @see MadPresentationModel
 * @see MadFluorescenceScanDataProcessor
 */
public class MadCalculateFAndFDoublePrimeAction extends
    TwoStateValueModelButtonAction {
  private static final long serialVersionUID = 1;

  protected Component cursorChangeComponent;
  protected ValueModel calculationDoneReasonModel;

  public MadCalculateFAndFDoublePrimeAction(
      MadPresentationModel presentationModel) {
    this(UiUtilities.getRootFrame(), presentationModel);
  }

  public MadCalculateFAndFDoublePrimeAction(Component cursorChangeComponent,
      MadPresentationModel presentationModel) {
    this("Process", "Cancel", cursorChangeComponent,
      presentationModel);
  }

  public MadCalculateFAndFDoublePrimeAction(String state1Label,
      String state2Label, Component cursorChangeComponent,
      MadPresentationModel presentationModel) {
    super(state1Label, state2Label,
      MadFluorescenceScanDataProcessorCommand.CALCULATE,
      MadFluorescenceScanDataProcessorCommand.CANCEL_CALCULATE,
      Boolean.FALSE, Boolean.TRUE,
      presentationModel.getModel(
        MadFacade.PROPNAME_DATA_PROCESSOR_CALCULATING),
      presentationModel.getModel(
        MadFacade.PROPNAME_DATA_PROCESSOR_COMMAND),
      true, false);
    this.cursorChangeComponent = cursorChangeComponent;
    calculationDoneReasonModel = presentationModel.getModel(
      MadFacade.PROPNAME_DATA_PROCESSOR_CALCULATION_DONE_REASON);
    initialize();
  }

  @Override
  public void changeToState1() {
    super.changeToState1();
    cursorChangeComponent.setCursor(Cursor.getDefaultCursor());
    MadFluorescenceScanDataProcessorCalculationDoneReason reason =
      getCalculationDoneReason();
    if (MadFluorescenceScanDataProcessorCalculationDoneReason.SUCCESS.equals(
        reason) ||
        MadFluorescenceScanDataProcessorCalculationDoneReason.CANCELED.equals(
        reason) ||
        MadFluorescenceScanDataProcessorCalculationDoneReason.NONE.equals(
          reason))
      return;

    if (MadFluorescenceScanDataProcessorCalculationDoneReason.
        NON_ZERO_EXIT_VALUE.equals(reason)) {
      JOptionPane.showMessageDialog(cursorChangeComponent,
        "Failed to calculate anomalous scattering factors.\n\n" +
        StringUtilities.wordWrap(
          "Chooch, the program invoked to calculate the f' and f'' values, " +
          "exited abnormally.  This could be due to a problem with the " +
          "Chooch program itself (e.g. a required shared library could not " +
          "be found), or it could be due to noisy fluorescence scan " +
          "data.  If the fluorescence scan data is noisy, you could try a " +
          "coarser scan type.", 78, "\n"),
        ImcabcFrame.FRAME_TITLE, JOptionPane.ERROR_MESSAGE);
    } else if (MadFluorescenceScanDataProcessorCalculationDoneReason.
        LOAD_DATA_FILE_IO_EXCEPTION.equals(reason)) {
      JOptionPane.showMessageDialog(cursorChangeComponent,
        "Failed to read scan data file.\n\n" +
        StringUtilities.wordWrap(
          "An error occurred while trying to read the scan data file.  " +
          "Check the file system permissions to ensure the file is " +
          "readable.", 78, "\n"),
        ImcabcFrame.FRAME_TITLE, JOptionPane.ERROR_MESSAGE);
    } else if (MadFluorescenceScanDataProcessorCalculationDoneReason.
        LOAD_DATA_FILE_PARSE_EXCEPTION.equals(reason)) {
      JOptionPane.showMessageDialog(cursorChangeComponent,
        "Could not recognize scan data file contents.\n\n" +
        StringUtilities.wordWrap(
          "The contents of the scan data file were unrecognized.  The file " +
          "may be empty if the scan that created it was aborted.  Or the " +
          "file may be formatted incorrectly if it was edited by hand.", 78,
          "\n"),
        ImcabcFrame.FRAME_TITLE, JOptionPane.ERROR_MESSAGE);
    } else if (MadFluorescenceScanDataProcessorCalculationDoneReason.
        CREATE_CHOOCH_INPUT_FILE_IO_EXCEPTION.equals(reason)) {
      JOptionPane.showMessageDialog(cursorChangeComponent,
        "Failed to create chooch input file.\n\n" +
        StringUtilities.wordWrap(
          "An error occurred while trying to create the chooch input " +
          "file.  Check the file system permissions to ensure the file can " +
          "be created in the same location as the scan data file, and " +
          "ensure the disk is not full.", 78, "\n"),
        ImcabcFrame.FRAME_TITLE, JOptionPane.ERROR_MESSAGE);
    } else if (MadFluorescenceScanDataProcessorCalculationDoneReason.
        READ_CHOOCH_OUTPUT_FILE_IO_EXCEPTION.equals(reason)) {
      JOptionPane.showMessageDialog(cursorChangeComponent,
        "Failed to read chooch output file.\n\n" +
        StringUtilities.wordWrap(
          "An error occurred while trying to read the chooch output file.  " +
          "Check the file system permissions to ensure the file is " +
          "readable.", 78, "\n"),
        ImcabcFrame.FRAME_TITLE, JOptionPane.ERROR_MESSAGE);
    } else if (MadFluorescenceScanDataProcessorCalculationDoneReason.
        READ_CHOOCH_OUTPUT_FILE_PARSE_EXCEPTION.equals(reason)) {
      JOptionPane.showMessageDialog(cursorChangeComponent,
        "Could not recognize chooch output file contents.\n\n" +
        StringUtilities.wordWrap(
          "The contents of the chooch output file were unrecognized.  It " +
          "is possible the version of chooch installed on this computer " +
          "has an output file format different than expected.  The output " +
          "file format is expect to match that of chooch version 5.0.2.  " +
          "Check the version of chooch installed to ensure compatibility.",
          78, "\n"),
        ImcabcFrame.FRAME_TITLE, JOptionPane.ERROR_MESSAGE);
    } else {
      JOptionPane.showMessageDialog(cursorChangeComponent,
        "Failed to calculate anomalous scattering factors.\n\n" +
        StringUtilities.wordWrap(
          "An error occurred while calculating the f' and f'' values.  It " +
          "is possible that the chooch program could not be found on " +
          "the command search path.  If this is not the problem, examine " +
          "the application console output or log for more details about " +
          "the error.", 78, "\n"),
        ImcabcFrame.FRAME_TITLE, JOptionPane.ERROR_MESSAGE);
    }
  }

  @Override
  public void changeToState2() {
    super.changeToState2();
    cursorChangeComponent.setCursor(
      Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
  }

  protected MadFluorescenceScanDataProcessorCalculationDoneReason
  getCalculationDoneReason() {
    return (MadFluorescenceScanDataProcessorCalculationDoneReason)
      calculationDoneReasonModel.getValue();
  }
}
