package imcacat.imcabc;

/**
 * I represent element edges in the context of a MAD experiment.
 * <p>
 * My low and high channel values are specific to the R&Ouml;NTEC XFlash
 * detector and were obtained from Kevin Battaile (2005-02-10). My channels
 * are specified in the range [0-4095].
 * <p>
 * My energy values were taken from the on-line version of the
 * <a href="http://xdb.lbl.gov/Section1/Sec_1-1.html">X-Ray Data Booklet,
 * section 1.1, Electron Binding Energies</a> (2007-05-16).
 *
 * @see MadElement
 */
public enum MadEdge {
  FE_K  (MadElement.FE, 1362, 1392,  7112.0),
  CO_K  (MadElement.CO, 1466, 1496,  7709.0),
  NI_K  (MadElement.NI, 1575, 1605,  8333.0),
  CU_K  (MadElement.CU, 1688, 1718,  8979.0),
  ZN_K  (MadElement.ZN, 1804, 1834,  9659.0),
  GA_K  (MadElement.GA, 1932, 1962, 10367.0),
  GE_K  (MadElement.GE, 2059, 2089, 11103.0),
  AS_K  (MadElement.AS, 2190, 2220, 11867.0),
  SE_K  (MadElement.SE, 2326, 2356, 12658.0),
  BR_K  (MadElement.BR, 2466, 2496, 13474.0),
  KR_K  (MadElement.KR, 2611, 2641, 14326.0),
  SM_L3 (MadElement.SM, 1209, 1239,  6716.0),
  EU_L2 (MadElement.EU, 1373, 1403,  7617.0),
  GD_L2 (MadElement.GD, 1424, 1454,  7930.0),
  TB_L2 (MadElement.TB, 1476, 1506,  8252.0),
  TB_L3 (MadElement.TB, 1336, 1366,  7514.0),
  DY_L2 (MadElement.DY, 1531, 1561,  8581.0),
  DY_L3 (MadElement.DY, 1380, 1410,  7790.0),
  HO_L2 (MadElement.HO, 1586, 1616,  8918.0),
  HO_L3 (MadElement.HO, 1425, 1455,  8071.0),
  ER_L2 (MadElement.ER, 1643, 1673,  9264.0),
  ER_L3 (MadElement.ER, 1441, 1471,  8358.0),
  TM_L2 (MadElement.TM, 1702, 1732,  9617.0),
  TM_L3 (MadElement.TM, 1517, 1547,  8648.0),
  YB_L2 (MadElement.YB, 1762, 1792,  9978.0),
  YB_L3 (MadElement.YB, 1564, 1594,  8944.0),
  LU_L2 (MadElement.LU, 1823, 1853, 10349.0),
  LU_L3 (MadElement.LU, 1612, 1642,  9244.0),
  HF_L2 (MadElement.HF, 1886, 1916, 10739.0),
  HF_L3 (MadElement.HF, 1661, 1691,  9561.0),
  TA_L2 (MadElement.TA, 1950, 1980, 11136.0),
  TA_L3 (MadElement.TA, 1711, 1741,  9881.0),
  W_L2  (MadElement.W,  2015, 2045, 11544.0),
  W_L3  (MadElement.W,  1761, 1791, 10207.0),
  RE_L2 (MadElement.RE, 2083, 2113, 11959.0),
  RE_L3 (MadElement.RE, 1812, 1842, 10535.0),
  OS_L2 (MadElement.OS, 2152, 2182, 12385.0),
  OS_L3 (MadElement.OS, 1864, 1894, 10871.0),
  IR_L2 (MadElement.IR, 2223, 2253, 12824.0),
  IR_L3 (MadElement.IR, 1916, 1946, 11215.0),
  PT_L2 (MadElement.PT, 2295, 2325, 13273.0),
  PT_L3 (MadElement.PT, 1969, 1999, 11564.0),
  AU_L2 (MadElement.AU, 2370, 2400, 13734.0),
  AU_L3 (MadElement.AU, 2022, 2052, 11919.0),
  HG_L2 (MadElement.HG, 2445, 2475, 14209.0),
  HG_L3 (MadElement.HG, 2079, 2109, 12284.0),
  TL_L3 (MadElement.TL, 2135, 2165, 12658.0),
  PB_L2 (MadElement.PB, 2604, 2634, 15200.0),
  PB_L3 (MadElement.PB, 2192, 2222, 13035.0),
  BI_L3 (MadElement.BI, 2249, 2279, 13419.0);

  protected MadElement element;
  protected int lowChannel;
  protected int highChannel;
  protected double energy;

  MadEdge(MadElement element, int lowChannel, int highChannel,
      double energy) {
    this.element = element;
    this.lowChannel = lowChannel;
    this.highChannel = highChannel;
    this.energy = energy;
  }

  public MadElement getElement() {
    return element;
  }

  public int getLowChannel() {
    return lowChannel;
  }

  public int getHighChannel() {
    return highChannel;
  }

  public double getEnergy() {
    return energy;
  }
}
