package imcacat.imcabc;

import java.util.ArrayList;
import java.util.EnumMap;
import java.util.List;

/**
 *  I represent elements in the context of a MAD experiment.
 *
 *  @see MadEdge
 */
public enum MadElement {
  FE,
  CO,
  NI,
  CU,
  ZN,
  GA,
  GE,
  AS,
  SE,
  BR,
  KR,
  SM,
  EU,
  GD,
  TB,
  DY,
  HO,
  ER,
  TM,
  YB,
  LU,
  HF,
  TA,
  W,
  RE,
  OS,
  IR,
  PT,
  AU,
  HG,
  TL,
  PB,
  BI;

  protected static EnumMap<MadElement,List<MadEdge>> ELEMENT_TO_EDGE_MAP;

  public synchronized List<MadEdge> getEdges() {
    if (ELEMENT_TO_EDGE_MAP == null) initializeElementToEdgeMap();
    return ELEMENT_TO_EDGE_MAP.get(this);
  }

  protected static void initializeElementToEdgeMap() {
    ELEMENT_TO_EDGE_MAP = new EnumMap<MadElement,List<MadEdge>>(
      MadElement.class);
    for (MadElement eachElement : MadElement.values()) {
      List<MadEdge> edges = ELEMENT_TO_EDGE_MAP.get(eachElement);
      if (edges == null) {
        edges = new ArrayList<MadEdge>();
        ELEMENT_TO_EDGE_MAP.put(eachElement, edges);
      }
      for (MadEdge eachEdge : MadEdge.values())
        if (eachEdge.getElement() == eachElement) edges.add(eachEdge);
    }
  }
}
