package imcacat.imcabc;

import imcacat.epicsca4j.util.StringUtilities;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.nio.charset.Charset;
import java.util.Locale;
import java.util.regex.Pattern;

/**
 * I write experiment parameters from a
 * {@link MadFluorescenceScanDataProcessor} in a simple field-value text
 * format.
 */
public class MadExperimentParametersWriter extends BufferedWriter {
  protected static final Pattern EDGE_SPLIT_PATTERN = Pattern.compile("_");
  protected static final String CHARACTER_SET_NAME = "US-ASCII";

  public MadExperimentParametersWriter(File file) throws IOException {
    this(new FileOutputStream(file));
  }

  public MadExperimentParametersWriter(OutputStream out) throws IOException {
    super(new OutputStreamWriter(out, Charset.forName(CHARACTER_SET_NAME)));
  }

  public void write(MadFluorescenceScanDataProcessor processor)
  throws IOException {
    writeEdge(processor.getEdge());
    writeNewLine();
    writePeakEdgeLowHighValues(processor);
  }

  protected void writeEdge(MadEdge edge) throws IOException {
    write("Edge: ");
    if (edge == null) {
      write(String.format((Locale)null, "unknown%n"));
    } else {
      write(String.format((Locale)null, "%s %s %.1f eV%n",
        StringUtilities.toTitleCase(
          edge.getElement().toString()),
          StringUtilities.toTitleCase(EDGE_SPLIT_PATTERN.split(
            edge.toString())[1]), edge.getEnergy()));
    }
  }

  protected void writePeakEdgeLowHighValues(
      MadFluorescenceScanDataProcessor processor) throws IOException {
    writeEnergyWavelengthFandFDoublePrime("Peak",
      processor.getPeakEnergy(),
      processor.getPeakWavelength(),
      processor.getPeakFPrime(),
      processor.getPeakFDoublePrime());
    writeNewLine();
    writeEnergyWavelengthFandFDoublePrime("Edge",
      processor.getEdgeEnergy(),
      processor.getEdgeWavelength(),
      processor.getEdgeFPrime(),
      processor.getEdgeFDoublePrime());
    writeNewLine();
    writeEnergyWavelengthFandFDoublePrime("Low",
      processor.getLowEnergy(),
      processor.getLowWavelength(),
      processor.getLowFPrime(),
      processor.getLowFDoublePrime());
    writeNewLine();
    writeEnergyWavelengthFandFDoublePrime("High",
      processor.getHighEnergy(),
      processor.getHighWavelength(),
      processor.getHighFPrime(),
      processor.getHighFDoublePrime());
  }

  protected void writeEnergyWavelengthFandFDoublePrime(String fieldNamePrefix,
      Double energy, Double wavelength, Double fPrime, Double fDoublePrime)
  throws IOException {
    write(fieldNamePrefix + "Energy: ");
    if (energy == null) writeNewLine();
    else write(String.format((Locale)null, "%.3f%n", energy));

    write(fieldNamePrefix + "Wavelength: ");
    if (wavelength == null) writeNewLine();
    else write(String.format((Locale)null, "%.7f%n", wavelength));

    write(fieldNamePrefix + "FPrime: ");
    if (fPrime == null) writeNewLine();
    else write(String.format((Locale)null, "%.2f%n", fPrime));

    write(fieldNamePrefix + "FDoublePrime: ");
    if (fDoublePrime == null) writeNewLine();
    else write(String.format((Locale)null, "%.2f%n", fDoublePrime));
  }

  protected void writeNewLine() throws IOException {
    write(Resources.getLineSeparator());
  }
}
