package imcacat.imcabc;

import imcacat.epicsca4j.bean.OneWayPropertyChangeMediator;
import imcacat.epicsca4j.bean.ExternallyFireablePropertyChangeBean;
import imcacat.epicsca4j.ca.EpicsCaException;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.ParseException;
import java.util.HashMap;
import java.util.Map;

import com.jgoodies.binding.beans.Model;
import com.jgoodies.binding.value.ValueModel;

/**
 * I am a Facade providing an easy to use interface for conducting a MAD
 * experiment.
 */
public class MadFacade extends Model
implements ExternallyFireablePropertyChangeBean {
  private static final long serialVersionUID = 1;

  public static final String PROPNAME_ELEMENT = "element";
  public static final String PROPNAME_EDGE = "edge";
  public static final String PROPNAME_SCAN_TYPE = "scanType";
  public static final String PROPNAME_SCAN_OUTPUT_DIRECTORY =
    "scanOutputDirectory";
  public static final String PROPNAME_FLUORESCENCE_SCAN = "fluorescenceScan";
  public static final String PROPNAME_SCAN_RUNNING = "scanRunning";
  public static final String PROPNAME_SCAN_STATUS = "scanStatus";
  public static final String PROPNAME_SCAN_PROGRESS = "scanProgress";
  public static final String PROPNAME_SCAN_COMMAND = "scanCommand";

  public static final String PROPNAME_DATA_FILE = "dataFile";
  public static final String PROPNAME_DATA_PROCESSOR_COMMAND =
    "dataProcessorCommand";
  public static final String PROPNAME_DATA_PROCESSOR_CALCULATING =
    "dataProcessorCalculating";
  public static final String PROPNAME_DATA_PROCESSOR_CALCULATION_DONE_REASON =
    "dataProcessorCalculationDoneReason";

  public static final String PROPNAME_PEAK_ENERGY = "peakEnergy";
  public static final String PROPNAME_PEAK_WAVELENGTH = "peakWavelength";
  public static final String PROPNAME_PEAK_F_PRIME = "peakFPrime";
  public static final String PROPNAME_PEAK_F_DOUBLE_PRIME =
    "peakFDoublePrime";
  public static final String PROPNAME_EDGE_ENERGY = "edgeEnergy";
  public static final String PROPNAME_EDGE_WAVELENGTH = "edgeWavelength";
  public static final String PROPNAME_EDGE_F_PRIME = "edgeFPrime";
  public static final String PROPNAME_EDGE_F_DOUBLE_PRIME =
    "edgeFDoublePrime";
  public static final String PROPNAME_LOW_ENERGY = "lowEnergy";
  public static final String PROPNAME_LOW_WAVELENGTH = "lowWavelength";
  public static final String PROPNAME_LOW_F_PRIME = "lowFPrime";
  public static final String PROPNAME_LOW_F_DOUBLE_PRIME =
    "lowFDoublePrime";
  public static final String PROPNAME_HIGH_ENERGY = "highEnergy";
  public static final String PROPNAME_HIGH_WAVELENGTH = "highWavelength";
  public static final String PROPNAME_HIGH_F_PRIME = "highFPrime";
  public static final String PROPNAME_HIGH_F_DOUBLE_PRIME =
    "highFDoublePrime";

  protected MadFluorescenceScan fluorescenceScan;
  protected OneWayPropertyChangeMediator fluorescenceScanMediator;

  protected MadFluorescenceScanDataProcessor dataProcessor;
  protected OneWayPropertyChangeMediator dataProcessorMediator;

  /**
   * XXX Fix me. Right now, the fluorescenceScanBeanChannel is not actually
   * supported. The MadFluorescenceScan instance is taken out of the channel
   * and used for the life of this instance. Changing the channel value has
   * no affect. This bean channel was added to support the mutually exclusive
   * "operation" behavior. Since the MAD model, presentation model, and view
   * were not written with bean channel support, I just take the instance out
   * of the channel and ignore the channel from then on.
   */
  public MadFacade(ValueModel fluorescenceScanBeanChannel)
  throws EpicsCaException {
    super();

    fluorescenceScan =
      (MadFluorescenceScan)fluorescenceScanBeanChannel.getValue();

    initialize();
  }

  protected void initialize() throws EpicsCaException {
    initializeSectionOne();
    initializeSectionTwoAndThree();
  }

  protected void initializeSectionOne()
  throws EpicsCaException {
    Map<String,String> mediatorPropertyMap = new HashMap<String,String>();
    mediatorPropertyMap.put(MadFluorescenceScan.PROPNAME_ELEMENT,
      MadFacade.PROPNAME_ELEMENT);
    mediatorPropertyMap.put(MadFluorescenceScan.PROPNAME_EDGE,
      MadFacade.PROPNAME_EDGE);
    mediatorPropertyMap.put(MadFluorescenceScan.PROPNAME_SCAN_TYPE,
      MadFacade.PROPNAME_SCAN_TYPE);
    mediatorPropertyMap.put(MadFluorescenceScan.PROPNAME_OUTPUT_DIRECTORY,
      MadFacade.PROPNAME_SCAN_OUTPUT_DIRECTORY);
    mediatorPropertyMap.put(MadFluorescenceScan.PROPNAME_RUNNING,
      MadFacade.PROPNAME_SCAN_RUNNING);
    mediatorPropertyMap.put(MadFluorescenceScan.PROPNAME_STATUS,
      MadFacade.PROPNAME_SCAN_STATUS);
    mediatorPropertyMap.put(MadFluorescenceScan.PROPNAME_PROGRESS,
      MadFacade.PROPNAME_SCAN_PROGRESS);
    mediatorPropertyMap.put(MadFluorescenceScan.PROPNAME_COMMAND,
      MadFacade.PROPNAME_SCAN_COMMAND);
    fluorescenceScanMediator = new OneWayPropertyChangeMediator(
      fluorescenceScan, this, mediatorPropertyMap);
  }

  protected void initializeSectionTwoAndThree() {
    dataProcessor = new MadFluorescenceScanDataProcessor();
    Map<String,String> mediatorPropertyMap = new HashMap<String,String>();
    mediatorPropertyMap.put(
      MadFluorescenceScanDataProcessor.PROPNAME_DATA_FILE,
      MadFacade.PROPNAME_DATA_FILE);
    mediatorPropertyMap.put(
      MadFluorescenceScanDataProcessor.PROPNAME_CALCULATING,
      MadFacade.PROPNAME_DATA_PROCESSOR_CALCULATING);
    mediatorPropertyMap.put(
      MadFluorescenceScanDataProcessor.PROPNAME_CALCULATION_DONE_REASON,
      MadFacade.PROPNAME_DATA_PROCESSOR_CALCULATION_DONE_REASON);
    mediatorPropertyMap.put(
      MadFluorescenceScanDataProcessor.PROPNAME_PEAK_ENERGY,
      MadFacade.PROPNAME_PEAK_ENERGY);
    mediatorPropertyMap.put(
      MadFluorescenceScanDataProcessor.PROPNAME_PEAK_WAVELENGTH,
      MadFacade.PROPNAME_PEAK_WAVELENGTH);
    mediatorPropertyMap.put(
      MadFluorescenceScanDataProcessor.PROPNAME_PEAK_F_PRIME,
      MadFacade.PROPNAME_PEAK_F_PRIME);
    mediatorPropertyMap.put(
      MadFluorescenceScanDataProcessor.PROPNAME_PEAK_F_DOUBLE_PRIME,
      MadFacade.PROPNAME_PEAK_F_DOUBLE_PRIME);
    mediatorPropertyMap.put(
      MadFluorescenceScanDataProcessor.PROPNAME_EDGE_ENERGY,
      MadFacade.PROPNAME_EDGE_ENERGY);
    mediatorPropertyMap.put(
      MadFluorescenceScanDataProcessor.PROPNAME_EDGE_WAVELENGTH,
      MadFacade.PROPNAME_EDGE_WAVELENGTH);
    mediatorPropertyMap.put(
      MadFluorescenceScanDataProcessor.PROPNAME_EDGE_F_PRIME,
      MadFacade.PROPNAME_EDGE_F_PRIME);
    mediatorPropertyMap.put(
      MadFluorescenceScanDataProcessor.PROPNAME_EDGE_F_DOUBLE_PRIME,
      MadFacade.PROPNAME_EDGE_F_DOUBLE_PRIME);
    mediatorPropertyMap.put(
      MadFluorescenceScanDataProcessor.PROPNAME_LOW_ENERGY,
      MadFacade.PROPNAME_LOW_ENERGY);
    mediatorPropertyMap.put(
      MadFluorescenceScanDataProcessor.PROPNAME_LOW_WAVELENGTH,
      MadFacade.PROPNAME_LOW_WAVELENGTH);
    mediatorPropertyMap.put(
      MadFluorescenceScanDataProcessor.PROPNAME_LOW_F_PRIME,
      MadFacade.PROPNAME_LOW_F_PRIME);
    mediatorPropertyMap.put(
      MadFluorescenceScanDataProcessor.PROPNAME_LOW_F_DOUBLE_PRIME,
      MadFacade.PROPNAME_LOW_F_DOUBLE_PRIME);
    mediatorPropertyMap.put(
      MadFluorescenceScanDataProcessor.PROPNAME_HIGH_ENERGY,
      MadFacade.PROPNAME_HIGH_ENERGY);
    mediatorPropertyMap.put(
      MadFluorescenceScanDataProcessor.PROPNAME_HIGH_WAVELENGTH,
      MadFacade.PROPNAME_HIGH_WAVELENGTH);
    mediatorPropertyMap.put(
      MadFluorescenceScanDataProcessor.PROPNAME_HIGH_F_PRIME,
      MadFacade.PROPNAME_HIGH_F_PRIME);
    mediatorPropertyMap.put(
      MadFluorescenceScanDataProcessor.PROPNAME_HIGH_F_DOUBLE_PRIME,
      MadFacade.PROPNAME_HIGH_F_DOUBLE_PRIME);
    dataProcessorMediator = new OneWayPropertyChangeMediator(
      dataProcessor, this, mediatorPropertyMap);

    fluorescenceScan.addPropertyChangeListener(
      MadFluorescenceScan.PROPNAME_OUTPUT_FILE,
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          dataProcessor.setDataFile((File)event.getNewValue());
        }
      });
  }

  public MadElement getElement() {
    return fluorescenceScan.getElement();
  }

  public void setElement(MadElement element) {
    fluorescenceScan.setElement(element);
  }

  public MadEdge getEdge() {
    return fluorescenceScan.getEdge();
  }

  public void setEdge(MadEdge edge) {
    fluorescenceScan.setEdge(edge);
  }

  public MadScanType getScanType() {
    return fluorescenceScan.getScanType();
  }

  public void setScanType(MadScanType scanType) {
    fluorescenceScan.setScanType(scanType);
  }

  public File getScanOutputDirectory() {
    return fluorescenceScan.getOutputDirectory();
  }

  public void setScanOutputDirectory(File directory) {
    fluorescenceScan.setOutputDirectory(directory);
  }

  public boolean isScanRunning()
  throws InterruptedException, EpicsCaException {
    return fluorescenceScan.isRunning();
  }

  public String getScanStatus()
  throws InterruptedException, EpicsCaException {
    return fluorescenceScan.getStatus();
  }

  public int getScanProgress() {
    return fluorescenceScan.getProgress();
  }

  public void setScanCommand(MadFluorescenceScanCommand command)
  throws InterruptedException, EpicsCaException, IOException {
    fluorescenceScan.setCommand(command);
  }

  public MadFluorescenceScan getFluorescenceScan() {
    return fluorescenceScan;
  }

  public File getDataFile() {
    return dataProcessor.getDataFile();
  }

  public void setDataFile(File newValue) {
    dataProcessor.setDataFile(newValue);
  }

  public boolean isDataProcessorCalculating() {
    return dataProcessor.isCalculating();
  }

  public MadFluorescenceScanDataProcessorCalculationDoneReason
  getDataProcessorCalculationDoneReason() {
    return dataProcessor.getCalculationDoneReason();
  }

  public void setDataProcessorCommand(
      MadFluorescenceScanDataProcessorCommand command)
  throws IOException, ParseException,
      MadFluorescenceScanDataProcessorException {
    dataProcessor.setCommand(command);
  }

  public Double getPeakEnergy() {
    return dataProcessor.getPeakEnergy();
  }

  public Double getPeakWavelength() {
    return dataProcessor.getPeakWavelength();
  }

  public Double getPeakFPrime() {
    return dataProcessor.getPeakFPrime();
  }

  public Double getPeakFDoublePrime() {
    return dataProcessor.getPeakFDoublePrime();
  }

  public Double getEdgeEnergy() {
    return dataProcessor.getEdgeEnergy();
  }

  public Double getEdgeWavelength() {
    return dataProcessor.getEdgeWavelength();
  }

  public Double getEdgeFPrime() {
    return dataProcessor.getEdgeFPrime();
  }

  public Double getEdgeFDoublePrime() {
    return dataProcessor.getEdgeFDoublePrime();
  }

  public Double getLowEnergy() {
    return dataProcessor.getLowEnergy();
  }

  public void setLowEnergy(Double newValue) {
    dataProcessor.setLowEnergy(newValue);
  }

  public Double getLowWavelength() {
    return dataProcessor.getLowWavelength();
  }

  public void setLowWavelength(Double newValue) {
    dataProcessor.setLowWavelength(newValue);
  }

  public Double getLowFPrime() {
    return dataProcessor.getLowFPrime();
  }

  public Double getLowFDoublePrime() {
    return dataProcessor.getLowFDoublePrime();
  }

  public Double getHighEnergy() {
    return dataProcessor.getHighEnergy();
  }

  public void setHighEnergy(Double newValue) {
    dataProcessor.setHighEnergy(newValue);
  }

  public Double getHighWavelength() {
    return dataProcessor.getHighWavelength();
  }

  public void setHighWavelength(Double newValue) {
    dataProcessor.setHighWavelength(newValue);
  }

  public Double getHighFPrime() {
    return dataProcessor.getHighFPrime();
  }

  public Double getHighFDoublePrime() {
    return dataProcessor.getHighFDoublePrime();
  }

  public void externallyFirePropertyChange(PropertyChangeEvent event) {
    firePropertyChange(event);
  }

  public File getExperimentParametersFile() {
    return dataProcessor.getExperimentParametersFile();
  }

  public void writeExperimentParameters(File destination)
  throws FileNotFoundException, IOException {
    dataProcessor.writeExperimentParameters(destination);
  }
}
