package imcacat.imcabc;

import imcacat.epicsca4j.bean.OneWayPropertyChangeMediator;
import imcacat.epicsca4j.bean.ExternallyFireablePropertyChangeBean;
import imcacat.epicsca4j.ca.EpicsCaException;
import imcacat.epicsca4j.util.ObjectUtilities;

import java.beans.PropertyChangeEvent;
import java.io.File;
import java.io.IOException;

import org.jfree.data.xy.XYDataset;

import com.jgoodies.binding.beans.BeanAdapter;
import com.jgoodies.binding.beans.Model;
import com.jgoodies.binding.value.ValueHolder;
import com.jgoodies.binding.value.ValueModel;

/**
 * I am a fluorescence scan for a MAD experiment.
 */
public abstract class MadFluorescenceScan extends Model
implements ExternallyFireablePropertyChangeBean {
  private static final long serialVersionUID = 1;

  public static final String PROPNAME_ELEMENT = "element";
  public static final String PROPNAME_EDGE = "edge";
  public static final String PROPNAME_SCAN_TYPE = "scanType";
  public static final String PROPNAME_OUTPUT_DIRECTORY = "outputDirectory";
  public static final String PROPNAME_OUTPUT_FILE = "dataOutputFile";
  public static final String PROPNAME_SHOULD_SAVE_DATA_TO_FILE =
    "shouldSaveDataToFile";
  public static final String PROPNAME_RUNNING = "running";
  public static final String PROPNAME_STATUS = "status";
  public static final String PROPNAME_PROGRESS = "progress";
  public static final String PROPNAME_COMMAND = "command";
  public static final String PROPNAME_XY_DATASET = "XYDataset";

  protected MadElement element;
  protected MadEdge edge;
  protected MadScanType scanType;
  protected File outputDirectory;
  protected File outputFile;
  protected boolean shouldSaveDataToFile;
  protected ValueModel scanRunnerChannel;
  protected BeanAdapter<MadFluorescenceScanRunner> scanRunnerBeanAdapter;
  protected OneWayPropertyChangeMediator scanRunnerRunningMediator;
  protected OneWayPropertyChangeMediator scanRunnerStatusMediator;
  protected OneWayPropertyChangeMediator scanRunnerProgressMediator;
  protected OneWayPropertyChangeMediator scanRunnerXyDatasetMediator;
  protected OneWayPropertyChangeMediator scanRunnerDataOutputFileMediator;

  public MadFluorescenceScan()
  throws InterruptedException, EpicsCaException {
    this(MadElement.SE, MadEdge.SE_K);
  }

  public MadFluorescenceScan(MadElement element, MadEdge edge)
  throws InterruptedException, EpicsCaException {
    this(element, edge, MadScanType.STANDARD);
  }

  public MadFluorescenceScan(MadElement element, MadEdge edge,
      MadScanType scanType)
  throws InterruptedException, EpicsCaException {
    this(element, edge, scanType, Resources.getUserHomeDirectory());
  }

  public MadFluorescenceScan(MadElement element, MadEdge edge,
      MadScanType scanType, File outputDirectory)
  throws InterruptedException, EpicsCaException {
    super();

    setElement(element);
    setEdge(edge);
    setScanType(scanType);
    setOutputDirectory(outputDirectory);

    shouldSaveDataToFile = true;

    scanRunnerChannel = new ValueHolder(createMadFluorescenceScanRunner(),
      true);
    scanRunnerBeanAdapter = new BeanAdapter<MadFluorescenceScanRunner>(
      scanRunnerChannel, true);
    scanRunnerRunningMediator = new OneWayPropertyChangeMediator(
      scanRunnerBeanAdapter.getValueModel(
        MadFluorescenceScanRunner.PROPNAME_RUNNING), this,
        ValueModel.PROPERTY_VALUE, PROPNAME_RUNNING);
    scanRunnerStatusMediator = new OneWayPropertyChangeMediator(
      scanRunnerBeanAdapter.getValueModel(
        MadFluorescenceScanRunner.PROPNAME_STATUS), this,
        ValueModel.PROPERTY_VALUE, PROPNAME_STATUS);
    scanRunnerProgressMediator = new OneWayPropertyChangeMediator(
      scanRunnerBeanAdapter.getValueModel(
        MadFluorescenceScanRunner.PROPNAME_PROGRESS), this,
        ValueModel.PROPERTY_VALUE, PROPNAME_PROGRESS);
    scanRunnerXyDatasetMediator = new OneWayPropertyChangeMediator(
      scanRunnerBeanAdapter.getValueModel(
        MadFluorescenceScanRunner.PROPNAME_XY_DATASET), this,
        ValueModel.PROPERTY_VALUE, PROPNAME_XY_DATASET);
    scanRunnerDataOutputFileMediator = new OneWayPropertyChangeMediator(
      scanRunnerBeanAdapter.getValueModel(
        MadFluorescenceScanRunner.PROPNAME_OUTPUT_FILE), this,
        ValueModel.PROPERTY_VALUE, PROPNAME_OUTPUT_FILE);
  }

  protected abstract MadFluorescenceScanRunner
    createMadFluorescenceScanRunner()
  throws InterruptedException, EpicsCaException;

  protected abstract MadFluorescenceScanRunner
  createMadFluorescenceScanRunner(MadEdge edge, MadScanType scanType,
    File outputDirectory)
  throws InterruptedException, EpicsCaException;

  protected MadFluorescenceScanRunner getScanRunner() {
    return (MadFluorescenceScanRunner)scanRunnerChannel.getValue();
  }

  public boolean isRunning() {
    return getScanRunner().isRunning();
  }

  public String getStatus() {
    return getScanRunner().getStatus();
  }

  public int getProgress() {
    return getScanRunner().getProgress();
  }

  public void setElement(MadElement newValue) {
    MadElement oldValue = element;
    element = newValue;
    if (oldValue != newValue)
      firePropertyChange(PROPNAME_ELEMENT, oldValue, newValue);
    setEdge(newValue.getEdges().get(0));
  }

  public MadElement getElement() {
    return element;
  }

  public void setEdge(MadEdge newValue) {
    if (!getElement().getEdges().contains(newValue))
      throw new IllegalArgumentException("Edge " + newValue + " is not " +
        "valid for current element " + getElement());

    MadEdge oldValue = edge;
    edge = newValue;
    if (oldValue != newValue)
      firePropertyChange(PROPNAME_EDGE, oldValue, newValue);
  }

  public MadEdge getEdge() {
    return edge;
  }

  public void setScanType(MadScanType newValue) {
    MadScanType oldValue = scanType;
    scanType = newValue;
    if (oldValue != newValue)
      firePropertyChange(PROPNAME_SCAN_TYPE, oldValue, newValue);
  }

  public MadScanType getScanType() {
    return scanType;
  }

  public void setOutputDirectory(File newValue) {
    if (!newValue.isDirectory())
      throw new IllegalArgumentException("Output directory \"" + newValue +
        "\" is not a directory");

    File oldValue = outputDirectory;
    outputDirectory = newValue;
    if (!ObjectUtilities.areEqual(oldValue, newValue))
      firePropertyChange(PROPNAME_OUTPUT_DIRECTORY, oldValue, newValue);
  }

  public File getOutputDirectory() {
    return outputDirectory;
  }

  public void setShouldSaveDataToFile(boolean newValue) {
    boolean oldValue = shouldSaveDataToFile;
    shouldSaveDataToFile = newValue;
    if (oldValue != newValue)
      firePropertyChange(PROPNAME_SHOULD_SAVE_DATA_TO_FILE, oldValue,
        newValue);
  }

  public boolean getShouldSaveDataToFile() {
    return shouldSaveDataToFile;
  }

  public XYDataset getXYDataset() {
    return getScanRunner().getXYDataset();
  }

  public File getDataOutputFile() {
    return getScanRunner().getDataOutputFile();
  }

  public void setCommand(MadFluorescenceScanCommand command)
  throws InterruptedException, EpicsCaException, IOException {
    if (command == MadFluorescenceScanCommand.GO) go();
    else if (command == MadFluorescenceScanCommand.STOP) stop();
    else throw new RuntimeException("Unexpected command: " + command);
  }

  public void go()
  throws InterruptedException, EpicsCaException, IOException {
    MadFluorescenceScanRunner scanRunner = createMadFluorescenceScanRunner(
      edge, scanType, outputDirectory);
    scanRunnerChannel.setValue(scanRunner);
    scanRunner.execute();
  }

  public void stop() throws InterruptedException, EpicsCaException {
    getScanRunner().cancel(true);
  }

  public void externallyFirePropertyChange(PropertyChangeEvent event) {
    // Without doing an identity comparison for PROPNAME_XY_DATASET, can have
    // the case where the old and new XYDataset instances are equal, thus the
    // property change event is not fired. So always use an identity
    // comparison for PROPNAME_XY_DATASET property changes.
    if (PROPNAME_XY_DATASET.equals(event.getPropertyName()))
      firePropertyChange(PROPNAME_XY_DATASET, event.getOldValue(),
        event.getNewValue(), true);
    else
      firePropertyChange(event);
  }
}
