package imcacat.imcabc;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYDataset;
import org.jfree.ui.RectangleInsets;

import com.jgoodies.binding.PresentationModel;
import com.jgoodies.binding.value.ValueHolder;
import com.jgoodies.binding.value.ValueModel;

/**
 * I am the presentation model for {@link MadFluorescenceScanChartPanel}.
 */
public class MadFluorescenceScanChartPresentationModel
extends PresentationModel<MadFluorescenceScan> {
  private static final long serialVersionUID = 1;

  protected ValueModel xyDatasetModel;
  protected XYPlot plot;
  protected JFreeChart chart;

  public MadFluorescenceScanChartPresentationModel(MadFluorescenceScan bean) {
    this(new ValueHolder(bean, true));
  }

  public MadFluorescenceScanChartPresentationModel(ValueModel beanChannel) {
    super(beanChannel);
    initializePlotAndChartModels();
    initializePlot();
    initializeChart();
    initializePlotAndChartModelChangeListeners();
  }

  protected void initializePlotAndChartModels() {
    xyDatasetModel = getModel(MadFluorescenceScan.PROPNAME_XY_DATASET);
  }

  protected void initializePlot() {
    NumberAxis xAxis = new NumberAxis("Energy (eV)");
    xAxis.setAutoRangeIncludesZero(false);
    NumberAxis yAxis = new NumberAxis("Fluorescence (a.u.)");
    yAxis.setAutoRangeIncludesZero(false);
    XYItemRenderer renderer = new XYLineAndShapeRenderer(true, false);
    plot = new XYPlot(getXYDataset(), xAxis, yAxis, renderer);
    plot.setOrientation(PlotOrientation.VERTICAL);
  }

  protected void initializeChart() {
    chart = new JFreeChart(null, JFreeChart.DEFAULT_TITLE_FONT, plot, false);
    // Hack. Setting the right inset to 15 pixels is required to work around
    // what appears to be a bug where if the x-axis tick label is being
    // displayed on the far right it gets truncated if it is long.
    chart.setPadding(new RectangleInsets(0, 0, 0, 15));
  }

  protected void initializePlotAndChartModelChangeListeners() {
    xyDatasetModel.addValueChangeListener(new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent e) {
        xyDatasetChanged((XYDataset)e.getNewValue());
      }
    });
  }

  public JFreeChart getChart() {
    return chart;
  }

  public XYDataset getXYDataset() {
    return getBean().getXYDataset();
  }

  protected void xyDatasetChanged(XYDataset newValue) {
    plot.setDataset(newValue);
  }
}
