package imcacat.imcabc;

import imcacat.epicsca4j.util.ArrayUtilities;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

import org.junit.Test;
import static org.junit.Assert.assertEquals;

/**
 * I test {@link MadFluorescenceScanDataWriter}.
 */
public class MadFluorescenceScanDataWriterTest {
  @Test
  public void write01() throws IOException {
    ByteArrayOutputStream stream = new ByteArrayOutputStream();
    MadFluorescenceScanDataWriter writer = null;
    try {
      writer = new MadFluorescenceScanDataWriter(stream);
      writer.write(new Double(12398.414), new Float(1234.0), new Float(2345.1));
      writer.flush();
    } finally {
      if (writer != null) writer.close();
    }
    assertEquals("12398.414  1234  2345\n",
      stream.toString(MadFluorescenceScanDataWriter.CHARACTER_SET_NAME));
  }

  @Test
  public void write02() throws IOException {
    ByteArrayOutputStream stream = new ByteArrayOutputStream();
    MadFluorescenceScanDataWriter writer = null;
    try {
      writer = new MadFluorescenceScanDataWriter(stream);
      writer.write(new Double(9000.0), new Float(1234.0), new Float(2345.1));
      writer.flush();
    } finally {
      if (writer != null) writer.close();
    }
    assertEquals(" 9000.000  1234  2345\n",
      stream.toString(MadFluorescenceScanDataWriter.CHARACTER_SET_NAME));
  }

  @Test
  public void write03() throws IOException {
    ByteArrayOutputStream stream = new ByteArrayOutputStream();
    MadFluorescenceScanDataWriter writer = null;
    try {
      writer = new MadFluorescenceScanDataWriter(stream);
      writer.write(new Double(9000.0), new Float(234.0), new Float(12345.1));
      writer.flush();
    } finally {
      if (writer != null) writer.close();
    }
    assertEquals(" 9000.000   234 12345\n",
      stream.toString(MadFluorescenceScanDataWriter.CHARACTER_SET_NAME));
  }

  @Test
  public void write04() throws IOException {
    ByteArrayOutputStream stream = new ByteArrayOutputStream();
    MadFluorescenceScanDataWriter writer = null;
    try {
      writer = new MadFluorescenceScanDataWriter(stream);
      writer.write(MadEdge.SE_K,
        ArrayUtilities.asList(new double[] { 9000.0,  12000.0  }),
        ArrayUtilities.asList(new float[]  {  200.0f, 20000.0f }),
        ArrayUtilities.asList(new float[]  {  400.0f,  5000.0f }));
      writer.flush();
    } finally {
      if (writer != null) writer.close();
    }
    assertEquals("Edge: Se K 12658.0 eV\n" +
                 " 9000.000   200   400\n" +
                 "12000.000 20000  5000\n",
      stream.toString(MadFluorescenceScanDataWriter.CHARACTER_SET_NAME));
  }

  @Test
  public void write05() throws IOException {
    ByteArrayOutputStream stream = new ByteArrayOutputStream();
    MadFluorescenceScanDataWriter writer = null;
    try {
      writer = new MadFluorescenceScanDataWriter(stream);
      writer.write(new Double(12398.4149), new Float(234.0), new Float(2345.1));
      writer.flush();
    } finally {
      if (writer != null) writer.close();
    }
    assertEquals("12398.415   234  2345\n",
      stream.toString(MadFluorescenceScanDataWriter.CHARACTER_SET_NAME));
  }
}
