package imcacat.imcabc;

import gov.aps.jca.TimeoutException;
import imcacat.epicsca4j.ca.EpicsCaException;
import imcacat.epicsca4j.model.EpicsMotor;

import java.io.File;

/**
 * I am a <code>MadFluorescenceScanRunner</code> for the 17-BM beamline.
 */
public class MadFluorescenceScanRunner17bm extends MadFluorescenceScanRunner {
  protected EpicsMotor energyMotor;

  public MadFluorescenceScanRunner17bm()
  throws InterruptedException, EpicsCaException {
    super();
  }

  public MadFluorescenceScanRunner17bm(MadEdge edge)
  throws InterruptedException, EpicsCaException {
    super(edge);
  }

  public MadFluorescenceScanRunner17bm(MadEdge edge, MadScanType scanType)
  throws InterruptedException, EpicsCaException {
    super(edge, scanType);
  }

  public MadFluorescenceScanRunner17bm(MadEdge edge, MadScanType scanType,
      File outputDirectory)
  throws InterruptedException, EpicsCaException {
    super(edge, scanType, outputDirectory);
  }

  protected void initialize() throws EpicsCaException, InterruptedException {
    super.initialize();

    energyMotor = new EpicsMotor(
      Resources.getDefault().getEnergyMotorPvName(), "eV");
  }

  public void release() {
    if (energyMotor != null) energyMotor.release();

    super.release();
  }

  protected void cleanUp() throws EpicsCaException, InterruptedException {
    Exception caughtException = null;

    try {
      scan.stop();
    } catch (Exception e) {
      if (caughtException == null) caughtException = e;
    }

    try {
      energyMotor.stop();
    } catch (Exception e) {
      if (caughtException == null) caughtException = e;
    }

    try {
      super.cleanUp();
    } catch (Exception e) {
      if (caughtException == null) caughtException = e;
    }

    if (caughtException != null) throw new RuntimeException(caughtException);
  }

  protected void moveEnergyAndWait(double scaledDesiredPosition)
  throws InterruptedException, TimeoutException, EpicsCaException {
    energyMotor.setDesiredPosition(scaledDesiredPosition);
    energyMotor.moveAndWait();
  }

  @Override
  protected void stopEnergyMove()
  throws InterruptedException, EpicsCaException {
    energyMotor.stop();
  }
}
