package imcacat.imcabc;

import gov.aps.jca.TimeoutException;
import imcacat.epicsca4j.ca.EpicsCaException;
import imcacat.epicsca4j.model.EpicsPv;

import java.io.File;
import java.util.List;

/**
 * I am a <code>MadFluorescenceScanRunner</code> for the 17-ID beamline.
 */
public class MadFluorescenceScanRunner17id extends MadFluorescenceScanRunner {
  protected EpicsPv sampleBpmIntensityAcquisitionTimePv;
  protected TpmacEnergyAxis energyAxis;

  public MadFluorescenceScanRunner17id()
  throws InterruptedException, EpicsCaException {
    super();
  }

  public MadFluorescenceScanRunner17id(MadEdge edge)
  throws InterruptedException, EpicsCaException {
    super(edge);
  }

  public MadFluorescenceScanRunner17id(MadEdge edge, MadScanType scanType)
  throws InterruptedException, EpicsCaException {
    super(edge, scanType);
  }

  public MadFluorescenceScanRunner17id(MadEdge edge, MadScanType scanType,
      File outputDirectory)
  throws InterruptedException, EpicsCaException {
    super(edge, scanType, outputDirectory);
  }

  protected void initialize() throws EpicsCaException, InterruptedException {
    super.initialize();

    sampleBpmIntensityAcquisitionTimePv = new EpicsPv(
      Resources.getDefault().getBpm3IntensityAcquisitionTimePvName(),
      false, false);

    energyAxis = new TpmacEnergyAxis(
      Resources.getDefault().getEnergyAssemblyPvName(),
      Resources.getDefault().getEnergyAxisPvName(), 1.0);
  }

  public void release() {
    if (sampleBpmIntensityAcquisitionTimePv != null)
      sampleBpmIntensityAcquisitionTimePv.release();
    if (energyAxis != null) energyAxis.release();

    super.release();
  }

  protected void cleanUp() throws EpicsCaException, InterruptedException {
    Exception caughtException = null;

    try {
      scan.stop();
    } catch (Exception e) {
      if (caughtException == null) caughtException = e;
    }

    try {
      energyAxis.stop();
    } catch (Exception e) {
      if (caughtException == null) caughtException = e;
    }

    try {
      super.cleanUp();
    } catch (Exception e) {
      if (caughtException == null) caughtException = e;
    }

    if (caughtException != null) throw new RuntimeException(caughtException);
  }

  protected void configureScan(List<Double> energyPositions)
  throws InterruptedException, EpicsCaException {
    super.configureScan(energyPositions);

    sampleBpmIntensityAcquisitionTimePv.setValue(
      scanType.getAcquisitionTime());
  }

  protected void moveEnergyAndWait(double scaledDesiredPosition)
  throws InterruptedException, TimeoutException, EpicsCaException {
    energyAxis.setDesiredPosition(scaledDesiredPosition);
    energyAxis.moveAndWait();
  }

  @Override
  protected void stopEnergyMove()
  throws InterruptedException, EpicsCaException {
    energyAxis.stop();
  }
}
