package imcacat.imcabc;

import imcacat.epicsca4j.ca.EpicsCaException;
import imcacat.epicsca4j.convert.EnumToStringConversionStrategy;
import imcacat.epicsca4j.format.DoubleDecimalFormat;
import imcacat.imcabc.util.Pair;

import java.awt.BorderLayout;
import java.text.DecimalFormat;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;

import com.jgoodies.binding.beans.PropertyConnector;
import com.jgoodies.binding.value.ValueModel;
import com.jgoodies.common.format.EmptyNumberFormat;
import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

/**
 * I am the MAD tab view.
 */
public class MadPanel extends JPanel {
  private static final long serialVersionUID = 1;

  protected MadPresentationModel model;

  protected ProgressBarMediator scanProgressBarMediator;
  protected PropertyConnector scanProgressBarConnector;

  public MadPanel(MadPresentationModel model)
  throws EpicsCaException {
    super();
    this.model = model;
    setLayout(new BorderLayout());
    add(buildMainPanel(), BorderLayout.CENTER);
  }

  public void release() {
    scanProgressBarMediator.release();
    scanProgressBarConnector.release();
  }

  protected JComponent buildMainPanel() throws EpicsCaException {
    FormLayout layout = new FormLayout(
      "pref, pref, fill:pref:grow",
      "fill:pref:grow");
    JPanel panel = new JPanel(layout);
    CellConstraints cc = new CellConstraints();
    panel.add(buildLeftPanel(), cc.xy(1, 1));
    panel.add(buildLeftRightSeparator(), cc.xy(2,1));
    panel.add(buildRightPanel(), cc.xy(3, 1));
    return panel;
  }

  protected JComponent buildLeftPanel() {
    FormLayout layout = new FormLayout(
      "right:max(75dlu;pref), 4dlu, fill:40dlu, 4dlu, " +
        "fill:40dlu, 4dlu, fill:40dlu, 4dlu, " +
        "fill:40dlu", // Columns
      "pref, 4dlu, pref, 2dlu, pref, 2dlu, pref, 2dlu, pref, 4dlu, pref, " +
        "4dlu, " + // Step 1
      "pref, 4dlu, pref, 4dlu, pref, 4dlu, " + // Step 2
      "pref, 4dlu, pref, 4dlu, pref, 2dlu, pref, 2dlu, pref, 2dlu, pref, " +
        "2dlu, pref, 2dlu, 4dlu, pref" // Step 3
    );
    layout.setRowGroups(new int[][] {
      {1, 13, 19}, // separators
      {3, 5, 7, 9, 11, 15, 17, 21, 25, 27, 29, 31, 33} // fields
    });
    layout.setColumnGroups(new int[][] {
        {3, 5, 7, 9} // results table columns
      });
    PanelBuilder builder = new PanelBuilder(layout);
    builder.border(Borders.DIALOG);
    CellConstraints cc = new CellConstraints();

    builder.addSeparator("1. Run fluorescence scan", cc.xyw(1, 1, 9));

    builder.addLabel("Edge:", cc.xy(1, 3));
    JComboBox elementComboBox = ComponentFactory.createComboBox(
      model.getElementSelectionInList(), new StrategyListCellRenderer(
        new EnumToStringConversionStrategy(
          Resources.getDefault().getMadElementToDisplayStringMap())));
    builder.add(elementComboBox, cc.xyw(3, 3, 3));

    JComboBox edgeComboBox = ComponentFactory.createComboBox(
      model.getEdgeSelectionInList(), new StrategyListCellRenderer(
        new EnumToStringConversionStrategy(
          Resources.getDefault().getMadEdgeToDisplayStringMap())));
    builder.add(edgeComboBox, cc.xyw(3, 5, 3));

    builder.addLabel("Scan type:", cc.xy(1, 7));
    JComboBox scanTypeComboBox = ComponentFactory.createComboBox(
      model.getScanTypeSelectionInList(), new StrategyListCellRenderer(
        new EnumToStringConversionStrategy(
          Resources.getDefault().getMadScanTypeToDisplayStringMap())));
    builder.add(scanTypeComboBox, cc.xyw(3, 7, 3));

    builder.addLabel("Scan output location:", cc.xy(1, 9));
    JFormattedTextField scanDataOutputDirTextField =
      ComponentFactory.createFormattedTextField(model.getModel(
        MadFacade.PROPNAME_SCAN_OUTPUT_DIRECTORY), new DirectoryFormatter());
    builder.add(scanDataOutputDirTextField, cc.xyw(3, 9, 5));
    Action chooseScanOutputDirAction = new ChooseDirectoryAction(this,
      "Choose...", "Scan output location", model.getModel(
        MadFacade.PROPNAME_SCAN_OUTPUT_DIRECTORY));
    builder.add(new JButton(chooseScanOutputDirAction), cc.xy(9, 9));

    Pair<JProgressBar,ProgressBarMediator> pair =
      ComponentFactory.createProgressBar(
        model.getModel(MadFacade.PROPNAME_SCAN_PROGRESS),
        model.getModel(MadFacade.PROPNAME_SCAN_STATUS));
    JProgressBar scanProgressBar = pair.getFirst();
    scanProgressBarMediator = pair.getSecond();
    scanProgressBarConnector = PropertyConnector.connect(
      model.getScanProgressBarVisibleModel(),
      ValueModel.PROPERTY_VALUE, scanProgressBar, "visible");
    scanProgressBarConnector.updateProperty2();
    builder.add(scanProgressBar, cc.xyw(3, 11, 5));
    Action goAction = model.getScanControlAction();
    builder.add(new JButton(goAction), cc.xy(9, 11));

    builder.addSeparator("2. Process fluorescence scan data",
      cc.xyw(1, 13, 9));

    builder.addLabel("Scan data file:", cc.xy(1, 15));
    JFormattedTextField scanDataFileTextField =
      ComponentFactory.createFormattedTextField(model.getModel(
        MadFacade.PROPNAME_DATA_FILE), new FileFormatter(true));
    builder.add(scanDataFileTextField, cc.xyw(3, 15, 5));
    Action chooseScanDataFileAction = new ChooseFileAction(this,
      "Choose...", "Scan data file", model.getModel(
        MadFacade.PROPNAME_DATA_FILE), new FileChooserFileFilter(
          new MadFluorescenceScanDataFileFilter(),
          "Imcabc fluorescence scans (??_??_scan_???.dat)"));
    builder.add(new JButton(chooseScanDataFileAction), cc.xy(9, 15));

    JButton calculateButton = new JButton(
      model.getDataProcessCalculateAction());
    builder.add(calculateButton, cc.xy(9, 17));

    builder.addSeparator("3. View results and save experiment parameters",
      cc.xyw(1, 19, 9));

    builder.addLabel("f' and f'' spectra plot:", cc.xy(1, 21));
    JButton viewSpectraPlotButton = new JButton(
      model.getViewSpectraPlotAction());
    builder.add(viewSpectraPlotButton, cc.xy(9, 21));

    builder.addLabel("Peak", cc.xy(3, 23, "center, default"));
    builder.addLabel("Edge", cc.xy(5, 23, "center, default"));
    builder.addLabel("Low", cc.xy(7, 23, "center, default"));
    builder.addLabel("High", cc.xy(9, 23, "center, default"));

    builder.addLabel("Energy (eV):", cc.xy(1, 25));
    JLabel peakEnergyLabel = ComponentFactory.createLabel(model.getModel(
        MadFacade.PROPNAME_PEAK_ENERGY), new EmptyIfNullFormat(
            new DecimalFormat("####0.000")));
    builder.add(peakEnergyLabel, cc.xy(3, 25, "right, default"));
    JLabel edgeEnergyLabel = ComponentFactory.createLabel(model.getModel(
        MadFacade.PROPNAME_EDGE_ENERGY), new EmptyIfNullFormat(
          new DecimalFormat("####0.000")));
    builder.add(edgeEnergyLabel, cc.xy(5, 25, "right, default"));
    JTextField lowEnergyTextField = ComponentFactory.createFormattedTextField(
      model.getModel(MadFacade.PROPNAME_LOW_ENERGY),
      new EmptyNumberFormat(new DoubleDecimalFormat("#####.000")));
    lowEnergyTextField.setHorizontalAlignment(JTextField.RIGHT);
    builder.add(lowEnergyTextField, cc.xy(7, 25));
    JTextField highEnergyTextField =
      ComponentFactory.createFormattedTextField(
        model.getModel(MadFacade.PROPNAME_HIGH_ENERGY),
        new EmptyNumberFormat(new DoubleDecimalFormat("####0.000")));
    highEnergyTextField.setHorizontalAlignment(JTextField.RIGHT);
    builder.add(highEnergyTextField, cc.xy(9, 25));

    builder.addLabel("Wavelength (\u00C5):", cc.xy(1, 27));
    JLabel peakWavelengthLabel = ComponentFactory.createLabel(model.getModel(
        MadFacade.PROPNAME_PEAK_WAVELENGTH), new EmptyIfNullFormat(
          new DecimalFormat("0.0000000")));
    builder.add(peakWavelengthLabel, cc.xy(3, 27, "right, default"));
    JLabel edgeWavelengthLabel = ComponentFactory.createLabel(model.getModel(
        MadFacade.PROPNAME_EDGE_WAVELENGTH), new EmptyIfNullFormat(
          new DecimalFormat("0.0000000")));
    builder.add(edgeWavelengthLabel, cc.xy(5, 27, "right, default"));
    JTextField lowWavelengthTextField =
      ComponentFactory.createFormattedTextField(
        model.getModel(MadFacade.PROPNAME_LOW_WAVELENGTH),
        new EmptyNumberFormat(new DoubleDecimalFormat("0.0000000")));
    lowWavelengthTextField.setHorizontalAlignment(JTextField.RIGHT);
    builder.add(lowWavelengthTextField, cc.xy(7, 27));
    JTextField highWavelengthTextField =
      ComponentFactory.createFormattedTextField(
          model.getModel(MadFacade.PROPNAME_HIGH_WAVELENGTH),
          new EmptyNumberFormat(new DoubleDecimalFormat("0.0000000")));
    highWavelengthTextField.setHorizontalAlignment(JTextField.RIGHT);
    builder.add(highWavelengthTextField, cc.xy(9, 27));

    builder.addLabel("f':", cc.xy(1, 29));
    JLabel peakFPrimeLabel = ComponentFactory.createLabel(model.getModel(
        MadFacade.PROPNAME_PEAK_F_PRIME), new EmptyIfNullFormat(
          new DecimalFormat("#0.00")));
    builder.add(peakFPrimeLabel, cc.xy(3, 29, "right, default"));
    JLabel edgeFPrimeLabel = ComponentFactory.createLabel(model.getModel(
        MadFacade.PROPNAME_EDGE_F_PRIME), new EmptyIfNullFormat(
          new DecimalFormat("#0.00")));
    builder.add(edgeFPrimeLabel, cc.xy(5, 29, "right, default"));
    JLabel lowFPrimeLabel = ComponentFactory.createLabel(model.getModel(
        MadFacade.PROPNAME_LOW_F_PRIME), new EmptyIfNullFormat(
          new DecimalFormat("#0.00")));
    builder.add(lowFPrimeLabel, cc.xy(7, 29, "right, default"));
    JLabel highFPrimeLabel = ComponentFactory.createLabel(model.getModel(
        MadFacade.PROPNAME_HIGH_F_PRIME), new EmptyIfNullFormat(
          new DecimalFormat("#0.00")));
    builder.add(highFPrimeLabel, cc.xy(9, 29, "right, default"));

    builder.addLabel("f'':", cc.xy(1, 31));
    JLabel peakFDoublePrimeLabel = ComponentFactory.createLabel(
        model.getModel(MadFacade.PROPNAME_PEAK_F_DOUBLE_PRIME),
        new EmptyIfNullFormat(new DecimalFormat("#0.00")));
    builder.add(peakFDoublePrimeLabel, cc.xy(3, 31, "right, default"));
    JLabel edgeFDoublePrimeLabel = ComponentFactory.createLabel(
        model.getModel(MadFacade.PROPNAME_EDGE_F_DOUBLE_PRIME),
        new EmptyIfNullFormat(new DecimalFormat("#0.00")));
    builder.add(edgeFDoublePrimeLabel, cc.xy(5, 31, "right, default"));
    JLabel lowFDoublePrimeLabel = ComponentFactory.createLabel(
        model.getModel(MadFacade.PROPNAME_LOW_F_DOUBLE_PRIME),
        new EmptyIfNullFormat(new DecimalFormat("#0.00")));
    builder.add(lowFDoublePrimeLabel, cc.xy(7, 31, "right, default"));
    JLabel highFDoublePrimeLabel = ComponentFactory.createLabel(
        model.getModel(MadFacade.PROPNAME_HIGH_F_DOUBLE_PRIME),
        new EmptyIfNullFormat(new DecimalFormat("#0.00")));
    builder.add(highFDoublePrimeLabel, cc.xy(9, 31, "right, default"));

    JButton saveButton = new JButton(
      model.getSaveExperimentParametersAction());
    builder.add(saveButton, cc.xy(9, 33));

    return builder.getPanel();
  }

  protected JComponent buildRightPanel() throws EpicsCaException {
    FormLayout layout = new FormLayout(
      "fill:pref:grow",
      "pref, 4dlu, fill:pref:grow"
    );
    PanelBuilder builder = new PanelBuilder(layout);
    builder.border(Borders.DIALOG);
    CellConstraints cc = new CellConstraints();

    builder.addSeparator("Fluorescence scan monitor", cc.xy(1, 1));

    JPanel panel = new JPanel(new AspectRatioLayout());
    panel.add(new MadFluorescenceScanChartPanel(
      new MadFluorescenceScanChartPresentationModel(
        model.getModel(MadFacade.PROPNAME_FLUORESCENCE_SCAN))));
    builder.add(panel, cc.xy(1, 3));

    return builder.getPanel();
  }

  protected JComponent buildLeftRightSeparator() {
    JPanel panel = new JPanel(new BorderLayout());
    panel.add(new JSeparator(SwingConstants.VERTICAL), BorderLayout.CENTER);
    panel.setBorder(new EmptyBorder(10, 0, 10, 0));
    return panel;
  }
}
