package imcacat.imcabc;

import imcacat.epicsca4j.ca.EpicsCaException;
import imcacat.epicsca4j.util.UiUtilities;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.List;

import javax.swing.Action;

import com.jgoodies.binding.PresentationModel;
import com.jgoodies.binding.beans.PropertyAdapter;
import com.jgoodies.binding.list.SelectionInList;
import com.jgoodies.binding.value.ValueHolder;
import com.jgoodies.binding.value.ValueModel;

/**
 * I am the presentation model for {@link MadPanel}.
 */
public class MadPresentationModel extends PresentationModel<MadFacade> {
  private static final long serialVersionUID = 1;

  protected ValueModel fluorescenceScanBeanChannel;
  protected ImcabcPresentationModel imcabcPresentationModel;
  protected PropertyAdapter<MadFluorescenceScan> fluorescenceScanRunningModel;
  protected PropertyAdapter<MadFluorescenceScan> fluorescenceScanStatusModel;

  protected SelectionInList<MadElement> elementSelection;
  protected SelectionInList<MadEdge> edgeSelection;
  protected SelectionInList<MadScanType> scanTypeSelection;
  protected ValueModel scanProgressBarVisibleModel;
  protected Action scanControlAction;

  protected Action calculateFAndFDoublePrimeAction;
  protected Action viewSpectraPlotAction;
  protected Action saveExperimentParametersAction;

  public MadPresentationModel(MadFacade bean,
      ImcabcPresentationModel imcabcPresentationModel)
  throws EpicsCaException {
    this(new ValueHolder(bean, true), imcabcPresentationModel);
  }

  public MadPresentationModel(ValueModel beanChannel,
      ImcabcPresentationModel imcabcPresentationModel) {
    super(beanChannel);
    this.imcabcPresentationModel = imcabcPresentationModel;

    fluorescenceScanBeanChannel = getModel(
      MadFacade.PROPNAME_FLUORESCENCE_SCAN);

    elementSelection = new SelectionInList<MadElement>(MadElement.values(),
      getModel(MadFacade.PROPNAME_ELEMENT));
    edgeSelection = new SelectionInList<MadEdge>(getEdgesForSelectedElement(),
      getModel(MadFacade.PROPNAME_EDGE));
    elementSelection.addValueChangeListener(new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent e) {
        makeEdgeListConsistentWithElement();
      }
    });
    scanTypeSelection = new SelectionInList<MadScanType>(MadScanType.values(),
      getModel(MadFacade.PROPNAME_SCAN_TYPE));

    fluorescenceScanRunningModel = new PropertyAdapter<MadFluorescenceScan>(
      fluorescenceScanBeanChannel, MadFluorescenceScan.PROPNAME_RUNNING,
      true);
    fluorescenceScanRunningModel.addValueChangeListener(
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          fluorescenceScanRunningChanged(event);
        }
      });

    fluorescenceScanStatusModel = new PropertyAdapter<MadFluorescenceScan>(
        fluorescenceScanBeanChannel, MadFluorescenceScan.PROPNAME_STATUS,
        true);
    fluorescenceScanStatusModel.addValueChangeListener(
        new PropertyChangeListener() {
          public void propertyChange(PropertyChangeEvent event) {
            fluorescenceScanStatusChanged(event);
          }
        });

    imcabcPresentationModel.addPropertyChangeListener(
      ImcabcPresentationModel.PROPNAME_FLUORESCENCE_SCAN_ENABLED,
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          isFluorescenceScanEnabledChanged(event);
        }
      });

    scanProgressBarVisibleModel = new ValueHolder(false);

    scanControlAction = new TwoStateValueModelButtonAction("Run", "Stop",
      MadFluorescenceScanCommand.GO, MadFluorescenceScanCommand.STOP,
      false, true, getModel(MadFacade.PROPNAME_SCAN_RUNNING),
      getModel(MadFacade.PROPNAME_SCAN_COMMAND), true);

    getModel(MadFacade.PROPNAME_SCAN_OUTPUT_DIRECTORY).addValueChangeListener(
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          updateScanControlActionEnablement(event, null);
        }
      });
//XXX Listening for this above on the fluorescenceScanRunningModel
//    Since this is a hack to get things working without adding bean channel
//    support to the MAD models, presentation models, and views, keeping this
//    here for now. If do add support, then will probably get rid of this.
//    Otherwise, might get rid of the above fluorescenceScanRunningModel and
//    go back to doing it this way.
//    getModel(MadFacade.PROPNAME_IS_SCAN_RUNNING).addValueChangeListener(
//        new PropertyChangeListener() {
//          public void propertyChange(PropertyChangeEvent event) {
//            updateScanControlActionEnablement(null, event);
//          }
//      });
    updateScanControlActionEnablement();

    calculateFAndFDoublePrimeAction =
      new MadCalculateFAndFDoublePrimeAction(this);

    viewSpectraPlotAction = new SetValueButtonAction("View",
      getModel(MadFacade.PROPNAME_DATA_PROCESSOR_COMMAND),
        MadFluorescenceScanDataProcessorCommand.VIEW_SPECTRA_PLOT);

    saveExperimentParametersAction  = new SaveMadExperimentParametersAction(
      UiUtilities.getRootFrame(), "Save...", getBean());

    getModel(MadFacade.PROPNAME_DATA_FILE).addValueChangeListener(
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          updateDataProcessCalculateActionEnablement(event);
          disableViewSpectraPlotAction();
          disableSaveExperimentParametersAction();
        }
      });

    getModel(MadFacade.PROPNAME_DATA_PROCESSOR_CALCULATION_DONE_REASON).
    addValueChangeListener(
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          dataProcessorCalculationDoneReasonChanged(event);
        }
      });

    updateDataProcessCalculateActionEnablement();
    dataProcessorCalculationDoneReasonChanged();
  }

  public SelectionInList<MadElement> getElementSelectionInList() {
    return elementSelection;
  }

  public SelectionInList<MadEdge> getEdgeSelectionInList() {
    return edgeSelection;
  }

  public SelectionInList<MadScanType> getScanTypeSelectionInList() {
    return scanTypeSelection;
  }

  public ValueModel getScanProgressBarVisibleModel() {
    return scanProgressBarVisibleModel;
  }

  protected void makeEdgeListConsistentWithElement() {
    edgeSelection.setList(getEdgesForSelectedElement());
    edgeSelection.clearSelection();
    edgeSelection.setSelectionIndex(0);
  }

  protected List<MadEdge> getEdgesForSelectedElement() {
    return elementSelection.getSelection().getEdges();
  }

  public MadFluorescenceScan getFluorescenceScan() {
    return getBean().getFluorescenceScan();
  }

  public Action getScanControlAction() {
    return scanControlAction;
  }

  public Action getDataProcessCalculateAction() {
    return calculateFAndFDoublePrimeAction;
  }

  public Action getViewSpectraPlotAction() {
    return viewSpectraPlotAction;
  }

  public Action getSaveExperimentParametersAction() {
    return saveExperimentParametersAction;
  }

  protected boolean isFlourescenceScanRunning() {
    return getFluorescenceScan().isRunning();
  }

  protected void scanControlActionActionPerformedStateChanged(int newValue) {
    if (TwoStateValueModelButtonAction.STATE_1 == newValue)
      disableCalculateFAndFDoublePrimeAction();
  }

  protected void fluorescenceScanRunningChanged(PropertyChangeEvent event) {
    updateScanProgressBarVisibility(null, event);
    updateScanControlActionEnablement(null, event);
    updateDataProcessCalculateActionEnablement(null);
  }

  protected void fluorescenceScanStatusChanged(PropertyChangeEvent event) {
    updateScanProgressBarVisibility(event, null);
  }

  protected void updateScanProgressBarVisibility(
      PropertyChangeEvent scanStatusEvent,
      PropertyChangeEvent scanRunningEvent) {
    boolean isScanRunning;
    if (scanRunningEvent == null) {
      try {
        isScanRunning = getBean().isScanRunning();
      } catch (InterruptedException e) {
        isScanRunning = false;
      } catch (EpicsCaException e) {
        isScanRunning = false;
      }
    } else {
      isScanRunning = (Boolean)scanRunningEvent.getNewValue();
    }
    if (isScanRunning) {
      scanProgressBarVisibleModel.setValue(true);
      return;
    }

    String scanStatus;
    if (scanStatusEvent == null) {
      try {
        scanStatus = getBean().getScanStatus();
      } catch (InterruptedException e) {
        scanStatus = "";
      } catch (EpicsCaException e) {
        scanStatus = "";
      }
    } else {
      scanStatus = (String)scanStatusEvent.getNewValue();
    }
    scanProgressBarVisibleModel.setValue(!"".equals(scanStatus));
  }

  protected void updateScanControlActionEnablement() {
    updateScanControlActionEnablement(null, null);
  }

  protected void updateScanControlActionEnablement(
      PropertyChangeEvent scanOutputDirectoryEvent,
      PropertyChangeEvent scanRunningEvent) {
    if (!imcabcPresentationModel.isFluorescenceScanEnabled()) {
      scanControlAction.setEnabled(false);
      return;
    }

    File outputDirectory = (scanOutputDirectoryEvent == null)
      ? getBean().getScanOutputDirectory()
      : (File)scanOutputDirectoryEvent.getNewValue();
    Boolean isScanRunning;
    if (scanRunningEvent == null) {
      try {
        isScanRunning = getBean().isScanRunning();
      } catch (InterruptedException e) {
        isScanRunning = true;
      } catch (EpicsCaException e) {
        isScanRunning = true;
      }
    } else {
      isScanRunning = (Boolean)scanRunningEvent.getNewValue();
    }
    scanControlAction.setEnabled(isScanRunning ? true :
        (outputDirectory == null ? false : outputDirectory.isDirectory()));
  }

  protected void isFluorescenceScanEnabledChanged(PropertyChangeEvent event) {
    if (!((Boolean)event.getNewValue()).booleanValue())
      scanControlAction.setEnabled(false);
    else
      updateScanControlActionEnablement(null, event);
  }

  protected void updateDataProcessCalculateActionEnablement() {
    updateDataProcessCalculateActionEnablement(null);
  }

  protected void updateDataProcessCalculateActionEnablement(
      PropertyChangeEvent dataFileValueChangeEvent) {
    File dataFile = (dataFileValueChangeEvent == null)
      ? getBean().getDataFile()
      : (File)dataFileValueChangeEvent.getNewValue();
    calculateFAndFDoublePrimeAction.setEnabled(
      dataFile == null ? false :
        dataFile.isFile() && !isFlourescenceScanRunning());
  }

  protected void dataProcessorCalculationDoneReasonChanged() {
    dataProcessorCalculationDoneReasonChanged(null);
  }

  protected void dataProcessorCalculationDoneReasonChanged(
      PropertyChangeEvent event) {
    MadFluorescenceScanDataProcessorCalculationDoneReason reason =
      (event == null)
      ? MadFluorescenceScanDataProcessorCalculationDoneReason.NONE
      : (MadFluorescenceScanDataProcessorCalculationDoneReason)
          event.getNewValue();
    viewSpectraPlotAction.setEnabled(
      MadFluorescenceScanDataProcessorCalculationDoneReason.SUCCESS.equals(
        reason));
    saveExperimentParametersAction.setEnabled(
      MadFluorescenceScanDataProcessorCalculationDoneReason.SUCCESS.equals(
        reason));
  }

  protected void disableCalculateFAndFDoublePrimeAction() {
    calculateFAndFDoublePrimeAction.setEnabled(false);
  }

  protected void disableViewSpectraPlotAction() {
    viewSpectraPlotAction.setEnabled(false);
  }

  protected void disableSaveExperimentParametersAction() {
    saveExperimentParametersAction.setEnabled(false);
  }
}
