package imcacat.imcabc;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * I model the types of scans for a MAD experiment.
 */
public enum MadScanType {
  STANDARD (1.0f,
            Arrays.asList(new Double[] {-50.0, -10.0, 15.0, 80.0}),
            Arrays.asList(new Double[] {  4.0,   0.4,  4.0})),
  SHORT    (1.0f,
            Arrays.asList(new Double[] {-30.0,  -5.0, 15.0,  80.0}),
            Arrays.asList(new Double[] {  4.0,   0.4,  4.0})),
  FINE     (1.0f,
            Arrays.asList(new Double[] {-50.0, -10.0, 15.0,  80.0}),
            Arrays.asList(new Double[] {  4.0,   0.1,  4.0})),
  COARSE   (1.0f,
            Arrays.asList(new Double[] {-50.0, -10.0, 20.0, 100.0}),
            Arrays.asList(new Double[] {  4.0,   1.0,  4.0}));

  protected float acquisitionTime;
  protected List<Double> regionBoundaries;
  protected List<Double> regionStepSizes;

  MadScanType(float acquisitionTime, List<Double> regionBoundaries,
      List<Double> regionStepSizes) {
    this.acquisitionTime = acquisitionTime;
    this.regionBoundaries = regionBoundaries;
    this.regionStepSizes = regionStepSizes;
  }

  public float getAcquisitionTime() {
    return acquisitionTime;
  }

  public List<Double> getRegionBoundaries() {
    return Collections.unmodifiableList(regionBoundaries);
  }

  public List<Double> getRegionStepSizes() {
    return Collections.unmodifiableList(regionStepSizes);
  }

  public double getFirstRegionBoundary() {
    return regionBoundaries.get(0);
  }

  public double getLastRegionBoundary() {
    return regionBoundaries.get(regionBoundaries.size() - 1);
  }

  public double getStepSize(double position, double edgeEnergy) {
    if (position >= edgeEnergy + regionBoundaries.get(0) &&
        position <  edgeEnergy + regionBoundaries.get(1))
      return regionStepSizes.get(0);
    if (position >= edgeEnergy + regionBoundaries.get(1) &&
        position <  edgeEnergy + regionBoundaries.get(2))
      return regionStepSizes.get(1);
    if (position >= edgeEnergy + regionBoundaries.get(2) &&
        position <= edgeEnergy + regionBoundaries.get(3))
      return regionStepSizes.get(2);
    throw new IllegalArgumentException("position not within regions: " +
      position);
  }
}
