package imcacat.imcabc;

import imcacat.epicsca4j.ca.EpicsCaException;
import imcacat.epicsca4j.util.UiUtilities;
import imcacat.imcabc.util.Pair;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.DecimalFormat;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;

import org.jfree.chart.ChartPanel;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

/**
 * I am a view of an {@link Optimization}.
 */
public class OptimizationPanel extends JPanel {
  private static final long serialVersionUID = 1;
  protected static final Dimension GRAPH_PREFERRED_SIZE =
    new Dimension(100, 100);

  protected OptimizationPresentationModel model;
  protected ProgressBarMediator statusProgressBarMediator;

  protected ChartPanel chartPanel;

  public OptimizationPanel() throws EpicsCaException {
    this(new OptimizationPresentationModel());
  }

  public OptimizationPanel(
      OptimizationPresentationModel model) {
    super();
    this.model = model;
    setLayout(new BorderLayout());
    add(buildMainPanel(), BorderLayout.CENTER);
  }

  public void release() {
    statusProgressBarMediator.release();
  }

  protected JPanel buildMainPanel() {
    FormLayout layout = new FormLayout(
      "right:pref, 4dlu, fill:40dlu, 4dlu, fill:40dlu, 4dlu, " +
        "fill:40dlu, 4dlu, fill:40dlu:grow", // Columns
      "pref, 4dlu, fill:pref:grow, 4dlu, pref, 4dlu, pref, 4dlu, " +
          // Rows: Graph
        "pref, 4dlu, pref, 2dlu, pref" // Rows: Progress
    );
    layout.setRowGroups(new int[][] {
      {1, 9}, // section labels
      {5, 7, 11, 13} // labels
    });
    layout.setColumnGroups(new int[][] {
        {3, 5, 7, 9} // crosshair values
      });
    PanelBuilder builder = new PanelBuilder(layout);
    builder.border(Borders.DIALOG);
    CellConstraints cc = new CellConstraints();

    builder.addSeparator("Graph", cc.xyw(1, 1, 9));

    CrosshairListener crosshairListener = new OptimizationCrosshairListener(
      model);
    model.getChart().addProgressListener(crosshairListener);
    chartPanel = new ChartPanel(model.getChart());
    chartPanel.addChartMouseListener(crosshairListener);
    model.getModel(Optimization.PROPNAME_XY_DATASET).addValueChangeListener(
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          resetChartPanelZoom();
        }
      });
    JPanel aspectRatioPanel = new JPanel(new AspectRatioLayout());
    aspectRatioPanel.add(chartPanel);
    aspectRatioPanel.setPreferredSize(UiUtilities.createScaledDownToFitIn(
      aspectRatioPanel.getPreferredSize(), GRAPH_PREFERRED_SIZE));
    builder.add(aspectRatioPanel, cc.xyw(1, 3, 9));

    builder.addLabel("X", cc.xy(3, 5, "center, default"));
    builder.addLabel("Y-Measured", cc.xy(5, 5, "center, default"));
    builder.addLabel("Y-Fit", cc.xy(7, 5, "center, default"));

    builder.addLabel("Crosshairs:", cc.xy(1, 7));
    JLabel xLabel = ComponentFactory.createLabel(model.getModel(
      Optimization.PROPNAME_X_POINT_OF_INTEREST), new EmptyIfNullFormat(
      new DecimalFormat("#####0.000")));
    builder.add(xLabel, cc.xy(3, 7, "right, default"));
    JLabel measuredYLabel = ComponentFactory.createLabel(model.getModel(
      Optimization.PROPNAME_Y_MEASURED_POINT_OF_INTEREST),
      new EmptyIfNullFormat(new DecimalFormat("#####0.000")));
    builder.add(measuredYLabel, cc.xy(5, 7, "right, default"));
    JLabel fitYLabel = ComponentFactory.createLabel(model.getModel(
      Optimization.PROPNAME_Y_FIT_POINT_OF_INTEREST), new EmptyIfNullFormat(
      new DecimalFormat("#######0.000")));
    builder.add(fitYLabel, cc.xy(7, 7, "right, default"));

    builder.addSeparator("Progress", cc.xyw(1, 9, 9));

    builder.addLabel("State:", cc.xy(1, 11));
    JLabel stateValueLabel = ComponentFactory.createBooleanLabel(
      model.getModel(Optimization.PROPNAME_RUNNING),
      "Running", "Idle", Color.RED, Color.BLACK);
    builder.add(stateValueLabel, cc.xy(3, 11));

    builder.addLabel("Status:", cc.xy(1, 13));
    Pair<JProgressBar,ProgressBarMediator> pair =
      ComponentFactory.createProgressBar(
        model.getModel(Optimization.PROPNAME_PROGRESS),
        model.getModel(Optimization.PROPNAME_STATUS));
    statusProgressBarMediator = pair.getSecond();
    builder.add(pair.getFirst(), cc.xyw(3, 13, 7));

    return builder.getPanel();
  }

  protected void resetChartPanelZoom() {
    chartPanel.restoreAutoBounds();
  }
}
