package imcacat.imcabc;

import imcacat.epicsca4j.ca.EpicsCaException;

import java.awt.Font;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.UIManager;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYDataset;
import org.jfree.ui.RectangleInsets;

import com.jgoodies.binding.PresentationModel;
import com.jgoodies.binding.value.ValueHolder;
import com.jgoodies.binding.value.ValueModel;

/**
 * I am the presentation model for {@link OptimizationPanel}.
 */
public class OptimizationPresentationModel
extends PresentationModel<Optimization> {
  private static final long serialVersionUID = 1;

  protected ValueModel xyDataset;
  protected ValueModel chartTitle;
  protected ValueModel xAxisLabel;
  protected ValueModel yAxisLabel;
  protected ValueModel xPointOfInterest;
  protected XYPlot plot;
  protected JFreeChart chart;

  public OptimizationPresentationModel()
  throws EpicsCaException {
    this(new NullOptimization());
  }

  public OptimizationPresentationModel(Optimization bean) {
    this(new ValueHolder(bean, true));
  }

  public OptimizationPresentationModel(ValueModel beanChannel) {
    super(beanChannel);
    initializePlotAndChartModels();
    initializePlot();
    initializeChart();
    initializePlotAndChartModelChangeListeners();
  }

  protected void initializePlotAndChartModels() {
    xyDataset = getModel(Optimization.PROPNAME_XY_DATASET);
    chartTitle = getModel(Optimization.PROPNAME_XY_DATASET_TITLE);
    xAxisLabel = getModel(Optimization.PROPNAME_X_DATA_LABEL);
    yAxisLabel = getModel(Optimization.PROPNAME_Y_DATA_LABEL);
    xPointOfInterest = getModel(Optimization.PROPNAME_X_POINT_OF_INTEREST);
  }

  protected void initializePlotAndChartModelChangeListeners() {
    xyDataset.addValueChangeListener(new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent e) {
        xyDatasetChanged((XYDataset)e.getNewValue());
      }
    });
    chartTitle.addValueChangeListener(new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent e) {
        chartTitleChanged((String)e.getNewValue());
      }
    });
    xAxisLabel.addValueChangeListener(new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent e) {
        xAxisLabelChanged((String)e.getNewValue());
      }
    });
    yAxisLabel.addValueChangeListener(new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent e) {
        yAxisLabelChanged((String)e.getNewValue());
      }
    });
  }

  protected void initializePlot() {
    NumberAxis xAxis = new NumberAxis(getXAxisLabel());
    xAxis.setAutoRangeIncludesZero(false);
    NumberAxis yAxis = new NumberAxis(getYAxisLabel());
    yAxis.setAutoRangeIncludesZero(false);
    XYItemRenderer renderer = new XYLineAndShapeRenderer(true, false);
    plot = new XYPlot(getXYDataset(), xAxis, yAxis, renderer);
    plot.setOrientation(PlotOrientation.VERTICAL);
    plot.setDomainCrosshairVisible(true);
    plot.setDomainCrosshairLockedOnData(false);
  }

  protected void initializeChart() {
    chart = new JFreeChart(getChartTitle(), getChartTitleFont(), plot, true);
    // Hack. Setting the right inset to 15 pixels is required to work around
    // what appears to be a bug where if the x-axis tick label is being
    // displayed on the far right it gets truncated if it is long.
    chart.setPadding(new RectangleInsets(0, 0, 0, 15));
  }

  public JFreeChart getChart() {
    return chart;
  }

  protected Font getChartTitleFont() {
    // This is based on the code in com.jgoodies.forms.factories.
    // DefaultComponentFactory.TitleLabel.getTitleFont()
    return UIManager.getFont("TitledBorder.font");
  }

  protected XYDataset getXYDataset() {
    return (XYDataset)xyDataset.getValue();
  }

  protected String getChartTitle() {
    return (String)chartTitle.getValue();
  }

  protected String getXAxisLabel() {
    return (String)xAxisLabel.getValue();
  }

  protected String getYAxisLabel() {
    return (String)yAxisLabel.getValue();
  }

  protected void xyDatasetChanged(XYDataset newValue) {
    plot.setDataset(newValue);
    // Hack. I can't find a good way to clear the crosshair.
    plot.setDomainCrosshairValue(Double.NEGATIVE_INFINITY);
  }

  protected void chartTitleChanged(String newValue) {
    chart.setTitle(newValue);
  }

  protected void xAxisLabelChanged(String newValue) {
    plot.getDomainAxis().setLabel(newValue);
  }

  protected void yAxisLabelChanged(String newValue) {
    plot.getRangeAxis().setLabel(newValue);
  }

  public void updatePointOfInterest(Double x, Double y) {
    xPointOfInterest.setValue(x);
  }
}
