package imcacat.imcabc;

import java.beans.PropertyChangeEvent;

import com.jgoodies.binding.value.ValueModel;

/**
 * I run an {@link Optimization}. When I am performed, I create an instance of
 * my <code>Optimization</code> class and execute it. I update my state based
 * on whether the <code>Optimization</code> is running until it finishes.
 */
public class OptimizeAction extends TwoStateButtonAction {
  private static final long serialVersionUID = 1;

  protected Class<? extends Optimization> optimizationClass;
  protected Optimization optimization;
  protected ValueModel optimizationBeanChannel;

  public OptimizeAction(Class<? extends Optimization> optimizationClass,
      ValueModel optimizationBeanChannel) {
    this("Optimize", "Stop", optimizationClass, optimizationBeanChannel);
  }

  protected OptimizeAction(String state1Label, String state2Label,
      Class<? extends Optimization> optimizationClass,
      ValueModel optimizationBeanChannel) {
    this(state1Label, state2Label, Boolean.FALSE, Boolean.TRUE, true,
      optimizationClass, optimizationBeanChannel);
  }

  protected OptimizeAction(String state1Label, String state2Label,
      Object state1Value, Object state2Value,
      boolean shouldEnforceOnePressPerCommand,
      Class<? extends Optimization> optimizationClass,
      ValueModel optimizationBeanChannel) {
    super(state1Label, state2Label, state1Value, state2Value,
      shouldEnforceOnePressPerCommand);
    this.optimizationClass = optimizationClass;
    this.optimizationBeanChannel = optimizationBeanChannel;
    initialize();
  }

  @Override
  public void propertyChange(PropertyChangeEvent event) {
    if (Optimization.PROPNAME_RUNNING.equals(event.getPropertyName()))
      updateFromStateValue(event.getNewValue());
  }

  @Override
  public Object getStateValue() {
    return optimization == null ?
      Boolean.FALSE : Boolean.valueOf(optimization.isRunning());
  }

  @Override
  public void performState1Action() {
    super.performState1Action();
    if (optimization != null) optimization.release();
    optimization = createOptimizationInstance();
    optimization.addPropertyChangeListener(this);
    optimizationBeanChannel.setValue(optimization);
    optimization.execute();
  }

  @Override
  public void performState2Action() {
    super.performState2Action();
    optimization.cancel(true);
  }

  protected Optimization getOptimization() {
    return optimization;
  }

  protected Optimization createOptimizationInstance() {
    try {
      return optimizationClass.newInstance();
    } catch (InstantiationException e) {
      throw new RuntimeException("Failed to create Optimization instance", e);
    } catch (IllegalAccessException e) {
      throw new RuntimeException("Failed to create Optimization instance", e);
    }
  }
}
