package imcacat.imcabc;

import imcacat.epicsca4j.bean.OneWayPropertyChangeMediator;
import imcacat.epicsca4j.ca.EpicsCaException;
import imcacat.epicsca4j.model.EpicsPv;

/**
 * I am an <code>OptimizeFacade</code> for the 17-ID beamline.
 */
public class OptimizeFacade17id extends OptimizeFacade {
  private static final long serialVersionUID = 1;

  public static final String PROPNAME_SECOND_CRYSTAL_PITCH_POSITION =
    "secondCrystalPitchPosition";
  public static final String PROPNAME_SECOND_CRYSTAL_ROLL_POSITION =
    "secondCrystalRollPosition";

  protected EpicsPv secondCrystalPitchPositionPv;
  protected OneWayPropertyChangeMediator secondCrystalPitchPositionMediator;

  protected EpicsPv secondCrystalRollPositionPv;
  protected OneWayPropertyChangeMediator secondCrystalRollPositionMediator;

  public OptimizeFacade17id() throws InterruptedException, EpicsCaException {
    super();
  }

  protected void initializeManualSection()
  throws InterruptedException, EpicsCaException {
    secondCrystalPitchPositionPv = new EpicsPv(
      Resources.getDefault().getSecondCrystalPitchPositionPvName(),
      false, true);
    secondCrystalPitchPositionMediator = new OneWayPropertyChangeMediator(
      secondCrystalPitchPositionPv, this, EpicsPv.PROPNAME_VALUE,
      PROPNAME_SECOND_CRYSTAL_PITCH_POSITION);

    secondCrystalRollPositionPv = new EpicsPv(
      Resources.getDefault().getSecondCrystalRollPositionPvName(),
      false, true);
    secondCrystalRollPositionMediator = new OneWayPropertyChangeMediator(
      secondCrystalRollPositionPv, this, EpicsPv.PROPNAME_VALUE,
      PROPNAME_SECOND_CRYSTAL_ROLL_POSITION);
  }

  public double getSecondCrystalPitchPosition()
  throws InterruptedException, EpicsCaException {
    return (Double)secondCrystalPitchPositionPv.getValue();
  }

  public double getSecondCrystalRollPosition()
  throws InterruptedException, EpicsCaException {
    return (Double)secondCrystalRollPositionPv.getValue();
  }
}
