package imcacat.imcabc;

import java.text.DecimalFormat;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import imcacat.epicsca4j.ca.EpicsCaException;
import imcacat.epicsca4j.util.UiUtilities;

/**
 * I am an <code>OptimizePanel</code> for the 17-BM beamline.
 */
public class OptimizePanel17bm
extends OptimizePanel<OptimizePresentationModel17bm> {
  private static final long serialVersionUID = 1;

  public OptimizePanel17bm() throws InterruptedException, EpicsCaException {
    this(new OptimizePresentationModel17bm());
  }

  public OptimizePanel17bm(OptimizePresentationModel17bm model)
      throws InterruptedException, EpicsCaException {
    super(model);
  }

  protected JComponent buildLeftPanel() {
    FormLayout layout = new FormLayout(
      "right:max(45dlu;pref), 4dlu, right:max(40dlu;pref), 4dlu, " +
        "max(40dlu;pref)", // Columns
      "pref, 4dlu, pref, 2dlu, pref, 4dlu, " + // Automatic
      "pref, 4dlu, pref, 4dlu, pref" // Manual
    );
    layout.setRowGroups(new int[][] {
      {1, 7}, // separators
      {3, 5, 9, 11} // labels and fields
    });
    layout.setColumnGroups(new int[][] {{3, 5}});

    PanelBuilder builder = new PanelBuilder(layout);
    builder.border(Borders.DIALOG);
    CellConstraints cc = new CellConstraints();

    builder.addSeparator("Automatic", cc.xyw(1, 1, 5));

    builder.addLabel("Beam focus:", cc.xy(1, 3));
    builder.add(new JButton(
      new OpenBeamFocusAutoOptimizationDetailsDialogAction17bm(
        UiUtilities.getRootFrame(), "Details...", "Beam focus details")),
      cc.xy(3, 3, "fill, default"));
    builder.add(new JButton(model.getAutoOptimizeBeamFocusAction()),
      cc.xy(5, 3));

    builder.addLabel("All:", cc.xy(1, 5));
    builder.add(new JButton(
      new OpenAllAutoOptimizationDetailsDialogAction17bm(
        UiUtilities.getRootFrame(), "Details...", "All details")),
      cc.xy(3, 5, "fill, default"));
    builder.add(new JButton(model.getAutoOptimizeAllAction()), cc.xy(5, 5));

    builder.addSeparator("Manual", cc.xyw(1, 7, 5));

    builder.addLabel("Monochromator", cc.xyw(3, 9, 3, "right, default"));

    builder.addLabel("Omega (\u00B5rad):", cc.xy(1, 11));
    JLabel omegaValueLabel = ComponentFactory.createLabel(
      model.getModel(OptimizeFacade17bm.PROPNAME_OMEGA_POSITION),
      new DecimalFormat("0.0"));
    builder.add(omegaValueLabel, cc.xy(3, 11));
    Action optimizeOmegaAction = model.getOptimizeOmegaAction();
    builder.add(new JButton(optimizeOmegaAction), cc.xy(5, 11));

    return builder.getPanel();
  }
}
