package imcacat.imcabc;

import java.text.DecimalFormat;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import imcacat.epicsca4j.ca.EpicsCaException;

/**
 * I am an <code>OptimizePanel</code> for the 17-ID beamline.
 */
public class OptimizePanel17id
extends OptimizePanel<OptimizePresentationModel17id> {
  private static final long serialVersionUID = 1;

  public OptimizePanel17id() throws InterruptedException, EpicsCaException {
    this(new OptimizePresentationModel17id());
  }

  public OptimizePanel17id(OptimizePresentationModel17id model)
  throws InterruptedException, EpicsCaException {
    super(model);
  }

  protected JComponent buildLeftPanel() {
    FormLayout layout = new FormLayout(
      "right:max(45dlu;pref), 4dlu, right:max(40dlu;pref), 4dlu, " +
        "max(40dlu;pref)", // Columns
      "pref, 4dlu, pref, 4dlu, pref, 2dlu, pref" // Manual
    );
    layout.setRowGroups(new int[][] {
      {1}, // separators
      {3, 5, 7} // labels and fields
    });
    layout.setColumnGroups(new int[][] {{3, 5}});

    PanelBuilder builder = new PanelBuilder(layout);
    builder.border(Borders.DIALOG);
    CellConstraints cc = new CellConstraints();

    builder.addSeparator("Manual", cc.xyw(1, 1, 5));

    builder.addLabel("Monochromator", cc.xyw(3, 3, 3, "right, default"));

    builder.addLabel("Second Crystal Pitch (mrad):", cc.xy(1, 5));
    JLabel xtal2PitchValueLabel = ComponentFactory.createLabel(
      model.getModel(
        OptimizeFacade17id.PROPNAME_SECOND_CRYSTAL_PITCH_POSITION),
      new DecimalFormat("0.000"));
    builder.add(xtal2PitchValueLabel, cc.xy(3, 5));
    Action optimizeSecondCrystalPitchAction =
      model.getOptimizeSecondCrystalPitchAction();
    builder.add(new JButton(optimizeSecondCrystalPitchAction), cc.xy(5, 5));

    builder.addLabel("Second Crystal Roll (mrad):", cc.xy(1, 7));
    JLabel xtal2RollValueLabel = ComponentFactory.createLabel(
      model.getModel(
        OptimizeFacade17id.PROPNAME_SECOND_CRYSTAL_ROLL_POSITION),
      new DecimalFormat("0.000"));
    builder.add(xtal2RollValueLabel, cc.xy(3, 7));
    Action optimizeSecondCrystalRollAction =
      model.getOptimizeSecondCrystalRollAction();
    builder.add(new JButton(optimizeSecondCrystalRollAction), cc.xy(5, 7));

    return builder.getPanel();
  }
}
