package imcacat.imcabc;

import imcacat.epicsca4j.ca.EpicsCaException;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;

import com.jgoodies.binding.PresentationModel;
import com.jgoodies.binding.value.ValueHolder;
import com.jgoodies.binding.value.ValueModel;

/**
 * I am the presentation model for {@link OptimizePanel}.
 */
public class OptimizePresentationModel<B> extends PresentationModel<B> {
  private static final long serialVersionUID = 1;

  protected ValueModel optimizationBeanChannel;
  protected ImcabcPresentationModel imcabcPresentationModel;

  protected List<OptimizeAction> optimizeActions;

  public OptimizePresentationModel(OptimizeFacade bean,
      ValueModel optimizationBeanChannel,
      ImcabcPresentationModel imcabcPresentationModel)
  throws EpicsCaException {
    this(new ValueHolder(bean, true), optimizationBeanChannel,
      imcabcPresentationModel);
  }

  public OptimizePresentationModel(ValueModel beanChannel,
      ValueModel optimizationBeanChannel,
      ImcabcPresentationModel imcabcPresentationModel)
  throws EpicsCaException {
    super(beanChannel);
    this.optimizationBeanChannel = optimizationBeanChannel;
    this.imcabcPresentationModel = imcabcPresentationModel;

    imcabcPresentationModel.addPropertyChangeListener(
      ImcabcPresentationModel.PROPNAME_OPTIMIZATION_RUNNING,
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          optimizationIsRunningChanged((Boolean)event.getNewValue());
        }
      });

    imcabcPresentationModel.addPropertyChangeListener(
      ImcabcPresentationModel.PROPNAME_OPTIMIZATION_ENABLED,
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          isOptimizationEnabledChanged((Boolean)event.getNewValue());
        }
      });

    optimizeActions = new ArrayList<OptimizeAction>();
  }

  public ValueModel getOptimizationBeanChannel() {
    return optimizationBeanChannel;
  }

  protected void optimizationIsRunningChanged(Boolean newValue) {
    for (OptimizeAction each : optimizeActions)
      if (each.getOptimization() == null ||
          !each.getOptimization().isRunning())
        each.setEnabled(!newValue.booleanValue());
  }

  protected void isOptimizationEnabledChanged(Boolean newValue) {
    for (OptimizeAction each : optimizeActions)
        each.setEnabled(newValue.booleanValue());
  }
}
