package imcacat.imcabc;

import javax.swing.Action;

import imcacat.epicsca4j.ca.EpicsCaException;

import com.jgoodies.binding.value.ValueHolder;
import com.jgoodies.binding.value.ValueModel;

/**
 * I am an <code>OptimizePresentationModel</code> for the 17-BM beamline.
 */
public class OptimizePresentationModel17bm extends
    OptimizePresentationModel<OptimizeFacade17bm> {
  private static final long serialVersionUID = 1;

  protected OptimizeAction autoOptimizeBeamFocusAction;
  protected OptimizeAction autoOptimizeAllAction;
  protected OptimizeAction manualOptimizeOmegaAction;

  public OptimizePresentationModel17bm()
  throws InterruptedException, EpicsCaException {
    this(new OptimizeFacade17bm(), new ValueHolder(new NullOptimization(),
      true), new ImcabcPresentationModel17id());
  }

  public OptimizePresentationModel17bm(OptimizeFacade17bm bean,
      ValueModel optimizationBeanChannel,
      ImcabcPresentationModel imcabcPresentationModel)
  throws EpicsCaException {
    this(new ValueHolder(bean, true), optimizationBeanChannel,
      imcabcPresentationModel);
  }

  public OptimizePresentationModel17bm(ValueModel beanChannel,
      ValueModel optimizationBeanChannel)
  throws InterruptedException, EpicsCaException {
    this(beanChannel, optimizationBeanChannel,
      new ImcabcPresentationModel17bm());
  }

  public OptimizePresentationModel17bm(ValueModel beanChannel,
      ValueModel optimizationBeanChannel,
      ImcabcPresentationModel imcabcPresentationModel)
  throws EpicsCaException {
    super(beanChannel, optimizationBeanChannel, imcabcPresentationModel);

    autoOptimizeBeamFocusAction =
      new OptimizeAction(BeamFocusAutoOptimization17bm.class,
        optimizationBeanChannel);
    optimizeActions.add(autoOptimizeBeamFocusAction);

    autoOptimizeAllAction =
      new OptimizeAction(AllAutoOptimization17bm.class,
        optimizationBeanChannel);
    optimizeActions.add(autoOptimizeAllAction);

    manualOptimizeOmegaAction =
      new OptimizeAction(OmegaManualOptimization17bm.class,
        optimizationBeanChannel);
    optimizeActions.add(manualOptimizeOmegaAction);
  }

  public Action getAutoOptimizeBeamFocusAction() {
    return autoOptimizeBeamFocusAction;
  }

  public Action getAutoOptimizeAllAction() {
    return autoOptimizeAllAction;
  }

  public Action getOptimizeOmegaAction() {
    return manualOptimizeOmegaAction;
  }
}
