package imcacat.imcabc;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JProgressBar;

import com.jgoodies.binding.value.ValueModel;

/**
 * I mediate between two <code>ValueModel</code> instances and a
 * <code>JProgressBar</code>. I allow the <code>ValueModel</code> instances to
 * act as the model for the <code>JProgressBar</code> instance's progress
 * ({@link JProgressBar#setValue(int)}) and progress string ({@link
 * JProgressBar#setString(String)}).
 * <p>
 * All of my methods must only be called from the Event Dispatch Thread (EDT).
 *
 * @see com.jgoodies.binding.value.ValueModel
 */
public class ProgressBarMediator {
  protected JProgressBar progressBar;
  protected ValueModel progressModel;
  protected PropertyChangeListener progressListener;
  protected ValueModel progressStringModel;
  protected PropertyChangeListener progressStringListener;

  public ProgressBarMediator(JProgressBar progressBar,
      ValueModel progressModel) {
    this(progressBar, progressModel, null);
  }

  public ProgressBarMediator(JProgressBar progressBar,
      ValueModel progressModel, ValueModel progressStringModel) {
    super();
    this.progressBar = progressBar;
    this.progressModel = progressModel;
    this.progressStringModel = progressStringModel;

    if (progressModel != null) {
      progressListener = new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          progressChanged((Integer)event.getNewValue());
        }
      };
      progressModel.addValueChangeListener(progressListener);
    }

    if (progressStringModel != null) {
      progressStringListener = new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          progressStringChanged((String)event.getNewValue());
        }
      };
      progressStringModel.addValueChangeListener(progressStringListener);
    }

    updateProgressBar();
  }

  protected void progressChanged(Integer newValue) {
    if (newValue != null) progressBar.setValue(newValue.intValue());
  }

  protected void progressStringChanged(String newValue) {
    if (newValue != null) progressBar.setString(newValue);
  }

  public void updateProgressBar() {
    if (progressModel != null)
      progressChanged((Integer)progressModel.getValue());
    if (progressStringModel != null)
      progressStringChanged((String)progressStringModel.getValue());
  }

  public void release() {
    if (progressListener != null)
      progressModel.removeValueChangeListener(progressListener);
    if (progressStringListener != null)
      progressStringModel.removeValueChangeListener(progressStringListener);
  }
}
