package imcacat.imcabc;

import imcacat.epicsca4j.util.StringUtilities;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import com.jgoodies.binding.value.ValueHolder;
import com.jgoodies.binding.value.ValueModel;

/**
 * I ask the user to provide a file via a file chooser dialog and then ask
 * my {@link MadFacade} model to save the MAD experiment parameters to that
 * file.
 */
public class SaveMadExperimentParametersAction extends AbstractAction {
  private static final long serialVersionUID = 1;

  private static final Logger LOG = Logger.getLogger(
    SaveMadExperimentParametersAction.class);

  protected MadFacade model;
  protected ValueModel selectedFileHolder;
  protected ChooseFileAction chooseFileAction;
  protected Component parent;

  public SaveMadExperimentParametersAction(Component parent, String name,
      MadFacade model) {
    super();
    this.parent = parent;
    this.selectedFileHolder = new ValueHolder();
    chooseFileAction = new ChooseFileAction(parent, name + " choose file",
      "Save experiment paremeters", "Save", selectedFileHolder,
      new FileChooserFileFilter(new ExtensionFileFilter(
        getExperimentParametersFileNameExtension()), "Text files (*.txt)"));
    this.model = model;
    putValue(Action.NAME, name);
  }

  public void actionPerformed(ActionEvent event) {
    selectedFileHolder.setValue(null);
    chooseFileAction.setSelectedFile(model.getExperimentParametersFile());
    chooseFileAction.doit();
    if (selectedFileHolder.getValue() == null) return;
    try {
      model.writeExperimentParameters((File)selectedFileHolder.getValue());
    } catch (FileNotFoundException e) {
      LOG.debug("Failed to save experiment parameters", e);
      JOptionPane.showMessageDialog(parent, StringUtilities.wordWrap(
          "The experiment paremeters file could not be saved because the " +
          "file " + ((File)selectedFileHolder.getValue()).getPath() +
          " was not found.", 78, "\n"),
        ImcabcFrame.FRAME_TITLE, JOptionPane.ERROR_MESSAGE);
    } catch (IOException e) {
      LOG.debug("Failed to save experiment parameters", e);
      JOptionPane.showMessageDialog(parent, StringUtilities.wordWrap(
          "The experiment paremeters file could not be saved because an " +
          "I/O error occured while trying to write the file.", 78, "\n"),
          ImcabcFrame.FRAME_TITLE, JOptionPane.ERROR_MESSAGE);
    }
    selectedFileHolder.setValue(null);
  }

  protected static String getExperimentParametersFileNameExtension() {
    return
      MadFluorescenceScanDataProcessor.EXPERIMENT_PARAMETERS_FILE_EXTENSION;
  }
}
