package imcacat.imcabc;

import java.beans.PropertyChangeEvent;

import imcacat.epicsca4j.bean.OneWayPropertyChangeMediator;
import imcacat.epicsca4j.bean.ExternallyFireablePropertyChangeBean;
import imcacat.epicsca4j.ca.EpicsCaException;
import imcacat.epicsca4j.model.EpicsPv;

import com.jgoodies.binding.beans.Model;

/**
 * I am the model for the Staff tab.
 */
public class StaffFacade extends Model
implements ExternallyFireablePropertyChangeBean {
  private static final long serialVersionUID = 1;

  public static final String PROPNAME_OMEGA_POSITION = "omegaPosition";
  public static final String PROPNAME_CHI_POSITION = "chiPosition";
  public static final String PROPNAME_FOCUS_POSITION = "focusPosition";
  public static final String PROPNAME_MIRROR_VERTICAL_JACK_POSITION =
    "mirrorVerticalJackPosition";
  public static final String PROPNAME_MIRROR_BEND_POSITION =
    "mirrorBendPosition";

  protected EpicsPv omegaPositionPv;
  protected OneWayPropertyChangeMediator omegaMediator;
  protected EpicsPv chiPositionPv;
  protected OneWayPropertyChangeMediator chiMediator;
  protected EpicsPv focusPositionPv;
  protected OneWayPropertyChangeMediator focusMediator;
  protected EpicsPv mirrorVerticalJackPositionPv;
  protected OneWayPropertyChangeMediator mirrorVerticalJackMediator;
  protected EpicsPv mirrorBendPositionPv;
  protected OneWayPropertyChangeMediator mirrorBendMediator;

  public StaffFacade()
  throws InterruptedException, EpicsCaException {
    super();
    initialize();
  }

  protected void initialize()
  throws InterruptedException, EpicsCaException {
    initializeManualSection();
  }

  protected void initializeManualSection()
  throws InterruptedException, EpicsCaException {
    omegaPositionPv = new EpicsPv(
      Resources.getDefault().getOmegaPositionPvName(), false, true);
    omegaMediator = new OneWayPropertyChangeMediator(omegaPositionPv, this,
      EpicsPv.PROPNAME_VALUE,
      StaffFacade.PROPNAME_OMEGA_POSITION);

    chiPositionPv = new EpicsPv(
      Resources.getDefault().getChiPositionPvName(), false, true);
    chiMediator = new OneWayPropertyChangeMediator(chiPositionPv, this,
      EpicsPv.PROPNAME_VALUE,
      StaffFacade.PROPNAME_CHI_POSITION);

    focusPositionPv = new EpicsPv(
      Resources.getDefault().getFocusPositionPvName(), false, true);
    focusMediator = new OneWayPropertyChangeMediator(focusPositionPv, this,
      EpicsPv.PROPNAME_VALUE,
      StaffFacade.PROPNAME_FOCUS_POSITION);

    mirrorVerticalJackPositionPv = new EpicsPv(
      Resources.getDefault().getMirrorVerticalJackPositionPvName(),
      false, true);
    mirrorVerticalJackMediator = new OneWayPropertyChangeMediator(
      mirrorVerticalJackPositionPv, this,
      EpicsPv.PROPNAME_VALUE,
      StaffFacade.PROPNAME_MIRROR_VERTICAL_JACK_POSITION);

    mirrorBendPositionPv = new EpicsPv(
      Resources.getDefault().getMirrorBendPositionPvName(), false, true);
    mirrorBendMediator = new OneWayPropertyChangeMediator(
      mirrorBendPositionPv, this,
      EpicsPv.PROPNAME_VALUE,
      StaffFacade.PROPNAME_MIRROR_BEND_POSITION);
  }

  public Double getOmegaPosition()
  throws InterruptedException, EpicsCaException {
    return (Double)omegaPositionPv.getValue();
  }

  public Double getChiPosition()
  throws InterruptedException, EpicsCaException {
    return (Double)chiPositionPv.getValue();
  }

  public Double getFocusPosition()
  throws InterruptedException, EpicsCaException {
    return (Double)focusPositionPv.getValue();
  }

  public Double getMirrorVerticalJackPosition()
  throws InterruptedException, EpicsCaException {
    return (Double)mirrorVerticalJackPositionPv.getValue();
  }

  public Double getMirrorBendPosition()
  throws InterruptedException, EpicsCaException {
    return (Double)mirrorBendPositionPv.getValue();
  }

  public void externallyFirePropertyChange(PropertyChangeEvent event) {
    firePropertyChange(event);
  }
}
