package imcacat.imcabc;

import imcacat.epicsca4j.ca.EpicsCaException;
import imcacat.epicsca4j.util.UiUtilities;

import java.awt.BorderLayout;
import java.text.DecimalFormat;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

/**
 * I am the Staff tab view.
 */
public class StaffPanel extends JPanel {
  private static final long serialVersionUID = 1;

  protected StaffPresentationModel model;

  public StaffPanel()
  throws InterruptedException, EpicsCaException {
    this(new StaffPresentationModel());
  }

  public StaffPanel(StaffPresentationModel model)
  throws EpicsCaException {
    super();
    this.model = model;
    setLayout(new BorderLayout());
    add(buildMainPanel(), BorderLayout.CENTER);
  }

  protected JComponent buildMainPanel() throws EpicsCaException {
    FormLayout layout = new FormLayout(
      "pref, pref, fill:pref:grow",
      "fill:pref:grow");
    JPanel panel = new JPanel(layout);
    CellConstraints cc = new CellConstraints();
    panel.add(buildLeftPanel(), cc.xy(1, 1));
    panel.add(buildLeftRightSeparator(), cc.xy(2,1));
    panel.add(buildRightPanel(), cc.xy(3, 1));
    return panel;
  }

  protected JComponent buildLeftPanel() {
    FormLayout layout = new FormLayout(
      "right:max(58dlu;pref), 4dlu, right:max(40dlu;pref), 4dlu, " +
        "max(40dlu;pref)", // Columns
      "pref, 4dlu, pref, 2dlu, pref, 4dlu, pref, 4dlu, pref, 4dlu, " +
        // Automatic
      "pref, 4dlu, pref, 4dlu, pref, 2dlu, pref, 2dlu, pref, 4dlu, pref, " +
        "4dlu, pref, 2dlu, pref" // Manual
    );
    layout.setRowGroups(new int[][] {
      {1, 11}, // separators
      {7, 13, 21}, // sub separators
      {3, 5, 9, 15, 17, 19, 23, 25} // labels and fields
    });
    layout.setColumnGroups(new int[][] {{3, 5}});

    PanelBuilder builder = new PanelBuilder(layout);
    builder.border(Borders.DIALOG);
    CellConstraints cc = new CellConstraints();

    builder.addSeparator("Automatic optimization", cc.xyw(1, 1, 5));

    builder.addLabel("Horiz. beam focus:", cc.xy(1, 3));
    builder.add(new JButton(
      new OpenHorizontalBeamFocusAutoOptimizationDetailsDialogAction17bm(
        UiUtilities.getRootFrame(), "Details...",
        "Horizontal beam focus details")),
      cc.xy(3, 3, "fill, default"));
    builder.add(new JButton(model.getAutoOptimizeHorizontalBeamFocusAction()),
      cc.xy(5, 3));

    builder.addLabel("Vert. beam focus:", cc.xy(1, 5));
    builder.add(new JButton(
      new OpenVerticalBeamFocusAutoOptimizationDetailsDialogAction17bm(
        UiUtilities.getRootFrame(), "Details...",
        "Vertical beam focus details")),
      cc.xy(3, 5, "fill, default"));
    builder.add(new JButton(model.getAutoOptimizeVerticalBeamFocusAction()),
      cc.xy(5, 5));

    builder.addLabel("Advanced", cc.xyw(3, 7, 3, "right, default"));

    builder.addLabel("BPM:", cc.xy(1, 9));
    builder.add(new JButton(
      new OpenBpmAutoOptimizationDetailsDialogAction17bm(
        UiUtilities.getRootFrame(), "Details...",
        "BPM details")),
      cc.xy(3, 9, "fill, default"));
    builder.add(new JButton(model.getAutoOptimizeBpmAction()),
      cc.xy(5, 9));

    builder.addSeparator("Manual optimization", cc.xyw(1, 11, 5));

    builder.addLabel("Monochromator", cc.xyw(3, 13, 3, "right, default"));

    builder.addLabel("Omega (\u00B5rad):", cc.xy(1, 15));
    JLabel omegaValueLabel = ComponentFactory.createLabel(
      model.getModel(StaffFacade.PROPNAME_OMEGA_POSITION),
      new DecimalFormat("0.0"));
    builder.add(omegaValueLabel, cc.xy(3, 15));
    Action optimizeOmegaAction = model.getOptimizeOmegaAction();
    builder.add(new JButton(optimizeOmegaAction), cc.xy(5, 15));

    builder.addLabel("Chi (\u00B5rad):", cc.xy(1, 17));
    JLabel chiValueLabel = ComponentFactory.createLabel(
      model.getModel(StaffFacade.PROPNAME_CHI_POSITION),
      new DecimalFormat("0.00"));
    builder.add(chiValueLabel, cc.xy(3, 17));
    Action optimizeChiAction = model.getOptimizeChiAction();
    builder.add(new JButton(optimizeChiAction), cc.xy(5, 17));

    builder.addLabel("Focus (\u03BCm):", cc.xy(1, 19));
    JLabel focusValueLabel = ComponentFactory.createLabel(
      model.getModel(StaffFacade.PROPNAME_FOCUS_POSITION),
      new DecimalFormat("0.00"));
    builder.add(focusValueLabel, cc.xy(3, 19));
    Action optimizeFocusAction = model.getOptimizeFocusAction();
    builder.add(new JButton(optimizeFocusAction), cc.xy(5, 19));

    builder.addLabel("Vertical focusing mirror",
      cc.xyw(3, 21, 3, "right, default"));

    builder.addLabel("Vertical jack (\u03BCm):", cc.xy(1, 23));
    JLabel mirrorVerticalJackValueLabel = ComponentFactory.createLabel(
      model.getModel(StaffFacade.PROPNAME_MIRROR_VERTICAL_JACK_POSITION),
      new DecimalFormat("0.0"));
    builder.add(mirrorVerticalJackValueLabel, cc.xy(3, 23));
    Action optimizeMirrorVerticalJackAction =
      model.getOptimizeMirrorVerticalJackAction();
    builder.add(new JButton(optimizeMirrorVerticalJackAction), cc.xy(5, 23));

    builder.addLabel("Bend (\u03BCm):", cc.xy(1, 25));
    JLabel mirrorBendValueLabel = ComponentFactory.createLabel(
      model.getModel(StaffFacade.PROPNAME_MIRROR_BEND_POSITION),
      new DecimalFormat("0.0"));
    builder.add(mirrorBendValueLabel, cc.xy(3, 25));
    Action optimizeMirrorBendAction =
      model.getOptimizeMirrorBendAction();
    builder.add(new JButton(optimizeMirrorBendAction), cc.xy(5, 25));

    return builder.getPanel();
  }

  protected JComponent buildRightPanel() throws EpicsCaException {
    return new OptimizationPanel(new OptimizationPresentationModel(
      model.getOptimizationBeanChannel()));
  }

  protected JComponent buildLeftRightSeparator() {
    JPanel panel = new JPanel(new BorderLayout());
    panel.add(new JSeparator(SwingConstants.VERTICAL), BorderLayout.CENTER);
    panel.setBorder(new EmptyBorder(10, 0, 10, 0));
    return panel;
  }
}
