package imcacat.imcabc;

import imcacat.epicsca4j.ca.EpicsCaException;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.Action;

import com.jgoodies.binding.PresentationModel;
import com.jgoodies.binding.value.ValueHolder;
import com.jgoodies.binding.value.ValueModel;

/**
 * I am the presentation model for {@link StaffPanel}.
 */
public class StaffPresentationModel extends PresentationModel<StaffFacade> {
  private static final long serialVersionUID = 1;

  protected ValueModel optimizationBeanChannel;
  protected ImcabcPresentationModel imcabcPresentationModel;

  protected List<OptimizeAction> optimizeActions;
  protected OptimizeAction autoOptimizeHorizontalBeamFocusAction;
  protected OptimizeAction autoOptimizeVerticalBeamFocusAction;
  protected OptimizeAction autoOptimizeBpmAction;
  protected OptimizeAction manualOptimizeOmegaAction;
  protected OptimizeAction manualOptimizeChiAction;
  protected OptimizeAction manualOptimizeFocusAction;
  protected OptimizeAction manualOptimizeMirrorVerticalJackAction;
  protected OptimizeAction manualOptimizeMirrorBendAction;

  public StaffPresentationModel()
  throws InterruptedException, EpicsCaException {
    this(new StaffFacade());
  }

  public StaffPresentationModel(StaffFacade bean)
  throws InterruptedException, EpicsCaException {
    this(new ValueHolder(bean, true));
  }

  public StaffPresentationModel(StaffFacade bean,
      ValueModel optimizationBeanChannel)
  throws InterruptedException, EpicsCaException {
    this(new ValueHolder(bean, true), optimizationBeanChannel);
  }

  public StaffPresentationModel(StaffFacade bean,
      ValueModel optimizationBeanChannel,
      ImcabcPresentationModel imcabcPresentationModel)
  throws EpicsCaException {
    this(new ValueHolder(bean, true), optimizationBeanChannel,
      imcabcPresentationModel);
  }

  public StaffPresentationModel(ValueModel beanChannel)
  throws InterruptedException, EpicsCaException {
    this(beanChannel, new ValueHolder(new NullOptimization(), true));
  }

  public StaffPresentationModel(ValueModel beanChannel,
      ValueModel optimizationBeanChannel)
  throws InterruptedException, EpicsCaException {
    this(beanChannel, optimizationBeanChannel,
      new ImcabcPresentationModel17bm());
  }

  public StaffPresentationModel(ValueModel beanChannel,
      ValueModel optimizationBeanChannel,
      ImcabcPresentationModel imcabcPresentationModel)
  throws EpicsCaException {
    super(beanChannel);
    this.optimizationBeanChannel = optimizationBeanChannel;
    this.imcabcPresentationModel = imcabcPresentationModel;

    imcabcPresentationModel.addPropertyChangeListener(
      ImcabcPresentationModel.PROPNAME_OPTIMIZATION_RUNNING,
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          optimizationIsRunningChanged((Boolean)event.getNewValue());
        }
      });

    imcabcPresentationModel.addPropertyChangeListener(
      ImcabcPresentationModel.PROPNAME_OPTIMIZATION_ENABLED,
      new PropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
          isOptimizationEnabledChanged((Boolean)event.getNewValue());
        }
      });

    optimizeActions = new ArrayList<OptimizeAction>();

    autoOptimizeHorizontalBeamFocusAction =
      new OptimizeAction(HorizontalBeamFocusAutoOptimization17bm.class,
        optimizationBeanChannel);
    optimizeActions.add(autoOptimizeHorizontalBeamFocusAction);

    autoOptimizeVerticalBeamFocusAction =
      new OptimizeAction(VerticalBeamFocusAutoOptimization17bm.class,
        optimizationBeanChannel);
    optimizeActions.add(autoOptimizeVerticalBeamFocusAction);

    autoOptimizeBpmAction =
      new OptimizeAction(BpmAutoOptimization17bm.class,
        optimizationBeanChannel);
    optimizeActions.add(autoOptimizeBpmAction);

    manualOptimizeOmegaAction =
      new OptimizeAction(OmegaManualOptimization17bm.class,
        optimizationBeanChannel);
    optimizeActions.add(manualOptimizeOmegaAction);

    manualOptimizeChiAction =
      new OptimizeAction(ChiManualOptimization17bm.class,
        optimizationBeanChannel);
    optimizeActions.add(manualOptimizeChiAction);

    manualOptimizeFocusAction =
      new OptimizeAction(FocusManualOptimization17bm.class,
        optimizationBeanChannel);
    optimizeActions.add(manualOptimizeFocusAction);

    manualOptimizeMirrorVerticalJackAction =
      new OptimizeAction(MirrorVerticalJackManualOptimization17bm.class,
        optimizationBeanChannel);
    optimizeActions.add(manualOptimizeMirrorVerticalJackAction);

    manualOptimizeMirrorBendAction =
      new OptimizeAction(MirrorBendManualOptimization17bm.class,
        optimizationBeanChannel);
    optimizeActions.add(manualOptimizeMirrorBendAction);
  }

  public ValueModel getOptimizationBeanChannel() {
    return optimizationBeanChannel;
  }

  public Action getAutoOptimizeHorizontalBeamFocusAction() {
    return autoOptimizeHorizontalBeamFocusAction;
  }

  public Action getAutoOptimizeVerticalBeamFocusAction() {
    return autoOptimizeVerticalBeamFocusAction;
  }

  public Action getAutoOptimizeBpmAction() {
    return autoOptimizeBpmAction;
  }

  public Action getOptimizeOmegaAction() {
    return manualOptimizeOmegaAction;
  }

  public Action getOptimizeChiAction() {
    return manualOptimizeChiAction;
  }

  public Action getOptimizeFocusAction() {
    return manualOptimizeFocusAction;
  }

  public Action getOptimizeMirrorVerticalJackAction() {
    return manualOptimizeMirrorVerticalJackAction;
  }

  public Action getOptimizeMirrorBendAction() {
    return manualOptimizeMirrorBendAction;
  }

  protected void optimizationIsRunningChanged(Boolean newValue) {
    for (OptimizeAction each : optimizeActions)
      if (each.getOptimization() == null ||
          !each.getOptimization().isRunning())
        each.setEnabled(!newValue.booleanValue());
  }

  protected void isOptimizationEnabledChanged(Boolean newValue) {
    for (OptimizeAction each : optimizeActions)
        each.setEnabled(newValue.booleanValue());
  }
}
