package imcacat.imcabc;

import imcacat.epicsca4j.convert.ConversionStrategy;

import java.awt.Component;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JList;

/**
 * I am a Strategy for rendering a list cell. I render a list cell just like
 * <code>DefaultListCellRenderer</code> except instead of using the
 * <code>toString()</code> method of the item to be rendered to obtain the
 * text to display in the cell, I use my <code>ConversionStrategy</code> to
 * convert the item to a different value (usually a <code>String</code>)
 * before rendering it.
 */
public class StrategyListCellRenderer extends DefaultListCellRenderer {
  private static final long serialVersionUID = 1;

  protected ConversionStrategy converter;

  public StrategyListCellRenderer(ConversionStrategy converter) {
    super();
    this.converter = converter;
  }

  @Override
  public Component getListCellRendererComponent(JList list, Object value,
      int index, boolean isSelected, boolean cellHasFocus) {
    return super.getListCellRendererComponent(list, converter.convert(value),
      index, isSelected, cellHasFocus);
  }
}
