package imcacat.imcabc;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import com.jgoodies.binding.value.ValueModel;

/**
 * I am a two state button <code>Action</code> that uses {@link ValueModel}
 * objects to get and set my state.
 */
public class TwoStateValueModelButtonAction extends TwoStateButtonAction
implements PropertyChangeListener {
  private static final long serialVersionUID = 1;

  protected Object state1WriteValue;
  protected Object state2WriteValue;
  protected ValueModel readState;
  protected ValueModel writeState;

  public TwoStateValueModelButtonAction(String state1Label,
      String state2Label, Object state1WriteValue, Object state2WriteValue,
      Object state1ReadValue, Object state2ReadValue,
      ValueModel readAndWriteState) {
    this(state1Label, state2Label, state1WriteValue, state2WriteValue,
      state1ReadValue, state2ReadValue, readAndWriteState, false);
  }

  public TwoStateValueModelButtonAction(String state1Label,
      String state2Label, Object state1WriteValue, Object state2WriteValue,
      Object state1ReadValue, Object state2ReadValue,
      ValueModel readAndWriteState, boolean shouldEnforceOnePressPerState) {
    this(state1Label, state2Label, state1WriteValue, state2WriteValue,
      state1ReadValue, state2ReadValue, readAndWriteState, readAndWriteState,
      shouldEnforceOnePressPerState);
  }

  public TwoStateValueModelButtonAction(String state1Label,
      String state2Label, Object state1WriteValue, Object state2WriteValue,
      Object state1ReadValue, Object state2ReadValue, ValueModel readState,
      ValueModel writeState, boolean shouldEnforceOnePressPerState) {
    this(state1Label, state2Label, state1WriteValue, state2WriteValue,
      state1ReadValue, state2ReadValue, readState, writeState,
      shouldEnforceOnePressPerState, true);
  }

  protected TwoStateValueModelButtonAction(String state1Label,
      String state2Label, Object state1WriteValue, Object state2WriteValue,
      Object state1ReadValue, Object state2ReadValue, ValueModel readState,
      ValueModel writeState, boolean shouldEnforceOnePressPerState,
      boolean shouldInvokeInitialzeMethod) {
    super(state1Label, state2Label, state1ReadValue, state2ReadValue,
      shouldEnforceOnePressPerState);
    this.state1Label = state1Label;
    this.state2Label = state2Label;
    this.state1WriteValue = state1WriteValue;
    this.state2WriteValue = state2WriteValue;
    this.readState = readState;
    this.writeState = writeState;
    if (shouldInvokeInitialzeMethod) initialize();
  }

  @Override
  public void propertyChange(PropertyChangeEvent event) {
    if (ValueModel.PROPERTY_VALUE.equals(event.getPropertyName()))
      updateFromStateValue(event.getNewValue());
  }

  @Override
  public Object getStateValue() {
    return readState.getValue();
  }

  @Override
  public void performState1Action() {
    super.performState1Action();
    writeState.setValue(state1WriteValue);
  }

  @Override
  public void performState2Action() {
    super.performState2Action();
    writeState.setValue(state2WriteValue);
  }

  @Override
  public void registerStateValueListener() {
    readState.addValueChangeListener(this);
  }

  @Override
  public void unregisterStateValueListener() {
    readState.removeValueChangeListener(this);
  }
}
