package imcacat.imcabc.util;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * I provide utility methods related to Files.
 */
public class FileUtilities {
  protected static final Pattern FILE_EXTENSION_PATTERN =
    Pattern.compile("\\.[a-zA-Z]{0,4}$");

  private FileUtilities() {}

  /**
   * Creates a file name with the specified extension. If the file name
   * already has an extension, it will be removed before the new one is added.
   * <p>
   * An extension is defined as a sequence of zero to four alphabetic
   * characters at the end of the file name immediately preceded by a period
   * ('.') character.
   * <p>
   * Examples:
   * <pre>
   * createFileNameWithExtension("resume", "pdf") -> "resume.pdf"
   * createFileNameWithExtension("resume.ps", "pdf") -> "resume.pdf"</pre>
   *
   * @param fileName  File name from which to create a new file name with the
   *                  specified <tt>extension</tt>
   * @param extension  Extension to add to <tt>fileName</tt>; should not
   *                   include a leading '.'
   *
   * @return New file name with specified extension
   */
  public static String createFileNameWithExtension(String fileName,
      String extension) {
    Matcher matcher = FILE_EXTENSION_PATTERN.matcher(fileName);
    if (matcher.find())
      return fileName.substring(0, matcher.start()) + "." + extension;
    else
      return fileName + "." + extension;
  }
}
