package imcacat.imcabc.util;

import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.NoSuchElementException;

/**
 * I provide methods related to {@link List} objects.
 */
public class ListUtilities {
  private ListUtilities() {}

  public static <T extends Object & Comparable<? super T>> int indexOfMin(
      List<? extends T> aList) {
    if (aList.isEmpty()) throw new NoSuchElementException("List is empty");
    T smallest = aList.get(0);
    int smallestIndex = 0;
    for (ListIterator<? extends T> i = aList.listIterator(); i.hasNext();) {
      T each = i.next();
      if (smallest.compareTo(each) > 0) {
        smallest = each;
        smallestIndex = i.nextIndex() - 1;
      }
    }
    return smallestIndex;
  }

  public static <T extends Object & Comparable<? super T>> int indexOfMax(
      List<? extends T> aList) {
    if (aList.isEmpty()) throw new NoSuchElementException("List is empty");
    T biggest = aList.get(0);
    int biggestIndex = 0;
    for (ListIterator<? extends T> i = aList.listIterator(); i.hasNext();) {
      T each = i.next();
      if (biggest.compareTo(each) < 0) {
        biggest = each;
        biggestIndex = i.nextIndex() - 1;
      }
    }
    return biggestIndex;
  }

  public static double[][] toDouble2DArray(List<? extends Number> xValues,
      List<? extends Number> yValues) {
    if (xValues.size() != yValues.size())
      throw new IllegalArgumentException("xValues size != yValues size");
    double[][] values = new double[2][xValues.size()];
    Iterator<? extends Number> xValuesInterator = xValues.iterator();
    Iterator<? extends Number> yValuesInterator = yValues.iterator();
    for (int i = 0; i < values[0].length; i++) {
      values[0][i] = xValuesInterator.next().doubleValue();
      values[1][i] = yValuesInterator.next().doubleValue();
    }
    return values;
  }
}
